/*
 * Copyright (C) 2005, 2007-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "basetools.h"

#include <iostream>
#include <stdlib.h>
#include <cstring>
#include <inttypes.h>

#include "../config.h"
#include "../shared/platform.h"
#include "../shared/lib_conversions.h"
#include "../shared/hirestime.h"
#include "../shared/readjpeg.h"
#include "../shared/imgxform.h"
#include "../shared/utf8.h"
#include "../shared/asciidoc.h"

#ifdef USE_EPEG
# include <Epeg.h>
#endif

#ifdef USE_INTEGRATED_EPEG
# include "../lib_epeg/Epeg.h"
#endif

#ifdef USE_LIBGRAPHICSMAGICK
# include <magick/api.h>
#endif

#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"
#include "../shared/thread.h"
#include "workerdata.h"

#define CREATETHUMBSTEST 0

void* createThumbs(void* wd)
{
  intptr_t nRetCode = BaseTools::createThumbs((WorkerData*)wd);
  
  SimpleThread::Exit();  
  return((void*)nRetCode);
}

void* copyFiles(void* wd)
{
  intptr_t nRetCode = BaseTools::copyFiles((WorkerData*)wd);
    
  SimpleThread::Exit();
  return((void*)nRetCode);
}

void BaseTools::getShrinkSize(int w, int h, int& wmax, int& hmax)
{
  if (w > wmax || h > hmax)
  {
    // An der Dimension orientieren, die eher am Limit ist
    if ((float)w/wmax > (float)h/hmax)
      // Die Breite ist limitierend
      hmax = (int)(0.5 + wmax * ((float)h/w));
    else
      // Die Höhe ist limitierend
      wmax  = (int)(0.5 + hmax * ((float)w/h));
  }
  else
  {
    // no need to shrink
    wmax = w;
    hmax = h;
  }
}

bool BaseTools::SuffixIsMovie(std::string& strSuffix)
{
  return(strSuffix.compare(".flv") == 0 ||
         strSuffix.compare(".mov") == 0 ||
         strSuffix.compare(".avi") == 0 ||
         strSuffix.compare(".mpg") == 0 ||
         strSuffix.compare(".mpeg") == 0 ||
         strSuffix.compare(".mkv") == 0 ||
         strSuffix.compare(".mts") == 0 ||
         strSuffix.compare(".mp4") == 0);
}

int BaseTools::createThumb(std::string strDst, std::string strSrc,
                           float quality, int nSpeed, 
                           int wmax, int hmax)
{
  bool fDelSrc = false;
  bool fDone = false;

  std::cout << "thumb from " << strSrc << "\n";
  std::cout << "  " << nSpeed << " ";
  
  bool fJPEG, fMovie;
  {
    std::string suff = strL(FileTools::getSuffix(strSrc));
    fJPEG = suff.compare(".jpeg") == 0 || suff.compare(".jpg") == 0;
    fMovie = SuffixIsMovie(suff);
    
    // Don't even try to convert certain files. I rather check that here instead
    // of where the list of files to process is created in order to have a single
    // place to decide whether and how to treat files.
    if (suff.compare(".xml") == 0 ||
        suff.compare(".htm") == 0 ||
        suff.compare(".html") == 0 ||
        suff.compare(".orf") == 0 ||
        suff.compare(".xcf") == 0 ||
        suff.compare(".wav") == 0 ||
        suff.compare(".mp2") == 0 ||
        suff.compare(".mp3") == 0)
      return(0);
  }
  
  // Mit Fehler abbrechen wenn die Datei nicht vorhanden ist
  if (FileTools::fileExists(strSrc) == false)
    return -1;
      
  if (fMovie)
  {
    // todo: quick hack ...
    std::string strTmpDir = FileTools::getHomeDir("Fotoalbum");
    std::string strCmd;

    // Meiner Erfahrung nach ist ffmpeg vorzuziehen, weil mplayer aus einigen Videos
    // zwar Bilder erzeugt hat, diese aber nicht vollständig waren.
    strCmd = "ffmpeg -ss 2.0 -i \"" + strSrc + "\" -vframes 1 \"" + strTmpDir + "00000001.jpg\"";
    strSrc = strTmpDir + "00000001.jpg";
    Platform::system(strCmd);
    
    if (FileTools::fileExists(strSrc) == false)
    {
      strCmd = "mplayer -nosound -ss 3 -frames 1 -vo jpeg:outdir=\"" + strTmpDir + "\" \"" + strSrc + "\"";
      Platform::system(strCmd);
      strSrc = strTmpDir + "00000001.jpg";
    }
    
    if (FileTools::fileExists(strSrc) == false)
      return -1;
    
    fJPEG   = true;
    fDelSrc = true;
  }
  
#if (USE_EPEG == 1 || USE_INTEGRATED_EPEG == 1)
  // If the source file is a jpeg, I can use epeg to speed things up.
  if (nSpeed == 2 && fJPEG)
  {
    std::cout << "epeg ";
    
    Epeg_Image *im;
    int w, h;
    
    im = epeg_file_open(strSrc.c_str());
    if (!im)
    {
      std::cerr << "Cannot open " << strSrc << "\n";
      return(-1);
    }
    
    epeg_size_get(im, &w, &h);
    // std::cout << "epeg: src is " << w << "x" << h;
      
    if (w > wmax || h > hmax)
    {
      double fw = ((double)(w))/wmax;
      double fh = ((double)(h))/hmax;
      
      if (fw > fh)
      {
        h = (h*wmax)/w;
        w = wmax;
      }
      else
      {
        w = (w*hmax)/h;
        h = hmax;
      }
      
      // std::cout << ", strDst is " << w << "x" << h << ", qual=" << quality << "\n";
      epeg_decode_size_set(im, w, h);
      epeg_quality_set(im, (int)(quality*100));
      epeg_thumbnail_comments_enable(im, 0);
      epeg_file_output_set(im, strDst.c_str());
      epeg_encode(im);        
      epeg_close(im);
      fDone = true;
    }
    else
    {
      epeg_close(im);
      // in this case I can just copy the input file without recompressing:
      std::cout << "copy ";
      if (FileTools::copy(strDst, strSrc) == 0)
        fDone = true;
    }
  }
#endif
  
  if (!fDone && nSpeed < 2 && fJPEG)
  {
    std::cout << "internal ";

    // 
    int nW = wmax;
    int nH = hmax;
    int nRatio;
    unsigned char* array = LoadScaledJPEG(strSrc.c_str(), nW, nH, nRatio);
    if (array)
    {
      if (nRatio == 1 && nW <= wmax && nH <= hmax)
      {
        // in this case I can just copy the input file without recompressing:
        std::cout << "copy ";
        if (FileTools::copy(strDst, strSrc) == 0)
          fDone = true;
        delete[] array;
      }
      else
      {
        int nD = 3;
        
        getShrinkSize(nW, nH, wmax, hmax);
        if (nW >= wmax*2 && nH >= hmax*2)
        {
          float flDiv  = (1.0*nW)/wmax;
          float flDivH = (1.0*nH)/hmax;
          if (flDiv > flDivH)
            flDiv = flDivH;
          
          unsigned char* tmp = array;
          array = ImgXForm::Shrink(tmp, nW, nH, nD, floor(flDiv));
          delete[] tmp;
        }
          
        unsigned char* array_scaled = ImgXForm::Bilinear(array, nW, nH, nD, wmax, hmax);
        delete[] array;
          
        if (WriteJPEG(strDst.c_str(), array_scaled, wmax, hmax, nD, quality*100) == 0)
          fDone = true;
        
        delete[] array_scaled;
      }
    }
  }
  
  if (!fDone)
  {
#ifdef USE_LIBGRAPHICSMAGICK
    std::cout << "libgraphicsmagick ";
    
    ExceptionInfo exception;
    Image* image;
    Image* resize_image;
    ImageInfo* image_info;
    
    GetExceptionInfo(&exception);
    image_info=CloneImageInfo((ImageInfo *) NULL);
    
    strcpy(image_info->filename, strSrc.c_str());    
    image=ReadImage(image_info, &exception);
    CatchException(&exception);
    if (image != 0)
    {
      getShrinkSize(image->columns, image->rows, wmax, hmax);
      // siehe http://www.graphicsmagick.org/api/resize.html
      //resize_image=ResizeImage(image, wmax, hmax, LanczosFilter, 1.0, &exception); // slow
      if (nSpeed > 1)
        resize_image = SampleImage(image, wmax, hmax, &exception); // fast, nearest neighbor
      else        
        resize_image = ScaleImage(image, wmax, hmax, &exception); // medium
    
      CatchException(&exception);
      if (resize_image)
      {
        strcpy(resize_image->filename, strDst.c_str());
        image_info->quality = (int)(quality*100 + 0.5);
        if (WriteImage(image_info, resize_image) == 0)
        {
          CatchException(&(resize_image->exception));
          std::cerr << "error1\n";
          return(-1);
        }
      }
      else
      {
        std::cerr << "error2\n";
        return(-1);
      }
      
      DestroyImage(image);
      DestroyImage(resize_image);
      DestroyImageInfo(image_info);
      DestroyExceptionInfo(&exception);
      
      fDone = true;
    }
#endif
  }
    
  if (!fDone)
  {
    std::cout << "convert ";
    
    std::string sizestring = itoStr(wmax, ' ', 1) + "x" + itoStr(hmax, ' ', 1);
    std::string qualstring = ftoStr(quality, 1, 2);
    
    // jwtodo: Option, um den Algorithmus zum resize zwischen schnell und hochqualitativ
    // umzuschalten. Möglicherweise geht das mit '-filter'.
    //
    // Anstelle von '-thumbnail' kann man auch '-resize' benutzen, aber das braucht hier
    // doppelt so lange...
    std::string call = "";
#ifndef WIN32
    call = "nice -19 ";
#endif
    call += "convert \""
      + strSrc
      + "\" -flatten -thumbnail " + sizestring
      + " -quality " + qualstring
      + " \"" + strDst + "\"";
    
    if (system(call.c_str()))
    {
      std::cerr << call << "\n";
      return(-1);
    }
  }
  
  if (fDelSrc)
  {
    FileTools::rm(strSrc);
  }
  
  std::cout << "\n";
  
  return(0);
}

int BaseTools::createThumbs(WorkerData* wd)
{
  int               nRetCode = 0;
  int               nRetCInt = 0;
  WorkThumbCreate*  work     = (WorkThumbCreate*)((WorkThumbCreate*)wd->work);
  unsigned int      uSize    = work->filelist.size();

  HiResTime hrt;
  hrt.Capture();
  
  std::cout << "--- Creating thumbnails ------------------------- \n";
  
  for (unsigned int uCnt=0; (uCnt<uSize) && (wd->nStop == 0) && (nRetCode > -100); uCnt+= 2)
  {
    nRetCInt = FileTools::makeSureDirExists(work->filelist[uCnt+1].substr(0, work->filelist[uCnt+1].rfind('/')));
    switch (nRetCInt)
    {
      case 0:
        break;
      case -200:
        // These errors are considered to be fatal so that no other thumbnail will be processed.
        std::cerr << "ro fs\n";
        nRetCode = -100;
        break;
      default:
        nRetCode = -1;
        break;
    }
    
    if (nRetCode > -100)
    {
      nRetCInt = createThumb(work->filelist[uCnt+1], work->filelist[uCnt], 
                             work->flQual, work->nType,
                             work->nMaxW, work->nMaxH);
      switch (nRetCInt)
      {
        case 0:
          break;
        case -200:
          // These errors are considered to be fatal so that no other thumbnail will be processed.
          nRetCode = -100;
          break;
        default:
          nRetCode = -1;
          break;
      }
    }
    
    if (uCnt == 0 && nRetCode < 0)
    {
      // check if filesystem is writeable
      int nFNum = 0;
      while (FileTools::fileExists(work->filelist[uCnt+1] + itoStr(nFNum, '0', 1)))
        nFNum++;
      std::string strTestFName = work->filelist[uCnt+1] + itoStr(nFNum, '0', 1);
      
      std::cout << "Test if filesystem is writeable: " << strTestFName << "\n";
      
      std::ofstream of;
      of.open(strTestFName.c_str());
      of << "fotoalbum: test if filesystem is writeable\n";
      of.close();
      if (FileTools::fileExists(strTestFName))
        FileTools::rm(strTestFName);
      else
        nRetCode = -100;        
    }
    
    //
    wd->uNumUnitsDone++;
  }

  std::cout << "timing.thumbnails=" << hrt.GetMSecSinceCapture() << "\n";
  
  ((WorkerData*)wd)->nRun     = 0;
  ((WorkerData*)wd)->nRetCode = nRetCode;
  
  return(nRetCode);
}

int BaseTools::copyFiles(WorkerData* wd)
{
  int                  nRetCode = 0;
  WorkThumbCreate*     work     = (WorkThumbCreate*)((WorkThumbCreate*)wd->work);
  unsigned int         uSize    = work->filelist.size();
  
  std::cout << "--- Copying files ------------------------------- \n";
  
  for (unsigned int uCnt=0; (uCnt<uSize) && (wd->nStop == 0); uCnt+= 2)
  {
    if (FileTools::makeSureDirExists(work->filelist[uCnt+1].substr(0, work->filelist[uCnt+1].rfind('/'))))
      nRetCode = -1;
    
    std::cout << "copying " << work->filelist[uCnt] << " to " << work->filelist[uCnt+1] << "\n";
    
#if CREATETHUMBSTEST == 0
    if (FileTools::copy(work->filelist[uCnt+1], work->filelist[uCnt]))
      nRetCode = -1;
#endif
    
    //
    wd->uNumUnitsDone++;
  }
  
  ((WorkerData*)wd)->nRun     = 0;
  ((WorkerData*)wd)->nRetCode = nRetCode;
  
  return(nRetCode);
}

std::string BaseTools::getFilePath(std::string relname)
{
  const char* installpaths[] = { "../",                     // win32
                                 STR_INSTALL_PREFIX "/",    // unix
                                 "/usr/local/"         ,    // unix, fallback
                                 "/opt/"               ,    // unix, fallback
                                 "/usr/"                };  // unix, fallback
  
  std::string fpath = FileTools::getHomeDir("Fotoalbum") + relname;
  if (FileTools::fileExists(fpath))
    return(fpath);
  
  for (unsigned int n=0; n<sizeof(installpaths)/sizeof(const char*); n++)
  {
    fpath = std::string(installpaths[n]) + relname;
    std::cout << "trying " << fpath << "\n";
    if (FileTools::fileExists(fpath))
      return(fpath);
  }
 
  return("");
}

std::string BaseTools::GetAsHTML(std::string strc)
{
  if (strc.find("<a href") != std::string::npos ||
      strc.find("<table") != std::string::npos ||
      strc.find("<ul>") != std::string::npos)
    return(strc);
  else
  {
    // asciidoc is assumed
    return(AsciiDoc::ToHtml(strc));
  }
}
