/*
 * Copyright (C) 2011-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef BLOCKINGTASK_H
#define BLOCKINGTASK_H

#include "../config.h"
#include <inttypes.h>

#include <string>
#include <list>

class BlockingTask
{
public:
  
  /**
   * returns zero on success
   */
  virtual int Exec() = 0;
  virtual ~BlockingTask() {};
};

class BlockingTask_System : public BlockingTask
{
public:  
  BlockingTask_System(std::string strCmd);
  virtual ~BlockingTask_System() {};
  virtual int Exec();
  
private:
  std::string strCmd;
};

class BlockingTask_Rotate : public BlockingTask
{
public:
  BlockingTask_Rotate(std::string strPath, int nDegrees);
  virtual ~BlockingTask_Rotate() {};
  virtual int Exec();
  
private:
  std::string strPath;
  int nDegrees;
};

class BlockTaskThread
{
public:
  BlockTaskThread();
  int nDone;
  int nSize;
  intptr_t nErrCnt;
  bool fPause;
  bool fPauseUser;
  bool fCancel;
  std::list<BlockingTask*> tasks;
};

class BlockTaskManager
{
public:
  void Add(BlockingTask* pTask);
  
  /**
   * Initially this was called "Status", but FLTK1.3 includes some header (from X11, I guess)
   * which defines a name "Status". That in turn makes the compiler dislike my 
   * method name "Status" here...
   */
  bool FooStatus(int& nDone, int& nSize);
  
  /**
   * -1: everything done
   *  0: unchanged
   *  1: changed from idle to active
   */
  int Step();
  
  void Pause(bool fPause);
  void PauseUser(bool fPause);
  void Cancel() { threadData.fCancel = true; };
private:
  std::list<BlockingTask*> tasks;
  BlockTaskThread threadData;
};

#endif
