/*
 * Copyright (C) 2005, 2006, 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "datetime.h"
#include "../shared/lib_conversions.h"
#include "../shared/calendar.h"

#include <stdlib.h>
#include <string.h>
#include <iostream>

std::string JWWDateTime::getTimeAsString()
{
  std::string tmp;
  
  if (nTime != 0)
  {
    tmp.append(itoStr((nTime >> 12) & 0x1F, '0', 2));
    tmp.append(":");
    
    tmp.append(itoStr((nTime >> 6) & 0x3F, '0', 2));
    tmp.append(":");
    
    tmp.append(itoStr(nTime & 0x3F, '0', 2));
  }
      
  return(tmp);
}
      
std::string JWWDateTime::getSomeDateAsString(int someDate)
{
  std::string tmp;

  if (someDate != 0)
  {
    tmp.append(itoStr((someDate >> 9) & 0x0FFF, '0', 4));
    tmp.append("-");
    
    tmp.append(itoStr((someDate >> 5) & 0xF, '0', 2));
    tmp.append("-");
    
    tmp.append(itoStr(someDate & 0x1F, '0', 2));
  }
      
  return(tmp);
}

std::string JWWDateTime::getDateAsString()
{
  return(getSomeDateAsString(nDate));
}

int JWWDateTime::assembleDate(int y,
                              int m,
                              int d)
{
  return((1 << 21)
         + (y << 9)
         + (m << 5)
         + d);
}

int JWWDateTime::assembleTime(int h,
                              int m,
                              int s)
{
  return((1 << 17)
         + (h << 12)
         + (m << 6)
         + s);
}

int JWWDateTime::setDate(const char* nD)
{
  int          retVal = 0;
  int          y, m, d;
  std::string  tmp;
      
  if (nD == (const char*)0 || strlen(nD) == 0)
  {
    nDate = 0;
    return(0);
  }
      
//  std::cout << "Lese Datum aus dem String >" << nD << "<\n";
      
  if (strlen(nD) != 10)
    return(-1);      
      
  tmp = nD;
  
  y = atoi(tmp.substr(0, 4).c_str());
  m = atoi(tmp.substr(5, 7).c_str());
  d = atoi(tmp.substr(8, 10).c_str());
  
  if (y < 0 || y > 9999 || m < 1 ||  m > 12 || d < 1 || d > 31)
    retVal = -1;
  else
    nDate = assembleDate(y, m, d);
  
  return(retVal);  
}

int JWWDateTime::setTime(const char* nT)
{
  int          retVal = 0;
  int          h, m, s;
  std::string  tmp;
      
  if (nT == (const char*)0 || strlen(nT) == 0)
  {
    nTime = 0;
    return(0);
  }
            
//  std::cout << "Lese Zeit aus dem String >" << nT << "<\n";
      
  if (strlen(nT) != 8)
    return(-1);      
      
  tmp = nT;
      
  h = atoi(tmp.substr(0, 2).c_str());
  m = atoi(tmp.substr(3, 5).c_str());
  s = atoi(tmp.substr(6, 8).c_str());
  
  if (h < 0 || h > 23 || m < 0 || m > 59 || s < 0 || s > 59)
    retVal = -1;
  else
    nTime = assembleTime(h, m, s);
      
  return(retVal);
}

int JWWDateTime::compare(JWWDateTime* ref)
{
  int nC = 0;

  if (isDateSet())
    nC |= 1;
  if (ref->isDateSet())
    nC |= 2;

  switch (nC)
  {
   case 0:
    return(0);
   case 1:
    return(1);
   case 2:
    return(-1);
   default:
    {
      if (nDate > ref->nDate)
        return(1);
      else if (nDate < ref->nDate)
        return(-1);
      else
      {
        nC = 0;
        
        if (isTimeSet())
          nC |= 1;
        if (ref->isTimeSet())
          nC |= 2;

        switch (nC)
        {
         case 0:
          return(0);
         case 1:
          return(1);
         case 2:
          return(-1);
         default:
          if (nTime > ref->nTime)
            return(1);
          else if (nTime < ref->nTime)
            return(-1);
          else
            return(0);
        }
      }
    } 
  }
}

std::string JWWDateTime::getDateTimeAsString()
{
  std::string tmp;
  
  tmp = getDateAsString();
  if (isDateSet())
    tmp += " ";
  tmp += getTimeAsString();
  
  return(tmp);
}

unsigned int JWWDateTime::getYear()
{
  if (isDateSet())
    return( (nDate>>9) & 0x0FFF );
  else
    return(0);
}

void JWWDateTime::shift(int nMin)
{
  shiftSec(nMin*60);
}

void JWWDateTime::shiftSec(int nSec)
{
  if (isTimeSet())
  {
    int h, m, s;
    
    h = (nTime >> 12) & 0x1F;
    m = (nTime >>  6) & 0x3F;
    s =  nTime        & 0x3F;
 
    s += nSec;
    while (s < 0)
    {
      m--;
      s += 60;
    }
    while (s >= 60)
    {
      m++;
      s -= 60;
    }
    
    while (m < 0)
    {
      h--;
      m += 60;
    }
    while (m >= 60)
    {
      h++;
      m -= 60;
    }
    
    while (h < 0)
    {
      shiftDate(-1);
      h += 24;
    } 
    while (h >= 24)
    {
      shiftDate(+1);
      h -= 24;
    }
    
    nTime = assembleTime(h, m, s);  
  }
}

void JWWDateTime::shiftDate(int days)
{
  if (isDateSet())
  {
    int d, m, y;
    int num = getNumberOfDay(getDay(), getMonth(), getYear()) + days;
    getDayByNumber(num, d, m, y);
    nDate = assembleDate(y, m, d);
  }
}
