/*
 * Copyright (C) 2005, 2006, 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef DATETIME_H
# define DATETIME_H

# include <string>

/**
 * Some parts of the code assume that there only is a time if there's date.
 */
class JWWDateTime
{
public:
  /**
   * Bit 0..4     Tag (0..31)
   * Bit 5..8     Monat (0..15)
   * Bit 9..20    Jahr  (0..4095)
   * Bit 21       1, wenn Datum gesetzt ist
   * 
   * Ob das Datum gesetzt ist oder nicht, wird 
   * aber an (date == 0) erkannt!
   */
  int nDate;
  
  /** 
   * Bit 0..5      Sekunde (0..63)
   * Bit 6..11     Minute (0..63)
   * Bit 12..16    Stunde (0..31)
   * Bit 17        1, wenn Zeit gesetzt ist
   * 
   * Ob die Zeit gesetzt ist oder nicht, wird 
   * aber an (time == 0) erkannt!
   */
  int nTime;
  
  inline int getDay()   { return(nDate & 0x01F); };
  inline int getMonth() { return((nDate >> 5) & 0xF); };
  
  inline int getHour()   { return((nTime>>12) & 0x1F); };
  inline int getMinute() { return((nTime>> 6) & 0x3F); };
  unsigned int getYear();
  
  inline bool isDateSet() { return((nDate != 0)); };
  
  inline bool isTimeSet() { return((nTime != 0)); };
  
  /**
   * Liefert die Zeit im Format HH:MM:SS
   */
  std::string getTimeAsString();
  
  /**
   * Liefert ein Datum im Format YYYY-MM-DD
   */
  static std::string getSomeDateAsString(int someDate);
  
  /**
   * Liefert das Datum in der Darstellung
   * wie es auch getSomeDateAsString(...) tut.
   */
  std::string getDateAsString();
  
  /**
   * 
   */
  std::string getDateTimeAsString();   
  
  /**
   * 
   */
  static int assembleDate(int y,
                          int m,
                          int d);
  
  /**
   * 
   */
  static int assembleTime(int h,
                          int m,
                          int s);
  
  /**
   * Setzt das Datum aus einem String der Form YYYY-MM-DD
   */
  int setDate(const char* nD);
  
  /**
   * Setzt die Zeit aus einem String der Form HH:MM:SS
   */
  int setTime(const char* nT);
  
  /**
   * 
   */
  int compare(JWWDateTime* ref);
    
  /**
   * Shift time (todo: shift date, too) by nMin minutes.    
   */
  void shift(int nMin);
  
  /**
   * Shift time (todo: shift date, too) by nSec seconds.
   */
  void shiftSec(int nSec);
  
  /** 
   * Shift date (todo: get it right) by nDays days.
   */
  void shiftDate(int days);
};

#endif
