/*
 * Copyright (C) 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef EXIF_H
# define EXIF_H

# include <string>
# include <set>

# include "../config.h"

# ifdef USE_FLEXIF   
#  include "../fl_exif/Fl_EXIF_Data.H"
# endif

# ifdef USE_LIBEXIF
#  include <libexif/exif-data.h>
# endif

# ifdef EXIV2_FOUND
#  include <exiv2/image.hpp>
#  include <exiv2/exif.hpp>  
# endif

/**
 * This class delivers EXIF info found in a JPEG-file. It hides different 
 * underlying implementations.
 * It has been extended to handle IPTC and stuff like that, too.
 */
class ExifInfo
{
public:
  ExifInfo(std::string filename);
  ~ExifInfo();
  
  /**
   * Is there anything available?
   */
  bool hasEXIF();
  
  /**
   * 
   */
  std::string getInfo();
  
  /**
   * Returns an empty string if there is no date/time available. Otherwise,
   * it does return YYYY-MM-DD HH:MM:SS
   */
  std::string getDateTime();
  
  std::string Get_Make();
  std::string Get_Model();
  int Get_ISO();
  int Get_ExposureTime_us(); ///< returns exposure time in us
  int Get_FNumber_m10();     ///< returns F number multiplied by ten
  int Get_focalLen();        ///< returns focal length in mm
  
  void ReadKeywords(std::set<std::string>& kwset);
  
# ifdef EXIV2_FOUND
  static void ReadKeywords(Exiv2::Image::AutoPtr& image, std::set<std::string>& kwset);  
# endif
  
  /**
   * merges 'old' and comments found in the file
   */
  std::string ReadComment(std::string old);
  
private:
  
# ifdef EXIV2_FOUND
  Exiv2::Image::AutoPtr image;
  std::string get(const Exiv2::ExifData& exifData,
                  const std::string& key, bool fLabel = true);
  std::string get(const std::string& key);
# endif
  
# ifdef USE_FLEXIF   
  Fl_EXIF_Data* exif;
  std::string getExifString(Fl_EXIF_Data* exif, 
                            unsigned      tag);
  
# endif
  
# ifdef USE_LIBEXIF
  ExifData*   ed;
  std::string getExifString(ExifData*   ed,
                            ExifTag     tag);
# endif
  
};

#endif
