// -*- mode: c; mode: fold -*-
//
/*
 * Copyright (C) 2006-2008, 2010-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "export.h"

#include <iostream>
#include "group.h"
#include "pic.h"
#include "treeview.h"
#include "strings.h"
#include "exif.h"
#include "basetools.h"
#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"
#include "../../version.h"
#include "../shared/filestream.h"

#ifdef WIN32_FLTK13
# include <FL/fl_utf8.h>
#endif

#if (EXIV2_FOUND == 1)
# include <exiv2/image.hpp>
# include <exiv2/exif.hpp>
# include <exiv2/iptc.hpp>
#endif

void Export::setOpts(SimpleXMLTransfer* exopts)
{
  SimpleXMLTransfer* xExp = exopts->getChild("export");
  
  type        = (eType)(xExp->attributeAsInt("type"));
  structure   = (eStructure)(xExp->attributeAsInt("structure"));
  fRecursive  = (xExp->attributeAsInt("recurse")     != 0);
  fThumbs     = (xExp->attributeAsInt("thumbs")      != 0);
  fDatafile   = (xExp->attributeAsInt("datafile")    != 0);
  fAddToJpeg  = (xExp->attributeAsInt("addtojpeg")   != 0);
  fPrefix     = (xExp->attributeAsInt("prefix")      != 0);
  fHtml       = (xExp->attributeAsInt("html")        != 0);
  uThumbSize  =  xExp->attributeAsInt("thumbsize", 200);
  uThumbQual  =  xExp->attributeAsDouble("thumbqual", 0.75)*100;
  uShrinkMode =  xExp->attributeAsInt("fast", 0);
  {
    if (type == eCopy)
    {
      // in order to have a unique name for html export:
      uShrinkQual = uShrinkMode = uShrinkW = uShrinkH = 0;
    }
    else
    {
      SimpleXMLTransfer* xShrink = exopts->getChild("shrink");
      uShrinkQual =  xShrink->attributeAsDouble("quality", 0.75)*100;
      xShrink = xShrink->getChildAt(xShrink->attributeAsInt("index"));
      uShrinkW = xShrink->attributeAsInt("w");
      uShrinkH = xShrink->attributeAsInt("h");
    }
  }
  
  nPicsInARow = 760 / (uThumbSize+3);
  
  // --- Constraints ------------------------------------------------
  if (fHtml)
  {
    structure = eHTML;
    fThumbs   = true;
  }
  if (structure != eView)
    fPrefix = false;
}

std::string Export::getUniqueName(SetOfStrings&  namesInDir,
                                  std::string    name)
{
  if (namesInDir.find(name) == namesInDir.end())
  {
    namesInDir.insert(name);
    return(name);
  }
  else
  {
    std::string  tmp;
    unsigned int cnt=0;
    unsigned int cntcopy;

    while (cnt<1048576)
    {
      tmp = "";
      cntcopy = cnt;
      for (unsigned int i=0; i<5; i++)
      {
        tmp += (char)((cntcopy&0xF) + 'a');
        cntcopy >>= 4;
      }
      tmp += name;

      if (namesInDir.find(tmp) == namesInDir.end())
      {
        namesInDir.insert(tmp);
        return(tmp);
      }

      cnt++;
    }

    // Da finde ich wohl keinen einzigartigen Namen.
    if (error.length() == 0)
      error = "Unable to create unique name. Lots of files with the same name in one directory?";
    return(name);
  }
}

std::string Export::GetHtmlName(int nNr)
{
  if (nNr >= 0)
    return(itoStr(nNr, '0', 6) + ".htm");
  else
    return("index.htm");
}

void Export::HtmlLinks(std::string   link_up,
                       int           nNr,
                       bool          fNext,
                       std::string&  strLinkPrev,
                       std::string&  strLinkNext,
                       std::string&  strLinkUp,
                       std::string&  strURLNext)
{
  strLinkPrev = "<span class=\"arr\"><a href=\"" + GetHtmlName(nNr-1) + "\" class=\"arr\">&larr;</a></span>";
  
  if (fNext)
  {
    strURLNext = GetHtmlName(nNr+1);
    strLinkNext = "<span class=\"arr\"><a href=\"" + strURLNext + "\" class=\"arr\">&rarr;</a></span>";
  }
  else
  {
    strURLNext = ""; // todo
    strLinkNext = "<span class=\"ari\">&rarr;</span>";
  }
  
  if (link_up.empty())      
    strLinkUp = "<span class=\"ari\">&uarr;</span>";
  else
    strLinkUp = "<span class=\"arr\"><a href=\"" + link_up + "\" class=\"arr\">&uarr;</a></span>";
}

void Export::WriteConvExif(std::ostream& out, std::string in)
{
  out << "<p>";

  for (std::string::size_type i=0; i<in.length(); )
  {
    if (in[i] == '\n')
    {
      out << "<br/>";
      i++;
    }
    else
    {
      out << in[i];
      i++;
    }
  }

  out << "</p>";
}

void Export::picture(ItPic*             item,
                     std::string        pathOfNames,
                     SetOfStrings&      namesInDir,
                     int                nNr,
                     std::string&       groupout,
                     int&               nType,
                     bool               fNext,
                     std::string        strUp,
                     ItGroup*           pParent)
{
  std::string  strPathSrc; // relativer Quellpfad des Bildes
  std::string  strPathDst; // relativer Pfad der Zieldatei des Bildes
  unsigned int cnt;
  std::string  name_html = "";
  std::string  html_thumbpath;
  
  bool fCreateDst = true;
  {
    // --- Care about the file itself ---------------------------------
    strPathSrc = data->getPicPath(item);
    std::string name_in = FileTools::name(strPathSrc);

    switch (structure)
    {
      case eFlat:
        strPathDst = getUniqueName(namesInDir, name_in);
        break;

      case eFileSys:
        strPathDst  = item->GetPath();
        break;
        
      case eHTML:
        strPathDst = pCache->UpToDate(strPathSrc, destdir, uShrinkW, uShrinkH, uShrinkMode, uShrinkQual);
        if (strPathDst.empty())
          strPathDst = pCache->Insert(strPathSrc, destdir, uShrinkW, uShrinkH, uShrinkMode, uShrinkQual);
        else
          fCreateDst = false;
        name_html = itoStr(nNr, '0', 6) + ".htm";
        break;
        
      default:
        {
          std::string strTmp = "";
          if (fPrefix)
            strTmp = itoStr(nNr, '0', 3) + "_" + name_in;
          else
            strTmp = name_in;
          strPathDst = pathOfNames + getUniqueName(namesInDir, strTmp);
        }
        break;
    }
    
    if (type == eShrink && structure != eHTML)
    {
      std::string suff = strL(FileTools::getSuffix(strPathDst));
      if (suff.compare(".jpg") != 0 && suff.compare(".jpeg") != 0)
      {
        // Don't just replace the original suffix with .jpg, because
        // there may be picture.gif and picture.png in one directory which
        // both would end up as picture.jpg.
        strPathDst += ".jpg";
      }
    }

    if (fCreateDst)
    {
      switch (type)
      {
        case eCopy:
          copy.push_back(strPathSrc);
          copy.push_back(destdir + strPathDst);
          break;
          
        case eShrink:
          shrink.push_back(strPathSrc);
          shrink.push_back(destdir + strPathDst);
          break;
          
        default:
          break;
      }
      annotate.push_back(PathAndPtr(destdir + strPathDst, item, pParent));
    }

    // --- Maybe there are thumbnails to copy? ------------------------
    if (fThumbs && !fHtml)
    {
      std::string sizestr;

      cnt = 0;
      while (TreeView::uThumbSizes[cnt] > 0)
      {
        std::string somepath = data->getPicPath(item, TreeView::uThumbSizes[cnt]);

        if (FileTools::fileExists(somepath))
        {
          copy.push_back(somepath);
          
          // Der thumbnail-Export ist nur sinnvoll, wenn man den Export anschließend in
          // fotoalbum laden kann, ohne dass Thumbnails neu erstellt werden müssen. Von
          // daher müssen sich die Thumbnail-Dateien an den für fotoalbum gültigen 
          // Namensregeln orientieren (siehe std::string TreeView::getPicPath(...)). 
          // Die Regel ist: wenn es nicht auf .jpg oder .jpeg endet, wird .jpeg angehängt.
          std::string filename = FileTools::name(strPathDst);
          std::string suff     = strL(FileTools::getSuffix(filename));
          if (suff.compare(".jpg") != 0 && suff.compare(".jpeg") != 0)
            filename += ".jpeg";
          
          somepath = FileTools::path(destdir + strPathDst) + ".thumbnails/";
          sizestr  = itoStr(TreeView::uThumbSizes[cnt], ' ', 0);
          somepath += sizestr + "x" + sizestr + "/" + filename;

          copy.push_back(somepath);
        }

        cnt++;
      }
    }

    if (fHtml)
    {
      html_thumbpath = pCache->UpToDate(strPathSrc, destdir, uThumbSize, uThumbSize, uShrinkMode, uThumbQual);
      if (html_thumbpath.empty())
      {
        html_thumbpath = pCache->Insert(strPathSrc, destdir, uThumbSize, uThumbSize, uShrinkMode, uThumbQual);
        vThumbs.push_back(strPathSrc);
        vThumbs.push_back(destdir + html_thumbpath);
      }
    }
  }

  // --- html export ------------------------------------------------
  if (fHtml)
  {
    groupout += "  <td align=\"center\"><a href=\"" + name_html + "\"><img src=\"" 
             + html_thumbpath + "\" alt=\"thumb\" border=\"0\"/></a></td>\n";

    if (++nType == (nPicsInARow)+1)
    {
      groupout += "</tr>\n<tr>\n";
      nType = 1;
    }

    std::string strLinkPrev, strLinkNext, strLinkUp, strURLNext;
    FileTools::makeSureDirExists(destdir);
    
#ifdef WIN32_FLTK13    
    FILE* fp = fl_fopen((destdir + name_html).c_str(), "wb");
#else
    FILE* fp = fopen((destdir + name_html).c_str(), "wb");
#endif    
    oFILEstream out(fp);
    
    HtmlLinks(strUp, nNr, fNext, strLinkPrev, strLinkNext, strLinkUp, strURLNext);

    for (unsigned int n=0; n<templ_pic->list.size(); n++)
    {
      switch (templ_pic->list[n].nID)
      {
        case eTVURLNext:
          out << strURLNext;
          break;          
        case eTVHead:
          out << templ_head;
          break;
        case eTVButPrev:
          out << strLinkPrev;
          break;
        case eTVButNext:
          out << strLinkNext;
          break;
        case eTVButUp:
          out << strLinkUp;
          break;
        case eTVDateTime:
          out << item->getDateString() << " " << item->getTimeString();
          break;
        case eTVFilename:
          out << strPathDst;
          break;
        case eTVPath:
          out << item->GetPath();
          break;
        case eTVExif:
          {
            ExifInfo* exif = new ExifInfo(data->getPicPath(item));
            if (exif->hasEXIF())
            {
              WriteConvExif(out, exif->getInfo());
            }
          }
          break;
        case eTVComment:
          out << BaseTools::GetAsHTML(item->GetComment());
          break;
        case eTVKeywords:
          out << item->GetKeywords();
          break;

        case eTVFA_URL:
          out << FA_URL;
          break;

        case eTVFA_Name:
          out << FA_NAME;
          break;

        case eTVFA_Version:
          out << FOTOALBUMVERSION;
          break;

        default:
          out << templ_pic->list[n].value;
          break;
      }
    }
  }
}

void Export::groupToFileHtml(std::string path,
                             std::string strLinkPrev,
                             std::string strLinkNext,
                             std::string strLinkUp,
                             int         nPrevGroup,
                             int         nNextGroup,
                             std::string strText,
                             std::string strComment,
                             std::string strLinks,
                             std::string strThumbs,
                             std::string strKeywords)
{
  FileTools::makeSureDirExists(FileTools::path(path));

#ifdef WIN32_FLTK13    
  FILE* fp = fl_fopen(path.c_str(), "wb");
#else
  FILE* fp = fopen(path.c_str(), "wb");
#endif    
  oFILEstream htmlfile(fp);
  
  for (unsigned int n=0; n<templ_group->list.size(); n++)
  {
    switch (templ_group->list[n].nID)
    {
      case eTVHead:
        htmlfile << templ_head;
        break;
      case eTVButPrev:
        htmlfile << strLinkPrev;
        break;
      case eTVButNext:
        htmlfile << strLinkNext;
        break;
      case eTVButUp:
        htmlfile << strLinkUp;
        break;
      case eTVButNextGroup:
        if (nNextGroup>=0)
          htmlfile << "<span class=\"arr\"><a href=\"" + GetHtmlName(nNextGroup) + "\" class=\"arr\">&raquo;</a></span>";
        else
          htmlfile << "<span class=\"ari\">&raquo;</span>";
        break;
      case eTVButPrevGroup:
        if (nPrevGroup>=0)
          htmlfile << "<span class=\"arr\"><a href=\"" + GetHtmlName(nPrevGroup) + "\" class=\"arr\">&laquo;</a></span>";
        else
          htmlfile << "<span class=\"ari\">&laquo;</span>";
        break;
      case eTVTitle:
        htmlfile << strText;
        break;
      case eTVComment:
        htmlfile << strComment;
        break;
      case eTVLinks:
        htmlfile << strLinks;
        break;
      case eTVThumbs:
        if (strThumbs.length())
          htmlfile << "<table><tr>" << strThumbs << "</tr></table>";
        break;
      case eTVKeywords:
        htmlfile << strKeywords;
        break;

      case eTVFA_URL:
        htmlfile << FA_URL;
        break;

      case eTVFA_Name:
        htmlfile << FA_NAME;
        break;

      case eTVFA_Version:
        htmlfile << FOTOALBUMVERSION;
        break;

      default:
        htmlfile << templ_group->list[n].value;
        break;
    }
  }
}

void Export::group(ItGroup*           item,
                   std::string        pathOfNames,
                   SetOfStrings&      namesInDir,
                   int&               nNr,
                   std::string&       groupout,
                   int&               nType,
                   bool               fNext,
                   std::string        strUp,
                   int&               nPrevGroup)
{
  std::string      name;
  SetOfStrings     namesInMe;
  SetOfStrings*    namesptr;
  int              htmltype = 1;
  int              nMyPrevGroup = nPrevGroup;

  switch (structure)
  {
    case eView:
      if (fHtml)
        name = itoStr(nNr, '0', 6);
      else
      {
        name = FileTools::cleanString(item->GetName());
        if (fPrefix)
          name = itoStr(nNr, '0', 3) + "_" + name;
        name = getUniqueName(namesInDir, name);
      }
      pathOfNames += name + "/";
      namesptr = &namesInMe;
      break;

    case eFlat:
      namesptr = &namesInDir;
      break;

    default:
      namesptr = &namesInMe;
      break;
  }

  // export all my children
  std::vector<ItGroup*> listGroups;
  if (fRecursive)
  {
    ItGroup* i = item->pFirstChild;
    while (i != 0)
    {
      listGroups.push_back(i);
      i = i->pNext;
    }
  }
  
  int nMyNum = nNr;
  std::string strFile = GetHtmlName(nNr++);
  
  std::string strThumbtable = "";
  for (unsigned int n=0; n<item->vPics.size(); n++)
    picture(item->vPics[n]->pPic,
            pathOfNames, *namesptr, nNr++, strThumbtable, htmltype, 
            fNext || !listGroups.empty() || (n<item->vPics.size()-1),
            strFile, item);
  
  int nFirstGroupInsideOfMe = -1;
  if (!listGroups.empty())
    nFirstGroupInsideOfMe = nNr;
  nPrevGroup = nMyNum;
  std::string strLinks = "";
  for (unsigned int n=0; n<listGroups.size(); n++)
    group(listGroups[n], 
          pathOfNames, *namesptr, nNr, strLinks, htmltype,
          fNext || (n<listGroups.size()-1),
          strFile, nPrevGroup);


  // --- html export ------------------------------------------------
  if (fHtml)
  {
    groupout += "<a href=\"" + strFile + "\">" + item->GetName() + "</a><br/>\n";
    nType = 0;

    std::string strLinkPrev, strLinkNext, strLinkUp, strURLNext;
    HtmlLinks(strUp, nMyNum, 
              fNext || !item->vPics.empty() || !listGroups.empty(),
              strLinkPrev, strLinkNext, strLinkUp, strURLNext);
    
    int nNextGroup = -1;
    if (nFirstGroupInsideOfMe >= 0)
      nNextGroup = nFirstGroupInsideOfMe;
    else if (fNext)
      nNextGroup = nNr;
    groupToFileHtml(destdir + strFile,
                    strLinkPrev, strLinkNext, strLinkUp,
                    nMyPrevGroup, nNextGroup,
                    item->GetName(), BaseTools::GetAsHTML(item->GetComment()),
                    strLinks, strThumbtable, item->GetKeywords());
  }
}

void Export::PrepareTemplate(Template* templ)
{
  for (unsigned int n=0; n<templ->list.size(); n++)
  {
    // std::cout << templ->list[n].nID << " " << templ->list[n].value << "\n----------------------------------------\n\n";
    if (templ->list[n].nID == 0)
    {
      std::string val = templ->list[n].value;
      if (val.compare("{HEAD}") == 0)
        templ->list[n].nID = eTVHead;
      else if (val.compare("{BUTTON_PREV}") == 0)
        templ->list[n].nID = eTVButPrev;
      else if (val.compare("{BUTTON_NEXT}") == 0)
        templ->list[n].nID = eTVButNext;
      else if (val.compare("{BUTTON_PREV_GROUP}") == 0)
        templ->list[n].nID = eTVButPrevGroup;
      else if (val.compare("{BUTTON_NEXT_GROUP}") == 0)
        templ->list[n].nID = eTVButNextGroup;
      else if (val.compare("{BUTTON_UP}") == 0)
        templ->list[n].nID = eTVButUp;
      else if (val.compare("{URL_NEXT}") == 0)
        templ->list[n].nID = eTVURLNext;
//      else if (val.compare("{BUTTON_LIN_NEXT}") == 0)
//        templ->list[n].nID = eTVButLinNext;
      else if (val.compare("{DATETIME}") == 0)
        templ->list[n].nID = eTVDateTime;
      else if (val.compare("{FILENAME}") == 0)
        templ->list[n].nID = eTVFilename;
      else if (val.compare("{PATH}") == 0)
        templ->list[n].nID = eTVPath;
      else if (val.compare("{EXIF}") == 0)
        templ->list[n].nID = eTVExif;
      else if (val.compare("{COMMENT}") == 0)
        templ->list[n].nID = eTVComment;
      else if (val.compare("{KEYWORDS}") == 0)
        templ->list[n].nID = eTVKeywords;
      else if (val.compare("{TITLE}") == 0)
        templ->list[n].nID = eTVTitle;
      else if (val.compare("{LINKS}") == 0)
        templ->list[n].nID = eTVLinks;
      else if (val.compare("{THUMBTABLE}") == 0)
        templ->list[n].nID = eTVThumbs;
      else if (val.compare("{FA_URL}") == 0)
        templ->list[n].nID = eTVFA_URL;
      else if (val.compare("{FA_Name}") == 0)
        templ->list[n].nID = eTVFA_Name;
      else if (val.compare("{FA_Version}") == 0)
        templ->list[n].nID = eTVFA_Version;
      else
      {
        std::cerr << "unknown: " << val << "\n";
        templ->list[n].nID = 1;
      }
    }
  }
}

void Export::run(std::vector<ItPicInGroup*> listPics, ItGroup* pPicParent, std::vector<ItGroup*> listGroups)
{
  int type = 0;
  int nPrevGroup = -1;

  // --- Read html templates ----------------------------------------
  {
    // search most local template (usr, home)
    std::string path = BaseTools::getFilePath("share/fotoalbum/templates/default/group.html");
    std::string templpath;
    if (path.length() == 0)
    {
      std::cout << "html template not found\n";
      std::cerr << "html template not found\n";
      templpath = "/home/jens/projekte/fotoalbum/CPP001/repo_fotoalbum/templates/default/";
    }
    else
      templpath = path.substr(0, path.length()-10);

    std::cout << "Using " << templpath << "\n";

    {
      std::ifstream in;
      in.open((templpath+"head.html").c_str());
      if (!in)
      {
        error = "Error opening head.html";
        return;
      }
      char inchar;
      while (!in.eof())
      {
        inchar = in.get();
        if (!in.eof())
          templ_head += inchar;
      }
      in.close();
    }

    try
    {
      templ_pic   = new Template(templpath + "picture.html");
      templ_group = new Template(templpath + "group.html");
    }
    catch (TemplateException e)
    {
      error = e.what();
      return;
    }

    PrepareTemplate(templ_group);
    PrepareTemplate(templ_pic);
  }

  // --- Prepare ----------------------------------------------------
  if (destdir[destdir.length()-1] != '/')
    destdir += '/';
  
  if (fHtml)
    pCache = new ExpPicCache(destdir);
  
  error = "";
    
  std::string strThumbtable = "";
  std::string strLinks = "";
  int nFirstGroupInsideOfMe = -1;
  {
    std::string   pathOfNames = "";
    SetOfStrings  namesInDir;
    
    {
      int nNr = 0;
      for (unsigned int cnt=0; cnt<listPics.size(); cnt++)
        picture(listPics[cnt]->pPic,
                pathOfNames, namesInDir, nNr++, strThumbtable, type,
                !listGroups.empty() || cnt<listPics.size()-1,
                "index.htm", pPicParent);
      if (!listGroups.empty())
        nFirstGroupInsideOfMe = nNr;
      for (unsigned int cnt=0; cnt<listGroups.size(); cnt++)
        group(listGroups[cnt],
              pathOfNames, namesInDir, nNr, strLinks, type,
              cnt<listGroups.size()-1,
              "index.htm", nPrevGroup);
    }
  }

  // html file
  if (fHtml)
  {
    std::string strLinkPrev, strLinkNext, strLinkUp, strURLNext;
    HtmlLinks("", -1, true, strLinkPrev, strLinkNext, strLinkUp, strURLNext);

    groupToFileHtml(destdir + "index.htm",
                    strLinkPrev, strLinkNext, strLinkUp,
                    -1, nFirstGroupInsideOfMe,
                    "Fotoalbum", "", strLinks, strThumbtable, "");
  }

  delete templ_pic;
  delete templ_group;
  if (fHtml)
    delete pCache;
}

void Export::run(std::vector<ItPicInGroup*> listPics, ItGroup* pPicParent)
{
  std::vector<ItGroup*> listGroups;
  run(listPics, pPicParent, listGroups);
}

void Export::run(std::vector<ItGroup*> listGroups)
{
  std::vector<ItPicInGroup*> listPics;
  run(listPics, 0, listGroups);
}

int Export::run_annotate()
{
  if (!fAddToJpeg)
    return(0);
  else
  {
    ItPic* pic;
    ItGroup* parent;
    int nRetCode = 0;
    std::set<std::string> kwlist;
    std::set<std::string>::iterator it;

    for (unsigned int n=0; n<annotate.size(); n++)
    {
      pic    = annotate[n].ptr;
      parent = annotate[n].parent;
#if (EXIV2_FOUND == 1)
      try
      {
        // std::cout << "Annotate " << annotate[n].path << "\n";
        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(annotate[n].path);
        if (image.get() != 0)
        {
          image->readMetadata();

          // exif
          {
            Exiv2::ExifData exifData = image->exifData();
            Exiv2::Exifdatum& tag = exifData["Exif.Photo.DateTimeOriginal"];
            std::string datestr = pic->getDateString();
            std::string timestr = pic->getTimeString();
            if (timestr.length() == 0)
              timestr = "00:00:00";
            if (datestr.length() == 0)
              datestr = "0000:00:00";
            else
            {
              datestr[4] = ':';
              datestr[7] = ':';
            }
            tag.setValue(datestr + " " + timestr);

            image->setExifData(exifData);
          }

# if 0
          // Doing this sometimes leads to fltk not being able to load the image, so
          // I don't export to jpeg comment. Data is in iptc caption, anyway. However,
          // every other app I tried is able to load the image (internal jpegtran is
          // able to do so, too).
          //
          // jpeg comment
          {
            if (pic->getComment().length())
              image->setComment(pic->getComment());
          }
# endif

          // iptc and xmp
          {
            ItPic* item = pic;
            Exiv2::IptcData& iptcData = image->iptcData();
            Exiv2::XmpData&  xmpData  = image->xmpData();
            Exiv2::IptcKey ek("Iptc.Application2.Keywords");
            Exiv2::XmpKey  xk("Xmp.dc.subject");

            // read existing keywords into kwlist
            kwlist.clear();
            if (fKwMerge)
              ExifInfo::ReadKeywords(image, kwlist);

            // delete all existing keywords
            if (!iptcData.empty())
            {
              Exiv2::IptcData::iterator md  = iptcData.begin();
              while (md != iptcData.end())
              {
                if (md->key().compare("Iptc.Application2.Keywords") == 0)
                  md = iptcData.erase(md);
                else
                  md++;
              }
            }

            // delete all existing keywords
            if (!xmpData.empty())
            {
              Exiv2::XmpData::iterator md  = xmpData.begin();
              while (md != xmpData.end())
              {
                if (md->key().compare("Xmp.dc.subject") == 0)
                  md = xmpData.erase(md);
                else
                  md++;
              }
            }

            // get keywords
            item->kw_set(kwlist);
            
            if (fKwInherit)
            {
              // inherit keywords of groups above
              ItGroup* g = parent;
              while (g)
              {
                // By using kw_list() the order of keywords is preserved, but duplicates are
                // possible. Using kw_set() removes duplicates but changes order, but it gives
                // an easy way to merge with keywords which are in the file already.
                g->kw_set(kwlist);
                g = g->pParent;
              }
            }
            
            //
            it = kwlist.begin();
            Exiv2::Value::AutoPtr vx = Exiv2::Value::create(Exiv2::xmpBag);
            while (it != kwlist.end())
            {
              Exiv2::Value::AutoPtr v = Exiv2::Value::create(Exiv2::string);
              v-> read(*it);
              vx->read(*it);
              iptcData.add(ek, v.get ());
              it++;
            }
            xmpData.add(xk, vx.get());

            // caption
            if (pic->GetComment().length())
            {
              // set comment if it is allowed to be overwritten or if there is no comment yet
              Exiv2::Iptcdatum& caption = iptcData["Iptc.Application2.Caption"];
              std::string captionstr = UTF8::trim(caption.toString());

              if (fCommentOverwrite || captionstr.length() == 0)
                iptcData["Iptc.Application2.Caption"] = pic->GetComment();
            }

            //
            image->setIptcData(iptcData);
          }

          //
          image->writeMetadata();
        }
      }
      catch (Exiv2::AnyError& e)
      {
        std::cerr << "Caught Exiv2 exception '" << e << "'\n";
        nRetCode = -1;
      }
#endif
    }

    return(nRetCode);
  }
}

ExpPicCache::ExpPicCache(std::string strDir)
{
  this->strDir = strDir;
  try
  {
    xState = new SimpleXMLTransfer(strDir + "/content.xml");
  }
  catch (XMLException e)
  {
    std::cerr << "Error loading export cache state: " << e.what() << "\n";
    xState = new SimpleXMLTransfer();
  }
  
  nNum = 0;
  
  std::string strTmp;
  for (int n=0; n<xState->getChildCount(); n++)
  {
    SimpleXMLTransfer* xItem = xState->getChildAt(n);
    strTmp = xItem->attribute("n");
    ItPic::DecodePath(strTmp);
    mapC.insert(std::make_pair(strTmp, xItem));
    
    int nNr = xItem->attributeAsInt("i");
    if (nNum <= nNr)
      nNum = nNr+1;    
  }
}

ExpPicCache::~ExpPicCache()
{
  xState->SaveToFile(strDir + "/content.xml", 0, 2);
  delete xState;
}

std::string ExpPicCache::UpToDate(std::string strSrc, std::string strDstDir,
                                  int nW, int nH, int nMode, int nQual)
{
  std::map<std::string, SimpleXMLTransfer*>::iterator it = mapC.find(strSrc);
  if (it == mapC.end())
    return("");
  
  std::string strName = Name(strSrc, nW, nH, nMode, nQual, it->second->attributeAsInt("i"));
  if (FileTools::isFileUpToDate(strSrc, strDstDir +"/"+strName) > 0)
    return(strName);
  else
    return("");
}

std::string ExpPicCache::Insert(std::string strSrc, std::string strDstDir,
                                int nW, int nH, int nMode, int nQual)
{
  SimpleXMLTransfer* xP;
  int nRes;
  std::map<std::string, SimpleXMLTransfer*>::iterator it = mapC.find(strSrc);
  if (it == mapC.end())
  {
    xP = new SimpleXMLTransfer();
    xP->setName("p");
    xP->addAttribute("n", ItPic::EncodePath(strSrc));
    nRes = nNum++;
    xP->addAttribute("i", nRes);
    xState->addChild(xP);
    mapC.insert(std::make_pair(strSrc, xP));
  }
  else
  {
    xP = it->second;
    nRes = xP->attributeAsInt("i");
  }

  std::string strName = Name(strSrc, nW, nH, nMode, nQual, nRes);

  return(strName);
}
 
std::string ExpPicCache::Name(std::string strSrc, int nW, int nH, int nMode, int nQual,
                              int nNr)
{
  std::string strSuff;
  if (nW == 0)
    strSuff = FileTools::getSuffix(strSrc);
  else
    strSuff = ".jpg";
  
  return("p/" + itoHexStr(nNr, 16) + "_" 
         + itoStr(nW, ' ', 1) + itoStr(nH, ' ', 1)
         + "_" + itoStr(nQual, '0', 3) + itoHexStr(nMode, 4) + strSuff);
}

