/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "keyword.h"

#include "basetools.h"
#include "../shared/utf8.h"
#include "../shared/lib_conversions.h"

void Keywords::set(std::set<std::string>& kwset, std::string strKeywords)
{
  std::set<std::string> mykwset = KeywordList_Set(strKeywords);
  
  // merge my keywords into kwset
  kwset.insert(mykwset.begin(), mykwset.end());
}
  
void Keywords::list(std::vector<std::string>& kwlist, std::string strKeywords)
{
  std::string            kw    = strKeywords;
  std::string::size_type pos   = kw.find(',');
  std::string::size_type start = 0;
  
  while (pos != std::string::npos)
  {
    if (pos > start)
      kwlist.push_back(kw.substr(start, pos-start));
    start = pos+1;
    pos   = kw.find(',', start);
  }
  if (start == 0 && kw.length())
    kwlist.push_back(kw);
  else if (start < kw.length())
    kwlist.push_back(kw.substr(start, kw.length()-start));
}


std::string Keywords::Keyword(std::string in)
{
  in = UTF8::trim(UTF8::collapse_space(in));
  for (std::string::size_type n=0; n<in.length(); n++)
  {
    if (in[n] == '.' || in[n] == ',')
      in[n] = '_';
  }
  return(in);
}

std::string Keywords::KeywordList_String(std::string in)
{
  std::string::size_type pos, start;
  
  for (pos=0; pos<in.length(); pos++)
  {
    if (in[pos] == '\n')
      in[pos] = ',';
    else if (UTF8::isspace(in[pos]))
      in[pos] = ' ';
  }
  
  in = UTF8::trim(UTF8::collapse_space(collapse_char(in, ',')));
  
  start = 0;
  while ((pos = in.find(", ", start)) != std::string::npos)
  {
    in = in.replace(pos, 2, ",");
    start = pos;
  }
  
  start = 0;
  while ((pos = in.find(" ,", start)) != std::string::npos)
  {
    in = in.replace(pos, 2, ",");
    start = pos;
  }
  
  in = collapse_char(in, ',');
        
  return(in);
}

std::set<std::string> Keywords::KeywordList_Set(std::string in)
{
  std::string::size_type pos, start;
  std::set<std::string>  kwset;
  std::string            kw;
  
  for (pos=0; pos<in.length(); pos++)
  {
    if (in[pos] == '\n')
      in[pos] = ',';
    else if (UTF8::isspace(in[pos]))
      in[pos] = ' ';
  }
  
  in = UTF8::trim(UTF8::collapse_space(collapse_char(in, ',')));
  
  pos   = in.find(',');
  start = 0;
  
  while (pos != std::string::npos)
  {
    if (pos > start)
    {
      kw = UTF8::trim(in.substr(start, pos-start));
      if (kw.length())
        kwset.insert(kw);
    }
    start = pos+1;
    pos   = in.find(',', start);
  }
  if (start == 0 && in.length()) // only one keyword, without trailing ','
  {
    kw = UTF8::trim(in);
    if (kw.length())
      kwset.insert(kw);
  }
  else if (start < in.length()) // the last keyword, without trailing ','
  {
    kw = UTF8::trim(in.substr(start, in.length()-start));
    if (kw.length())
      kwset.insert(kw);
  }
  
  return(kwset);
}

std::string Keywords::RemoveDupes(std::string kw)
{
  std::string out = "";
  std::set<std::string> kwset;
  std::pair<std::set<std::string>::iterator,bool> p;
  std::vector<std::string> kwlist;
  list(kwlist, kw);
  
  for (unsigned int n=0; n<kwlist.size(); n++)
  {
    p = kwset.insert(kwlist[n]);
    if (p.second == true)
      out += "," + kwlist[n];
  }
  
  if (out.length())
    out += ",";
  
  return out;
}
