/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef KEYWORD_H
# define KEYWORD_H

#include <string>
#include <set>
#include <vector>

class Keywords
{
public:
  
  /**
   * Translates reserved characters in a keyword to underscores.
   */
  static std::string Keyword(std::string in);
  
  /**
   * Trim string 'in' which contains a keywords list (user input!) and return the result.
   * Conversions:
   *   newline        -> ','
   *   any whitespace -> ' '
   *   ",,"           -> ','
   *   "  "           -> ' '
   * 
   * The result will be a string with keywords separated by ','. Leading and trailing 
   * whitespace is stripped from keywords. Examples:
   *   ,keyword1,
   *   ,keyword1,keyword2,
   *   ,keyword1,keyword2,keyword3,
   */
  static std::string KeywordList_String(std::string in);
  
  /**
   * 'in' contains a keywords list (user input), is converted to a set and returned.
   */
  static std::set<std::string> KeywordList_Set(std::string in);
  
  /**
   * Adds strKeywords to the list (no dupes, alphabetically)
   */
  static void set(std::set<std::string>& kwset, std::string strKeywords);
  
  /**
   * Adds strKeywords to the list (dupes, unchanged order)
   */
  static void list(std::vector<std::string>& kwlist, std::string strKeywords);

  
  /**
   * returns "in" without duplicates
   */
  static std::string RemoveDupes(std::string in);

};


#endif
