/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef META_H
# define META_H

#include <set>
#include <vector>
#include <string>

#include "keyword.h"

class ItMeta
{
public:
  virtual std::string GetComment() = 0;
  virtual std::string GetKeywords() = 0;
  virtual const std::string* GetCommentPtr() = 0;
  virtual const std::string* GetKeywordsPtr() = 0;
  virtual void SetComment(std::string strNew) {};
  virtual void SetKeywords(std::string strNew) {};
  
  /**
   * Adds keywords to the list (no dupes, alphabetically)
   */
  void kw_set(std::set<std::string>& kwset) { Keywords::set(kwset, GetKeywords()); };
  
  /**
   * Adds keywords to the list (dupes, unchanged order)
   */
  void kw_list(std::vector<std::string>& kwlist) { Keywords::list(kwlist, GetKeywords()); };

  void kw_add(std::set<std::string> kwset);
  
  /**
   * mode=0: beginning must match
   * mode=1: whole keyword must match
   */
  void kw_replace(std::string from, std::string to, int mode);
  
  /**
   * While in memory, the keyword list always has leading and trailing comma, 
   * which makes it easy to search for whole keywords, or just the beginning 
   * or end:
   *   ,wholekeyword,
   *   ,startofkeyword
   *   endofkeyword,
   * 
   * This function makes sure that this is the case.
   */
  void kw_fix(std::string& k);

  std::string GetKeywordsFile();
  
  std::string strID;
  std::string GetID() { return strID; };    
  const std::string* GetIDPtr() { return &strID; };    
};

#endif
