/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef BASE_PIC_H
# define BASE_PIC_H

# include <string>
# include "../shared/SimpleXMLTransfer.h"
# include "datetime.h"
# include "meta.h"

class ItPic : public ItMeta
{
private:
  std::string strPath;
public:
  std::string GetPath() { return strPath; };
  const std::string* GetPathPtr() { return &strPath; };
  
  std::string getDescr(int level, bool fShort, const std::string strBasepath);
    
  /**
   * number of levels for 
   * std::string ItPic::getDescr()
   */
  enum { eDescrLevels = 6 };
    
  /**
   * eEXIF:          1. exif  2. date = none, time = none
   * eEXIFFILE:      1. exif  2. file
   * eEXIFFILEDATE:  1. exif  2. date = file, time = none
   * eFILE:          1. file
   */
  enum eDateTimeSource { eEXIF = 0, eEXIFFILE = 1, eEXIFFILEDATE = 2, eFILE = 3, eABORT = 100 };

  void setData(std::string       newpath, 
               std::string       basepath,
               eDateTimeSource   eDTS);
  
  
private:
  JWWDateTime dt;
public:
  bool hasTime() { return(dt.isTimeSet()); };
  bool hasDate() { return(dt.isDateSet()); };
  std::string getDateString() { return(dt.getDateAsString()); };
  std::string getTimeString() { return(dt.getTimeAsString()); };
  JWWDateTime getDT() { return(dt); };
  void timeshift(int nSec) { dt.shiftSec(nSec); };
  int SetDate(const char* nD) { return(dt.setDate(nD)); };
  int SetTime(const char* nT) { return(dt.setTime(nT)); };
  
  /**
   * Includes renaming the file on disk
   */
  int FileRename(std::string strBase,
                 std::string strNew);
  
  /**
   * only changes internal data, but not the disk
   */
  void FileReplace(std::string strNew) { strPath = strNew; };
  
  
public:
  int nRefCnt;
  int IncRefCnt() { return(++nRefCnt); };
  int DecRefCnt() { return(--nRefCnt); };
  
private:
  std::string strKeywords;
  std::string strComment;
public:
  virtual std::string GetComment()  { return strComment;  };
  virtual std::string GetKeywords() { return strKeywords; };
  virtual const std::string* GetCommentPtr()   { return &strComment;  };
  virtual const std::string* GetKeywordsPtr()  { return &strKeywords; };
  virtual void SetComment(std::string strNew)  { strComment  = strNew; };
  virtual void SetKeywords(std::string strNew) { strKeywords = strNew; kw_fix(strKeywords); };  
  
  
public:
  enum { eRatingNone = 0, eRatingWorst = 1, eRatingBest=5 };
  std::vector<int> vRating;
  void SetRating(int nRate);
  
  
public:
  
  /**
   * Read from XML, versions before 10
   */
  ItPic(SimpleXMLTransfer* xIn);
  
  /**
   * Read from XML, version 10 and later
   */
  ItPic(SimpleXMLTransfer* xIn, std::string strPathPrefix);
  
  ItPic();
  virtual ~ItPic() {};
  
  void ChangeBasepath(std::string oldbp, std::string newbp);
  
  void SaveToXML(std::string& strPathPrefix, SimpleXMLTransfer* xPics);

public:
  /**
   * Filenames may not be UTF8 but anything else, therefore a special coding
   * is needed to put them into UTF8 xml
   */  
  static std::string EncodePath(std::string path);
  static void DecodePath(std::string& path);
};

class ItPicInGroup
{
public:
  ItPic* pPic;
  bool   fMarked;
  
  ItPicInGroup();
  ItPicInGroup(ItPic* pPic);
};

#endif
