/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

int tf_push_str(TestStack* pStack, std::string strCmd)
{
  pStack->Push(TestStack::eStr, strCmd);
  return 0;
}

int tf_push_date(TestStack* pStack, std::string strCmd)
{
  pStack->Push(TestStack::eDate, strCmd);
  return 0;
}

int tf_push_time(TestStack* pStack, std::string strCmd)
{
  pStack->Push(TestStack::eTime, strCmd);
  return 0;
}

int tf_push_int(TestStack* pStack, std::string strCmd)
{
  pStack->Push(TestStack::eInt, strCmd);
  return 0;
}

int tf_test_int(TestStack* pStack, std::string strCmd)
{
  if (pStack->GetType(0) == TestStack::eInt)
    return 0;
  else
  {
    pStack->SetErr(strCmd + " expects [0] to be an int");
    return -1;
  }
}

int tf_pop_int(TestStack* pStack, std::string strCmd)
{
  std::string strDummy;
  if (pStack->Pop(strDummy) == TestStack::eInt)
    return 0;
  else
  {
    pStack->SetErr(strCmd + " expects [0] to be an int");
    return -1;
  }
}

int tf_binary(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA, strExplainB;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  TestStack::eDataType b = pStack->Pop(strExplainB);
  
  if (a == TestStack::eErr || b == TestStack::eErr)
  {
    pStack->SetErr(strCmd + ": stack underflow");
    return -1;
  }
    
  if (a == b && a != TestStack::eStr)
  {
    pStack->Push(a, strExplainB + " " + strExplainA + " " + strCmd);
    return 0;
  }
  else
  {
    pStack->SetErr(strCmd + " expects [0] and [-1] to be of the same type and not string");
    return -1;
  }
}

int tf_unary(TestStack* pStack, std::string strCmd)
{
  std::string strExplain;
  TestStack::eDataType a = pStack->Pop(strExplain);
  
  if (a == TestStack::eErr)
  {
    pStack->SetErr(strCmd + ": stack underflow");
    return -1;
  }
    
  if (a == TestStack::eInt)
  {
    pStack->Push(a, strExplain + " " + strCmd);
    return 0;
  }
  else
  {
    pStack->SetErr(strCmd + " expects [0] to be an int");
    return -1;
  }
}

int tf_cmp(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA, strExplainB;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  TestStack::eDataType b = pStack->Pop(strExplainB);

  if (a == TestStack::eErr || b == TestStack::eErr)
  {
    pStack->SetErr(strCmd + ": stack underflow");
    return -1;
  }
    
  if (a == b && a != TestStack::eStr)
  {
    pStack->Push(TestStack::eInt, strExplainB + " " + strExplainA + " " + strCmd);
    return 0;
  }
  else
  {
    pStack->SetErr(strCmd + " expects [0] and [-1] to be of the same type and not string");
    return -1;
  }
}

int tf_nop(TestStack* pStack, std::string strCmd)
{
  return 0;
}

int tf_str_eval(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  
  if (a != TestStack::eStr)
  {
    pStack->SetErr(strCmd + " expects [0] to be a string");
    return -1;
  }
  pStack->Push(TestStack::eInt, strExplainA + " " + strCmd);
  pStack->Push(a, strExplainA);
    
  return 0;
}

int tf_str_cat(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA, strExplainB;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  TestStack::eDataType b = pStack->Pop(strExplainB);
  
  if (a == TestStack::eErr || b == TestStack::eErr)
  {
    pStack->SetErr(strCmd + ": stack underflow");
    return -1;
  }
    
  if (a == b && a == TestStack::eStr)
  {
    pStack->Push(a, strExplainA + " " + strExplainB + " " + strCmd);
    return 0;
  }
  else
  {
    pStack->SetErr(strCmd + " expects [0] and [-1] to be strings");
    return -1;
  }
}

int tf_dup(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  if (a != TestStack::eErr)
  {
    pStack->Push(a, strExplainA);
    pStack->Push(a, strExplainA);
    return 0;
  }
  else
  {
    pStack->SetErr(strCmd + " expects a non-empty stack");
    return -1;
  }
}

int tf_swap(TestStack* pStack, std::string strCmd)
{
  std::string strExplainA, strExplainB;
  TestStack::eDataType a = pStack->Pop(strExplainA);
  TestStack::eDataType b = pStack->Pop(strExplainB);

  if (a == TestStack::eErr || b == TestStack::eErr)
  {
    pStack->SetErr(strCmd + ": stack underflow");
    return -1;
  }

  pStack->Push(a, strExplainA);
  pStack->Push(b, strExplainB);
  return 0;
}

int tf_pop(TestStack* pStack, std::string strCmd)
{
  std::string strExplain;
  TestStack::eDataType a = pStack->Pop(strExplain);
  if (a == TestStack::eErr)
  {
    pStack->SetErr(strCmd + " expects a non-empty stack");
    return -1;
  }
  else
    return 0;
}

