/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef SEARCHPROG_H
# define SEARCHPROG_H

#include <string>
#include <vector>
#include <map>
#include "exif.h"

class ItPicInGroup;
class ItGroup;
class TreeView;

class SearchContext
{
public:
  ItPicInGroup* pPic;
  ItGroup* pParent;
  std::string strBasePath;
  std::vector<std::string> vStringsTemp;
  int nTmpStrIdx;
  std::string strPCom;
  std::string strPKey;
  ExifInfo* pExif;
  
  void ReadExif(std::string strPath);
};

class TestStack
{
public:
  enum eDataType { eInt, eTime, eDate, eStr, eErr };

  void SetErr(std::string strMsg) { if (strErr.length() == 0) strErr = strMsg; };
  void Push(eDataType type, std::string strExplain);
  eDataType GetType(int nOff);
  eDataType Pop(std::string& strExplain);
  int Size() { return stackType.size(); };
  void Print(std::string& strOut);
  
  std::string GetErr() { return strErr; };
  
  TestStack()
  {
    strErr = "";
  };
  
private:
  std::vector<eDataType>   stackType;
  std::vector<std::string> stackExplain;
  std::string strErr;
};

typedef int(*ptrFunSearch)(intptr_t*&, SearchContext*, intptr_t);
typedef int(*ptrFunTest)(TestStack*, std::string);

class SearchFun
{
public:
  SearchFun(ptrFunTest pTestFun, ptrFunSearch pFun)
  {
    this->pTestFun = pTestFun;
    this->pFun = pFun;
    this->nCreatesTempStrings = 0;
  };
  
  SearchFun(ptrFunTest pTestFun, ptrFunSearch pFun, int nCreatesTempStrings)
  {
    this->pTestFun = pTestFun;
    this->pFun = pFun;
    this->nCreatesTempStrings = nCreatesTempStrings;
  };
  
  ptrFunTest pTestFun;
  ptrFunSearch pFun;
  int nCreatesTempStrings;
};

class ProgInstr
{
public:
  ProgInstr(SearchFun* sf,
            intptr_t   arg)
  {
    this->pFun = sf->pFun;
    this->arg  = arg;
  };
  
  ptrFunSearch pFun;
  intptr_t     arg;
};

class SearchProg
{
public:
  SearchProg(std::string strExpr);
  ~SearchProg();
  std::string GetErr() { return strErr; };
  std::string GetLog() { return strLog; };
  int Run(ItPicInGroup* pPic, ItGroup* pParent, std::string strBasePath, std::string strPCom, std::string strPKey);

private:
  bool fNeedsPCom;
  bool fNeedsPKey;
public:
  bool NeedsPCom() { return fNeedsPCom; };
  bool NeedsPKey() { return fNeedsPKey; };
  
  
private:
  std::string strErr, strLog;
  std::vector<ProgInstr*>   vInstr;
  std::vector<std::string*> vStringsConst;
  intptr_t* stack;
  SearchContext context;
  
private:
  static std::map<std::string, SearchFun*> mapFunsNoArg;  
  static std::map<std::string, SearchFun*> mapFunsStrArg;  
};

#endif

