/**
 * compile using
 * c++ -g -lpthread -o threadtest threadtest.cpp
 * 
 * This simple program shows the same problem as fotoalbum
 * when executed via
 *   xterm -e threadtest
 * It quits after the worker thread has finished.
 */

#define USE_JOIN   1
#define USE_SLEEP  1

#include <iostream>
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>

#include "workerdata.cpp"
#include "../shared/lib_conversions.cpp"
#include "../shared/filetools.cpp"
#include "basetools.cpp"

/** Do I receive a signal? Yes! That's the solution!
    But why do I receive SIGHUP? */
void signal_handler (int signum) 
{  
  std::cerr << "Received signal: " << signum << "  " << strsignal(signum) << "\n";
  sleep(2);
}

int main()
{  
  signal (SIGHUP, signal_handler);
  
  WorkThumbCreate work;

  work.filelist.push_back("ref");
  work.filelist.push_back("result");
  work.filelist.push_back("ref");
  work.filelist.push_back("result");
  work.filelist.push_back("ref");
  work.filelist.push_back("result");

  if (work.filelist.size())
  {
    work.nMaxW  = 50;
    work.nMaxH  = 50;
    work.flQual = 0.6;

    WorkerData   wd;
    wd.init(work.filelist.size()/2, (void*)&work);

    int       nRetCode;

    pthread_t       workerThread;
    pthread_mutex_init(&workermutex, NULL);

  /* Initialize and set thread detached attribute */
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);

    if (pthread_create(&workerThread, &attr, createThumbs, (void*)&wd))
    {
      std::cerr << "error creating thread\n";
      std::cerr.flush();
      while (1) {};
      nRetCode = -1;
    }
    else
    {
      /* Free attribute and wait for the other threads */
      pthread_attr_destroy(&attr);

#if USE_JOIN != 0
      if (pthread_join(workerThread, (void**)&nRetCode))
      {
        std::cerr << "error joining thread\n";
        std::cerr.flush();
        while (1) {};
      }
#else
      while (wd.nRun) {};
      nRetCode = wd.nRetCode;
#endif
    }
    pthread_mutex_destroy(&workermutex);
  }

  
  for (int n=0; n<5; n++)
  {
    std::cout << "warte " << n << "\n";
    std::cout.flush();
#if USE_SLEEP != 0
    sleep(1);
#else
    struct timeval timeout;
    timeout.tv_sec  = 1;
    timeout.tv_usec = 0;
     
    select(FD_SETSIZE, NULL, NULL, NULL, &timeout);
#endif
  }
  
  
  pthread_exit((void*)0);

  //  return(0);
}
