// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treeview.h"

#include "../shared/SimpleXMLTransfer.h"
#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../shared/hirestime.h"
#include "treekwhandler.h"
#include "utf_conv.h"
#include "pic.h"
#include "group.h"
#include "user.h"
#include "xmldefs.h"

#include <iostream>
#include <fstream>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>  /* defines FILENAME_MAX */
#ifdef WIN32
# include <direct.h>
# include <windows.h>
# include <Lmcons.h>
#else
# include <unistd.h>
#endif

#define FILE_VERSION_CURRENT 10

const unsigned int TreeView::uThumbSizes[] = {50, 100, 150, 200, 250, 300, 450, 0};

#include "tv_loadv09.cpp"

TreeView::TreeView()
{
  std::cout << "------------------------------------------------\n";
  std::cout << "TreeView(" << this << ")::TreeView()\n";
  
  kw    = new TreeKwHandler(this);
  perm  = new T_Perm();
  pUser = new User();
    
  // create root and first (visible) child
  pGRoot = new ItGRoot("A", "Root");
  pGRoot->pFirstChild = new ItGData(0, pGRoot);
  pGRoot->pFirstChild->SetName("my gallery");
  pGRoot->pFirstChild->strID = pUser->GetIDGroup();
  vGroups.push_back(pGRoot->pFirstChild);
  mapGroups.insert(std::make_pair(pGRoot->pFirstChild->strID, pGRoot->pFirstChild));
  CreateOrphanedGroup();

  //
  fAddDelPicAllowed = true;

  thumbdir       = eHome; // safe default to not clutter someones data directory
  thumbdirselect = "";
  
  projectfilePath = "";
  strAddPath = "";
  nVersionInfo = FILE_VERSION_CURRENT;
  std::cout << "------------------------------------------------\n";
}


TreeView::~TreeView()                                           
{
  std::cout << "------------------------------------------------\n";
  std::cout << "TreeView(" << this << ")::~TreeView()\n";
  
  delete kw;
  delete perm;
  
  std::cout << "------------------------------------------------\n";  
}

int TreeView::loadFromFile(std::string newProjectfilePath, std::string strCharset,
                           ptrFunUserIDCallback funptr_UserIDCallback)
{
  int retCode;

  if (FileTools::fileExists(newProjectfilePath) == false)
    return(eLE_NOFILE);
  
  std::cout << "------------------------------------------------------------------------------------\n";
  std::cout << "int TreeView::loadFromFile(std::string newProjectfilePath = " << newProjectfilePath << ")\n";
  
  basePaths.clear();
  fAddDelPicAllowed = false;
    
  // Load data into a SimpleXMLTransfer and copy to internal structure afterwards.
  // The internal classes could be enabled to read a file by themselves, but in this
  // case I'd have to maintain more than one method of reading/writing xml.
  std::cout << "Loading " << newProjectfilePath << " into SimpleXMLTransfer\n";
  SimpleXMLTransfer* data;
  try
  {
    HiResTime hrt;
    hrt.Capture();
    
    data = new SimpleXMLTransfer(newProjectfilePath);
    std::cout << "timing.load=" << hrt.GetMSecSinceCapture() << "\n";
  }
  catch (XMLException e)
  {
    std::cerr << "Exception when loading datafile: " << e.what() << "\n";
    return(eLE_XML);
  }
  
  if (data->getInt("version", 2) > FILE_VERSION_CURRENT)
  {
    return(eLE_VERSION);    
  }
    
  perm->SetPermissions(data->getInt("permission.this",    0x7FFFFFFF),
                       data->getInt("permission.save_as", 0x7FFFFFFF));
  
  // 
  if (data->getChild("config")->indexOfChild("thumbdir") >= 0)
  {
    SimpleXMLTransfer* xTD = data->getChild("config.thumbdir");
    
    thumbdir = (eThumbDir)(xTD->attributeAsInt("type"));
    thumbdirselect = xTD->attribute("dir");
  }

  nVersionInfo = data->attributeAsInt("version");
  
  if (data->getInt("version", 2) < 10)
    retCode = Load_v09(newProjectfilePath, data, strCharset);
  else
    retCode = Load_v10(newProjectfilePath, data, funptr_UserIDCallback);
        
  std::cout << "Deleting SimpleXMLTransfer\n";
  delete data;
  
  nVersionInfo = FILE_VERSION_CURRENT;
  
  std::cout << "------------------------------------------------------------------------------------\n";

  projectfilePath = newProjectfilePath;
  
  // GetUserRoot()->DebugShowStructure(0);
  
  if (false)
  {
    // refresh all search groups
    for (unsigned int n=0; n<vGroups.size(); n++)
    {
      ItGSearch* sg = dynamic_cast<ItGSearch*>(vGroups[n]);
      if (sg != 0)
      {
        sg->Refresh(getBasepath());
        sg->DebugShowStructure(0);
      }
    }
  }
  
  return(retCode);
}

SimpleXMLTransfer* TreeView::saveToSXT(std::string newProjectfilePath, int nVersion)
{
  std::cout << "TreeView::saveToSXT()\n"; std::cout.flush();

  SimpleXMLTransfer* project;
  
  project = new SimpleXMLTransfer();
  project->setName("album");
  project->addAttribute("version", nVersion);

  // permission info
  {
    SimpleXMLTransfer* xPerm;
    xPerm = new SimpleXMLTransfer();
    xPerm->setName("permission");
    xPerm->addAttribute("this",    perm->uPermMask);
    xPerm->addAttribute("save_as", perm->uPermMaskSaveAs);
    project->addChild(xPerm);
  }
  
  //
  pUser->WriteBack();
  project->addChild(new SimpleXMLTransfer(pUser->xData));
  if (pUser->GetUserID().length())
  {
    SimpleXMLTransfer* xUser = project->getChild("user.id." + pUser->GetUserID());
      
    xUser->setAttributeOverwrite("basePath", strBasePath);
    
    if (perm->IsPermitted(PERM_OverwriteAlbum))
    {
#ifdef WIN32
      char cCurrentPath[FILENAME_MAX];
      _getcwd(cCurrentPath, sizeof(cCurrentPath) / sizeof(TCHAR));
      // todo: sicherstellen, dass newProjectfilePath ein absoluter Pfad ist
#else
      if (newProjectfilePath[0] != '/')
      {
        char cCurrentPath[FILENAME_MAX];
        getcwd(cCurrentPath, sizeof(cCurrentPath));
        newProjectfilePath = FileTools::createAbsPath(std::string(cCurrentPath) + "/", newProjectfilePath);
      }
#endif
      xUser->setAttributeOverwrite("basePathRel", FileTools::createRelPath(newProjectfilePath, strBasePath));
    }
    else
    {
      xUser->setAttributeOverwrite("basePathRel", strBasepathRelOriginal);
    }
    
    xUser->setAttributeOverwrite("addPath", strAddPath);
  }
  std::cout << "user info written\n"; std::cout.flush();
  
  // config info
  SimpleXMLTransfer* config;
  config = new SimpleXMLTransfer();
  config->setName("config");
  
  //
  {
    config->setAttribute("thumbdir.type", thumbdir);
    config->setAttribute("thumbdir.dir",  thumbdirselect);      
  }
  
  {
    SimpleXMLTransfer* alt;
    SimpleXMLTransfer* abp;
    
    alt = new SimpleXMLTransfer();
    alt->setName("alternativeBasePaths");
    
    for (unsigned int n=0; n<basePaths.size(); n++)
    {
      abp = new SimpleXMLTransfer();
      abp->setName("basePath");
      abp->addAttribute("string", basePaths[n]);
      alt->addChild(abp);
    }
    config->addChild(alt);
  }
  project->addChild(config);
  std::cout << "config info written\n"; std::cout.flush();
  
  // Copy internal data to a SimpleXMLTransfer
  {
    int nUsers = project->getChild("user.id")->getChildCount();
    std::string strCurUser = pUser->GetUserID();
    SimpleXMLTransfer* xUser[nUsers];
    if (strCurUser.length())
      xUser[0] = project->getChild("user.id." + strCurUser);
    else
      xUser[0] = 0;
    
    {
      int nI = 1;
      SimpleXMLTransfer* xU = project->getChild("user.id");
      for (int m=0; m<xU->getChildCount(); m++)
      {
        SimpleXMLTransfer* xR = xU->getChildAt(m);
        if (xR->getName().compare(strCurUser) != 0)
          xUser[nI++] = xR;
      }
    }
    
    SimpleXMLTransfer* xPics = new SimpleXMLTransfer();   
    std::string strPathPrefix = "";
    xPics->setName("pics");
    project->addChild(xPics);
    for (unsigned int n=0; n<vPics.size(); n++)
    {
      vPics[n]->SaveToXML(strPathPrefix, xPics);
      
      std::vector<int>& vRating = vPics[n]->vRating;
      for (unsigned int m=0; m<vRating.size(); m++)
        if (vRating[m] != ItPic::eRatingNone && xUser[m])
        {
          SimpleXMLTransfer* xRate = new SimpleXMLTransfer();
          xRate->setName(vPics[n]->GetID());
          xRate->addAttribute("r", vRating[m]);
          xUser[m]->addChild(xRate);
        }
    }
  }
  std::cout << "ratings written\n"; std::cout.flush();
  
  {
    SimpleXMLTransfer* xData = new SimpleXMLTransfer();
    xData->setName("groups");
    project->addChild(xData);
    for (unsigned int n=0; n<vGroups.size(); n++)
      vGroups[n]->SaveToXML(xData);
  }
  std::cout << "groups written\n"; std::cout.flush();

  return(project);
}

int TreeView::saveAsNewProject(std::string newProjectfilePath)
{
  // change permission if the file is "saved as"
  if (newProjectfilePath.compare(projectfilePath) != 0)
    perm->SetPermissions(perm->uPermMaskSaveAs, perm->uPermMaskSaveAs);
  
  SimpleXMLTransfer* project = saveToSXT(newProjectfilePath, FILE_VERSION_CURRENT);
  nVersionInfo = FILE_VERSION_CURRENT;
  
  HiResTime hrt;
  hrt.Capture();
  // Save to file: not using indentation to make file smaller, which also
  // speeds up loading and saving.
  int nRet = project->SaveToFile(newProjectfilePath, 0, 0, 0);
  delete project;
  if (nRet)
  {
    std::cerr << "int TreeView::saveAsNewProject(std::string newProjectfilePath = " << newProjectfilePath << "): " << nRet << "\n";
    return(-2);
  }
  std::cout << "timing.savetofile=" << hrt.GetMSecSinceCapture() << "\n";
  
  projectfilePath = newProjectfilePath;
  
  return(0);  
}


int TreeView::saveToProjectFile()                               
{
  return(saveAsNewProject(projectfilePath));
}


ItGroup* TreeView::browserItemPrev(ItGroup* p)
{
  ItGroup* prev = p->pPrev;
  
  if (prev == (ItGroup*)0)
  {
    // There is no previous item in this group. The previous item is 
    // the group itself.
    return(p->pParent);
  }
  else
  {
    // Now prev points to a group or to 0.
    if (prev == (ItGroup*)0)
    {
      // There is no previous item in this group. The previous item is 
      // the group itself.
      return(p->pParent);
    }
    
    // Now I have to go down as far as possible, starting from the previous group.
    
    while (1)
    {
      // prev points to a group
      if (prev->fExpanded == false || prev->pFirstChild == 0)
        return(prev);
      else
      {
        // The group is expanded. I need to go down as far as possible and return
        // that item. Because the group is expanded, it contains something to display, 
        // which means I won't reach 0 when going backwards starting from its last child.
        prev = prev->pFirstChild;
        while (prev->pNext != (ItGroup*)0)
          prev = prev->pNext;
        // Now prev points to the last item of the group.        
      }
    }    
  }
}


ItGroup* TreeView::browserItemNext(ItGroup* p)
{
  ItGroup* next;

  if (p == (ItGroup*)0)
    return(p);
  
  if (p->fExpanded == true && p->pFirstChild)
  {
    // The group is expanded. So it does contain at least one item which can be displayed.
    return p->pFirstChild;
  }
  else
  {
    while (1)
    {
      next = p->pNext;
      
      // Now next points to a group or to 0.
      if (next != (ItGroup*)0)
        return(next);
    
      // I reached the end of a group.
      p = p->pParent;
      if (p == (ItGroup*)0)
        return(p);
    }
  }    
}

std::string TreeView::getPicPath(ItPic* item,
                                 int    nSize)               
{
  if (nSize)  
  {
    std::string picPath;
    eThumbDir   th;

    exchangeThumbSettings(th, picPath, false);
    
    switch (th)
    {
     case eSrc:
      picPath = strBasePath + item->GetPath();
      break;
     case eHome:
      picPath = FileTools::getHomeDir("Fotoalbum") + FileTools::stripAbsFromPath(strBasePath) + item->GetPath();
      break;
     default:
      picPath += "/" + FileTools::stripAbsFromPath(strBasePath) + item->GetPath();
      break;
    }
    
    std::string tmp        = picPath.substr(0, picPath.rfind('/')+1);
    std::string sizestring = itoStr(nSize, ' ', 0);
    
    tmp.append(".thumbnails/");
    tmp.append(sizestring);
    tmp.append("x");
    tmp.append(sizestring);
    tmp.append("/");
    tmp.append(picPath.substr(picPath.rfind('/')+1));
      
    // Sicherstellen, dass die Endung auf jpeg-Format deutet.
    std::string::size_type pos = picPath.rfind('.');
    if (pos != std::string::npos)
    {
      std::string endung = strU(picPath.substr(pos+1));
      
      if (endung.compare("JPG") != 0 && endung.compare("JPEG") != 0)
        tmp.append(".jpeg");
    }
    else
      tmp.append(".jpeg");
    
    return(tmp);
  }
  else
    return(strBasePath + item->GetPath());
}

bool TreeView::makeVisible(ItGroup* item)
{
  bool     fTmp  = false;
  ItGroup* group = item->pParent;
  
  while (group != 0)
  {
    if (!group->fExpanded)
    {
      group->fExpanded = true;
      fTmp             = true;
    }
    group = group->pParent;
  }

  return(fTmp);
}

bool TreeView::hasData()                                        
{
  return(
         (pGRoot != (ItGRoot*)0) && (pGRoot->pFirstChild != (ItGRoot*)0)
         );
}

std::string TreeView::resolveExternalLink(std::string linktext,
                                          ItGroup*    pGroup)
{
  if (linktext.find("<FILE=") == std::string::npos)
    return("");

  linktext = linktext.substr(6);
  linktext = linktext.substr(0, linktext.length()-1);
  
  std::cout << "linktext: *" << linktext << "*\n";
    
  return pGroup->FindFile(strBasePath, linktext);
}

std::string TreeView::getProjectfilePath()                      
{
  return(projectfilePath);
}

int  TreeView::addPicsFromDir(const char*            picdir, 
                              ItGroup*               pDst,
                              ItPic::eDateTimeSource eDTS)
{
  int nRetCode = 0;
  
  std::string baseURL = picdir;
  
  if (baseURL.length() == 0)
    baseURL = "/";

  if (baseURL[baseURL.length()-1] != '/')
    baseURL += "/";      

  std::cout << "----------------------------------------------\n";
  std::cout << "Adding pictures from " << baseURL << "\n";

  updateBasepath(baseURL.c_str());

  pDst->addPics(baseURL, strBasePath, eDTS, this);

  return(nRetCode);
}

int  TreeView::addFile(const char*            picdir, 
                       ItGroup*               pDst,
                       ItPic::eDateTimeSource eDTS)
{
  int nRetCode = 0;
  
  std::string baseURL = FileTools::path(picdir);
    
  updateBasepath(baseURL.c_str());

  pDst->addFile(picdir, strBasePath, eDTS, this);
  
  return(nRetCode);
}

int  TreeView::updateBasepath(const char* picdir)                 
{
  std::string baseURL = picdir;
  
  {
    strAddPath = picdir;
    std::string::size_type pos = strAddPath.rfind('/');
    if (pos == strAddPath.length()-1)
      pos = strAddPath.rfind('/', pos-1);
    if (pos != std::string::npos)
      strAddPath = strAddPath.substr(0, pos);
  }
  
  std::cout << "-----------------------------------------------\n";
  std::cout << "basepath old = " << strBasePath << "\n";
    
  // Wenn es strBasePath noch nicht gibt, kann ich ihn mal neu setzen.
  // Wie ich es jetzt auch immer mache, es wird falsch sein. Aber 
  // immerhin ist er mal gesetzt.
  if (basePaths.size() == 0)
  {
    basePaths.push_back(baseURL);
    strBasePath       = baseURL;
    fAddDelPicAllowed = true;
  }
            
  // Wenn der neue Pfad nicht unterhalb von strBasePath liegt, muss ich den
  // anpassen. 
  if (baseURL.find(strBasePath) != 0)
  {
    std::string newbp;
    
    // Maybe baseURL is the new basepath?
    if (strBasePath.find(baseURL) == 0)
    {
      // Yes! Easy...
      newbp = baseURL;
    }
    else
    {
      // Jetzt muss ich die gemeinsame Wurzel nehmen.
      // Um die gemeinsame Wurzel zu finden, schneidet man den einen 
      // so lange ab, bis er im anderen auftaucht:
      baseURL = baseURL.substr(0, baseURL.rfind('/'));      
      while (strBasePath.find(baseURL) == std::string::npos && baseURL.length() > 1)
      {
        baseURL = baseURL.substr(0, baseURL.rfind('/'));
      }
      
      newbp = baseURL + "/";
    }
    
    // Gut. 'baseURL' ist also der neue 'strBasePath'.
    // Die vorhandenen Bilder anpassen:
    for (unsigned int n=0; n<vPics.size(); n++)
      vPics[n]->ChangeBasepath(strBasePath, newbp);
    // Jetzt ändern:
    strBasePath  = newbp;
    basePaths[0] = strBasePath;
  }

  std::cout << "basepath new = " << strBasePath << "\n";
  std::cout << "-----------------------------------------------\n";  
  
  return(0);
}

std::string TreeView::getBasepath()                               
{
  return(strBasePath);
}

bool TreeView::isAddDelAllowed()                                  
{
  return(fAddDelPicAllowed);
}

bool TreeView::dataSaved()
{
  bool fSaved;
  
  HiResTime hrt;
  hrt.Capture();
    
  // Save with current version in order to not apply file format updates (would result
  // in file changes although the user didn't change anything).  
  SimpleXMLTransfer* data_new = saveToSXT(projectfilePath, nVersionInfo);
  
  std::cout << "timing.save=" << hrt.GetMSecSinceCapture() << "\n";
  hrt.Capture();
    
  SimpleXMLTransfer* data_old = new SimpleXMLTransfer(projectfilePath);
  std::cout << "timing.load=" << hrt.GetMSecSinceCapture() << "\n";
  
  // remove variable entries
  {
    SimpleXMLTransfer* tmp;
    int idx = data_new->indexOfChild("permission");
    if (idx >= 0)
    {
      tmp = data_new->getChildAt(idx);
      data_new->removeChildAt(idx);
      delete tmp;
    }
    idx = data_old->indexOfChild("permission");
    if (idx >= 0)
    {
      tmp = data_old->getChildAt(idx);
      data_old->removeChildAt(idx);
      delete tmp;
    }
  }
  
  hrt.Capture();
  fSaved = data_old->equalsOrdered(data_new);
  std::cout << "timing.compare=" << hrt.GetMSecSinceCapture() << "\n";
  
  if (false && !fSaved)
  {
    std::ofstream of;
    of.open((projectfilePath + ".new").c_str());
    data_new->print(of, 0, 0, 0);
    of.close();
  }
  
  delete data_new;
  delete data_old;
  
  return(fSaved);
}

void TreeView::exchangeThumbSettings(eThumbDir&   th,
                                     std::string& tds, 
                                     bool         fWrite) 
{
  if (fWrite)
  {
    thumbdir       = th;
    thumbdirselect = tds;
  }
  else
  {
    th  = thumbdir;
    tds = thumbdirselect;
  }
}

#if 0 == 1
void TreeView::RecursiveList(const std::vector<TreeItem*>& list, std::set<TreeItem*>& set)
{
  for (unsigned int n=0; n<list.size(); n++)
    list[n]->RecursiveList(set);
}
#endif

ItGroup* TreeView::GetUserRoot()
{
  return pGRoot->pFirstChild; 
};

void TreeView::AddPic(ItPic* pPic)
{
  pPic->strID = pUser->GetIDPic();
  vPics.push_back(pPic);
  mapPics.insert(std::make_pair(pPic->strID, pPic));
}

void TreeView::AddGroup(ItGroup* pGroup)
{
  pGroup->strID = pUser->GetIDGroup();
  vGroups.push_back(pGroup);
  mapGroups.insert(std::make_pair(pGroup->strID, pGroup));
  
  ItGSearch* sg = dynamic_cast<ItGSearch*>(pGroup);
  if (sg)
    vSearches.push_back(sg);
}

void TreeView::Delete(ItPic* pP)
{
  for (unsigned int n=0; n<vPics.size(); n++)
    if (vPics[n] == pP)
    {
      vPics.erase(vPics.begin() + n);
      mapPics.erase(pP->strID);
      for (unsigned int n=0; n<vSearches.size(); n++)
        vSearches[n]->Deleted(pP);
      delete pP;
                 
      return;
    }
  std::cerr << "internal error: void TreeView::Delete(ItPic* pP)\n";
}

void TreeView::Delete(ItGroup* pG)
{
  ItGSearch* sg = dynamic_cast<ItGSearch*>(pG);
  if (sg)
  {
    for (unsigned int n=0; n<vSearches.size(); n++)
      if (vSearches[n] == pG)
      {
        vSearches.erase(vSearches.begin() + n);
        break;
      }
  }
  
  for (unsigned int n=0; n<vGroups.size(); n++)
    if (vGroups[n] == pG)
    {
      vGroups.erase(vGroups.begin() + n);
      mapGroups.erase(pG->strID);
      for (unsigned int n=0; n<vSearches.size(); n++)
        vSearches[n]->Deleted(pG);
      delete pG;
            
      return;
    }  
  std::cerr << "internal error: void TreeView::Delete(ItGroup* pG)\n";
}

ItGroup* TreeView::GetGroupPtr(std::string strID)
{
  std::map<std::string, ItGroup*>::iterator itG = mapGroups.find(strID);
  if (itG == mapGroups.end())
    return 0;
  else
    return itG->second;
}
 
ItPic* TreeView::GetPicPtr(std::string strID)
{
  std::map<std::string, ItPic*>::iterator itG = mapPics.find(strID);
  if (itG == mapPics.end())
    return 0;
  else
    return itG->second;
}

ItPicInGroup* TreeView::GetPicInGroupPtr(std::string strID, ItGroup*& pParent)
{
  ItPic* pPic = GetPicPtr(strID);
  
  if (pPic == 0)
    return 0;
  
  pParent = 0;
  ItPicInGroup* p;
  for (unsigned int n=0; n<vGroups.size(); n++)
  {
    p = vGroups[n]->GetPic(pPic);
    if (p)
    {
      pParent = vGroups[n];
      return p;
    }
  }
  
  return 0;
}

void TreeView::CreateOrphanedGroup()
{
  pGOrphanedPics = new ItGRoot("B", "OrphanedPics");
  pGOrphanedPics->pParent = pGRoot;
  pGRoot->pFirstChild->pNext = pGOrphanedPics;
  pGOrphanedPics->pPrev = pGRoot->pFirstChild;
}

int TreeView::Load_v10(std::string newProjectfilePath, SimpleXMLTransfer* data,
                       ptrFunUserIDCallback funptr_UserIDCallback)
{
  int retCode = eLE_NO_ERROR;
  
  pUser->Read(data->getChild("user"));
  if (pUser->GetUserID().length() == 0 && pUser->AllowIDSelect())
    funptr_UserIDCallback((void*)pUser);
  
  try
  {
    // read pictures
    {
      SimpleXMLTransfer* xP = data->getChild("pics");
      unsigned int c = xP->getChildCount();
      std::cout << c << " pictures\n";
      vPics.reserve(c);
      std::string strPre = "";
      for (unsigned int n=0; n<c; n++)
      {
        SimpleXMLTransfer* xI = xP->getChildAt(n);
        if (xI->getName().compare("P") == 0)
          strPre = xI->attribute("p");
        else
        {
          ItPic* p = new ItPic(xI, strPre);
          vPics.push_back(p);
          mapPics.insert(std::make_pair(p->strID, p));
        }
      }
    }

    // delete all of pGRoot's children -- although there should be only one real group
    {
      ItGroup* pToDel = pGRoot->pFirstChild; 
      pGRoot->Unlink(pToDel);
      Delete(pToDel);
      pGRoot->DeleteMyContent();
    }
    
    // read groups
    {
      SimpleXMLTransfer* xP = data->getChild("groups");
      unsigned int c = xP->getChildCount();
      vGroups.reserve(c);
      for (unsigned int n=0; n<c; n++)
      {
        SimpleXMLTransfer* xI = xP->getChildAt(n);
        
        ItGroup* p = ItGroup::Factory(xI);
        if (n == 0)
        {
          p->pParent = pGRoot;
          pGRoot->pFirstChild = p;
        }

        vGroups.push_back(p);
        mapGroups.insert(std::make_pair(p->strID, p));
        {
          ItGSearch* sg = dynamic_cast<ItGSearch*>(p);
          if (sg)
            vSearches.push_back(sg);          
        }
      }
    }
    
    // connect links
    {
      SimpleXMLTransfer* xP = data->getChild("groups");
      unsigned int c = xP->getChildCount();
      for (unsigned int n=0; n<c; n++)
      {
        SimpleXMLTransfer* xI = xP->getChildAt(n);
        
        std::map<std::string, ItGroup*>::iterator itG = mapGroups.find(xI->getName());
        retCode |= itG->second->SetChildren(xI, mapPics, mapGroups);
      }    
    }

    // special groups
    CreateOrphanedGroup();
    
    // fill orphaned group
    for (unsigned int n=0; n<vPics.size(); n++)
      if (vPics[n]->nRefCnt == 0)
        pGOrphanedPics->vPics.push_back(new ItPicInGroup(vPics[n]));

    // ratings
    {
      int nUsers = data->getChild("user.id")->getChildCount();
      std::string strCurUser = pUser->GetUserID();
      SimpleXMLTransfer* xU = data->getChild("user.id");
      
      if (strCurUser.length() && xU->indexOfChild(strCurUser) >= 0)
      {
        SimpleXMLTransfer* xR = xU->getChild(strCurUser);
        for (int n=0; n<xR->getChildCount(); n++)
        {
          SimpleXMLTransfer* xP = xR->getChildAt(n);
          std::map<std::string, ItPic*>::iterator itP = mapPics.find(xP->getName());
          if (itP != mapPics.end())
          {
            itP->second->vRating.resize(nUsers, ItPic::eRatingNone);
            itP->second->vRating[0] = xP->attributeAsInt("r");
          }
        }
      }
      else
        nUsers++;
      
      int nI = 1;
      for (int m=0; m<xU->getChildCount(); m++)
      {
        SimpleXMLTransfer* xR = xU->getChildAt(m);        
        if (xR->getName().compare(strCurUser) != 0)
        {
          for (int n=0; n<xR->getChildCount(); n++)
          {
            SimpleXMLTransfer* xP = xR->getChildAt(n);
            std::map<std::string, ItPic*>::iterator itP = mapPics.find(xP->getName());
            if (itP != mapPics.end())
            {
              if (itP->second->vRating.size() == 0)
                itP->second->vRating.resize(nUsers, ItPic::eRatingNone);
              itP->second->vRating[nI] = xP->attributeAsInt("r");
            }
          }
          nI++;
        }
      }            
    }
    
    // expand album
    pGRoot->pFirstChild->fExpanded = true;
  }
  catch (XMLException e)
  {
    std::cerr << "XMLException: " << e.what() << "\n";
    retCode |= eLE_XML;
  }
  
  std::cout << "Basepath = ";
  {
    std::string basepath;
    SimpleXMLTransfer* xUI = data->getChild("user.id");
                                              
    if (pUser->GetUserID().length() && xUI->indexOfChild(pUser->GetUserID()) >= 0)
    {
      SimpleXMLTransfer* xUser = xUI->getChild(pUser->GetUserID());
      strBasepathRelOriginal = xUser->getString("basePathRel", "");
      strAddPath = xUser->getString("addPath", "");
    }
    else
    {
      strBasepathRelOriginal = "";
      strAddPath = "";
    }
    
    if (data->getChild("config")->indexOfChild("alternativeBasePaths") >= 0)
    {
      SimpleXMLTransfer* group = data->getChild("config.alternativeBasePaths");
      int nSize = group->getChildCount();
      for (int n=0; n<nSize; n++)
      {
        basepath = group->getChildAt(n)->attribute("string");
        basePaths.push_back(basepath);
      }
    }
        
    if (vPics.size() == 0)
    {
      strBasePath       = "";
      fAddDelPicAllowed = true;
    }
    else
    {
      std::string testpath = vPics[0]->GetPath();
      if (FileTools::fileExists(basePaths[0] + testpath))
      {
        strBasePath       = basePaths[0];
        fAddDelPicAllowed = true;
      }
      else
      {
        // relative path?
        strBasePath = FileTools::createAbsPath(FileTools::path(newProjectfilePath),
                                               strBasepathRelOriginal);
        
        if (FileTools::fileExists(strBasePath + testpath) == false)
        {
          strBasePath = "";
          
          for (unsigned int n=1; n<basePaths.size() && strBasePath.length() == 0; n++)
          {
            if (FileTools::fileExists(basePaths[n] + testpath))
              strBasePath = basePaths[n];            
          }
          
          if (strBasePath.length() == 0)
          {
            strBasePath  = basePaths[0];
            std::cout << "file " << testpath << " not found\n";
            retCode  |= (int)(eLE_BASEPATH);
          }
        }  
        else
        {
          // Apparently, album file and data directory have been moved. Add
          // basepath accordingly if it is not already there.
          bool fAlreadyThere = false;
          for (unsigned int n=0; n<basePaths.size(); n++)
          {
            if (basePaths[n].compare(strBasePath) == 0)
              fAlreadyThere = true;
          }
          if (!fAlreadyThere)
            basePaths.push_back(strBasePath);
        }
      }
    }    
    std::cout << strBasePath << "\n";
  }
  
  return retCode;
}

