/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "../shared/textparser.h"
#include "../shared/utf8.h"


// resolve chained links
void resolveChainedLink(intptr_t&           nLinkTo,
                        std::map<int, int>& mapIDOldMultiLink)
{
  std::map<int, int>::iterator itA;
  do
  {
    itA = mapIDOldMultiLink.find(nLinkTo);
    if (itA != mapIDOldMultiLink.end())
      nLinkTo = itA->second;
  }
  while (itA != mapIDOldMultiLink.end());
}

bool TreeView::Load_v09AItem(SimpleXMLTransfer*          xData, 
                             std::map<int, std::string>& mapIDOldNew,
                             std::map<int, int>&         mapIDOldMultiLink)
{
  int idx = xData->indexOfAttribute("u");
  std::string strID;
  
  switch (xData->attribute("a")[0])
  {
    case 'g':
    case 'h':
      strID = pUser->GetIDGroup();
//      xData->print(std::cout);
      xData->addAttribute("UID", strID);
      if (idx >= 0)
        mapIDOldNew.insert(std::make_pair(atoi(xData->attributeVal(idx).c_str()), strID));
      return true;
      
    case 'p':
      strID = pUser->GetIDPic();
  //    xData->print(std::cout);
      xData->addAttribute("UID", strID);
      if (idx >= 0)
        mapIDOldNew.insert(std::make_pair(atoi(xData->attributeVal(idx).c_str()), strID));
      return false;
      
    case 'q':
      if (idx >= 0)
        mapIDOldMultiLink.insert(std::make_pair(atoi(xData->attributeVal(idx).c_str()), 
                                                xData->attributeAsInt("l")));
      return false;
      
    default:
      std::cout << "unhandled type: " << xData->attribute("a")[0] << "\n";
      return false;
  }        
}

void TreeView::Load_v09ARec(SimpleXMLTransfer*          xData, 
                            std::map<int, std::string>& mapIDOldNew,
                            std::map<int, int>&         mapIDOldMultiLink)
{
  int size = xData->getChildCount();
  for (int n=0; n<size; n++)
  {
    SimpleXMLTransfer* xI = xData->getChildAt(n);
    if (Load_v09AItem(xI, mapIDOldNew, mapIDOldMultiLink))
      Load_v09ARec(xI, mapIDOldNew, mapIDOldMultiLink);
  }
}


ItPicInGroup* TreeView::Load_v09BItem(SimpleXMLTransfer*             xData, 
                                      std::map<int, std::string>&    mapIDOldNew,
                                      std::map<int, int>&            mapIDOldMultiLink,
                                      std::map<std::string, ItPic*>& mapPicsPath)
{
  // transform comment links
  {
    int idx = xData->indexOfAttribute("c");
    if (idx >= 0)
    {
      std::string strComment = xData->attributeVal(idx);      
      std::string::size_type p1 = strComment.find("<ID=");
      if (p1 != std::string::npos)
      {
        std::string::size_type p0 = 0;
        std::string::size_type p2 = strComment.find(">", p1+5);
        std::string strOut = "";        
        
        while (p1 != std::string::npos && p2 != std::string::npos)
        {
          strOut.append(strComment.substr(p0, p1-p0+4));
          
          intptr_t nLinkTo = atoi(strComment.substr(p1+4, p2-p1-4).c_str());
          // std::cout << nLinkTo << " ";
          resolveChainedLink(nLinkTo, mapIDOldMultiLink);
          // std::cout << " " << nLinkTo << " ";            
          
          std::map<int, std::string>::iterator it = mapIDOldNew.find(nLinkTo);
          if (it != mapIDOldNew.end())
          {
            strOut.append(it->second);
            // std::cout << " " << it->second << "\n";
          }
          else
          {
            strOut.append(strComment.substr(p1+4, p2-p1-4));
            std::cerr << "Link target not found: " << nLinkTo << "\n";
          }
          
          p0 = p2;
          p1 = strComment.find("<ID=", p0);
          p2 = strComment.find(">", p1+5);
        }
        strOut.append(strComment.substr(p0));
        
        strOut = UTF8::trim(strOut);
        if (false && strOut.compare(strComment) != 0)
          std::cout << strComment << "*\n" << strOut << "*\n";
        xData->overwriteAttribute(idx, strOut);
        // std::cout << "---------------------------\n";
      }
    }
  }
  
  switch (xData->attribute("a")[0])
  {
    case 'g':
      {
        ItGroup* p = new ItGData(xData, 9);
        vGroups.push_back(p);
        mapGroups.insert(std::make_pair(p->strID, p));        
      }
      return 0;
      
    case 'h':
      {
        ItGLinkRef* p = new ItGLinkRef(xData, 9);
        
        intptr_t nLinkTo = xData->attributeAsInt("l");
        resolveChainedLink(nLinkTo, mapIDOldMultiLink);
          
        p->pLink = ((ItGroup*)(nLinkTo)); // fill with valid data later
        vGroups.push_back(p);
        mapGroups.insert(std::make_pair(p->strID, p));
      }
      return 0;
      
    case 'p':
      {
        ItPic* pNewPic = new ItPic(xData);
        mapPicsPath.insert(std::make_pair(pNewPic->GetPath(), pNewPic));
        mapPics.insert(std::make_pair(pNewPic->strID, pNewPic));
        return new ItPicInGroup(pNewPic);
      }
      
    case 'q':
      {
        intptr_t nLinkTo = xData->attributeAsInt("l");
        resolveChainedLink(nLinkTo, mapIDOldMultiLink);
          
        // fill with valid data later
        ItPicInGroup* p = new ItPicInGroup((ItPic*)(nLinkTo));
        p->fMarked = true;
        return p;
      }
      
    default:
      return 0;
  }        
}

void TreeView::Load_v09BRec(SimpleXMLTransfer*             xData, 
                            std::map<int, std::string>&    mapIDOldNew,
                            std::map<int, int>&            mapIDOldMultiLink,
                            std::map<std::string, ItPic*>& mapPicsPath,
                            ItGroup* pParent)
{
  int size = xData->getChildCount();
  for (int n=0; n<size; n++)
  {
    SimpleXMLTransfer* xI = xData->getChildAt(n);
    ItPicInGroup* pNewPic = Load_v09BItem(xI, mapIDOldNew, mapIDOldMultiLink, mapPicsPath);
    if (pNewPic == 0)
    {
      ItGroup* pNewGroup = vGroups[vGroups.size()-1];
      pParent->AddGroup(pNewGroup);
      Load_v09BRec(xI, mapIDOldNew, mapIDOldMultiLink, mapPicsPath, pNewGroup);
    }
    else
      pParent->vPics.push_back(pNewPic);      
  }
}


int TreeView::Load_v09(std::string newProjectfilePath, SimpleXMLTransfer* data, std::string strCharset)
{
  int retCode = eLE_NO_ERROR;
  
  
  if (data->getInt("version", 2) < 4)
  {
    if (strCharset.empty())
    {
      // test whether there are any illegal utf8 characters
      if (!UTFConv::Valid(data))
        return(eLE_NotUTF8);
    }
    else
    {
      if (UTFConv::Convert(data->getChild("data"), strCharset))
        return(eLE_UTFConv);
    }
  }

  // user data has been filled by constructor
  
  try
  {
    // delete all of pGRoot's children -- although there should be only one real group
    Delete(pGRoot->pFirstChild);
    pGRoot->Unlink(pGRoot->pFirstChild);
    pGRoot->DeleteMyContent();
    
    // first run: set IDs and fill conversion maps
    std::map<int, std::string> mapIDOldNew;
    std::map<int, int>         mapIDOldMultiLink;
    SimpleXMLTransfer* xData = data->getChild("data");
    Load_v09AItem(xData, mapIDOldNew, mapIDOldMultiLink); // root group
    Load_v09ARec (xData, mapIDOldNew, mapIDOldMultiLink);
    
   
    // read pictures and groups
    // Read pictures into a second map before appending them to vPics in order
    // to have them sorted by path (results in much smaller data file).
    {
      std::map<std::string, ItPic*> mapPicsPath;
      Load_v09BItem(xData, mapIDOldNew, mapIDOldMultiLink, mapPicsPath); // root group    
      {
        ItGroup* pNewGroup = vGroups[0];
        pNewGroup->pParent  = pGRoot;
        pGRoot->pFirstChild = pNewGroup;
        Load_v09BRec(xData, mapIDOldNew, mapIDOldMultiLink, mapPicsPath, pNewGroup);
      }
      // fill vPics
      for (std::map<std::string, ItPic*>::iterator it = mapPicsPath.begin(); it != mapPicsPath.end(); it++)
        vPics.push_back(it->second);
    }
    
    // connect links
    {
      for (unsigned int n=0; n<vGroups.size(); n++)
      {
        ItGroup* pG = vGroups[n];
        ItGLinkRef* pLink = dynamic_cast<ItGLinkRef*>(pG);
        if (pLink)
        {
          std::map<int, std::string>::iterator it = mapIDOldNew.find((intptr_t)(pLink->pLink));
          if (it == mapIDOldNew.end())
            throw XMLException("no item with id=" + itoStr((intptr_t)(pLink->pLink), ' ', 1));
          
          std::string strID = it->second;
          std::map<std::string, ItGroup*>::iterator itG = mapGroups.find(strID);
          if (itG == mapGroups.end())
            throw XMLException("no group with id=" + strID);
          else
            pLink->pLink = itG->second;
        }
        
        for (unsigned int m=0; m<pG->vPics.size(); m++)
        {
          if (pG->vPics[m]->fMarked)
          {
            intptr_t nID = (intptr_t)(pG->vPics[m]->pPic);
            std::map<int, std::string>::iterator it = mapIDOldNew.find(nID);
            if (it == mapIDOldNew.end())
              throw XMLException("no item with id=" + itoStr(nID, ' ', 1));
            
            std::string strID = it->second;
            std::map<std::string, ItPic*>::iterator itG = mapPics.find(strID);
            if (itG == mapPics.end())
              throw XMLException("no pic with id=" + strID);
            else
              pG->vPics[m]->pPic = itG->second;
            pG->vPics[m]->fMarked = false;
          }
          pG->vPics[m]->pPic->IncRefCnt();
        }
      }
    }
        
    // special groups
    CreateOrphanedGroup();
    
    // fill orphaned group: only to spot errors; there must not be orphaned pics
    for (unsigned int n=0; n<vPics.size(); n++)
      if (vPics[n]->nRefCnt == 0)
        pGOrphanedPics->vPics.push_back(new ItPicInGroup(vPics[n]));
    
    // expand album
    pGRoot->pFirstChild->fExpanded = true;
  }
  catch (XMLException e)
  {
    std::cerr << "XMLException: " << e.what() << "\n";
    retCode |= eLE_XML;
  }
  
  std::cout << "Basepath?\n";
  {
    std::string  basepath;

    basepath = data->getString("config.basePath");
    basePaths.push_back(basepath);
    
    if (data->getChild("config")->indexOfChild("alternativeBasePaths") >= 0)
    {
      SimpleXMLTransfer* group = data->getChild("config.alternativeBasePaths");      
      int nSize = group->getChildCount();
      for (int n=0; n<nSize; n++)
      {
        basepath = group->getChildAt(n)->attribute("string");
        basePaths.push_back(basepath);
      }
    }
    
    if (vPics.size() == 0)
    {
      strBasePath       = basePaths[0];
      fAddDelPicAllowed = true;
    }
    else
    {
      std::string testpath = vPics[0]->GetPath();
      if (FileTools::fileExists(basePaths[0] + testpath))
      {
        strBasePath       = basePaths[0];
        fAddDelPicAllowed = true;
      }
      else
      {
        // relative path?
        strBasePath = FileTools::createAbsPath(FileTools::path(newProjectfilePath),
                                               strBasepathRelOriginal);
        
        if (FileTools::fileExists(strBasePath + testpath) == false)
        {
          strBasePath = "";
          
          for (unsigned int n=1; n<basePaths.size() && strBasePath.length() == 0; n++)
          {
            if (FileTools::fileExists(basePaths[n] + testpath))
              strBasePath = basePaths[n];            
          }
          
          if (strBasePath.length() == 0)
          {
            strBasePath  = basePaths[0];
            retCode  |= (int)(eLE_BASEPATH);
          }
        }  
        else
        {
          // Apparently, album file and data directory have been moved. Update
          // primary basepath accordingly.
          basePaths[0] = strBasePath;
        }
      }
    }    
  }

  strAddPath = data->getString("config.addPath", strBasePath);

  return retCode;
}
