/*
 * Copyright (C) 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "user.h"

#ifdef WIN32
#include <windows.h>
#include <Lmcons.h>
#endif

#include <iostream>
#include <set>
#include <stdlib.h>
#include <unistd.h>
#include "../shared/SimpleXMLTransfer.h"
#include "../shared/lib_conversions.h"
 
User::User()
{
#ifdef WIN32
  DWORD size = UNLEN+1;
  TCHAR name[size];
  if (GetUserName((TCHAR*)name, &size))
    strName = name;
  else
    strName = "win32_unknown";
  char host[MAX_COMPUTERNAME_LENGTH+1];
  size = sizeof(host);
  if (GetComputerName(host, &size))
    strHost = host;
  else
    strHost = "win32_unknown";
#else
  strName = getenv("USER");
  {
    char hostname[1024];
    hostname[1023] = '\0';
    gethostname(hostname, 1023);
    strHost = hostname;
  }  
#endif
  std::cout << "user@host: " << strName << "@" << strHost << "\n";
  
  strID = "a";
  
  xData = new SimpleXMLTransfer();
  xData->setName("user");
  xData->setAttribute("allowIDSelect", "1");
  xData->setAttribute("id." + strID + ".max_group", "0");
  xData->setAttribute("id." + strID + ".max_pic",   "0");
  
  SimpleXMLTransfer* xU = new SimpleXMLTransfer();
  xU->setName("user");  
  xData->addChild(xU);
  AddUser();
  xData->setAttribute("user.user.id", strID);
  
  pic = group = 0;
}

int User::Read(SimpleXMLTransfer* xUD)
{
  strID = "";
  
  // copy data
  if (xData)
    delete xData;
  xData = new SimpleXMLTransfer(xUD);
  
  // delete rating information
  SimpleXMLTransfer* xIds = xData->getChild("id");
  for (int n=0; n<xIds->getChildCount(); n++)
  {
    SimpleXMLTransfer* xId = xIds->getChildAt(n);
    while (xId->getChildCount())
    {
      SimpleXMLTransfer* xDel = xId->getChildAt(0);
      xId->removeChildAt(0);
      delete xDel;
    }  
  }


  SimpleXMLTransfer* xUser = GetThisUsersEntry();
  bool fUserFound = xUser != 0;
  if (fUserFound)
    strID = xUser->attribute("id", "");
  
  if (strID.length())
  {
    SimpleXMLTransfer* xUI = xData->getChild("id");
    if (xUI->indexOfChild(strID) < 0)
    {
      xUser = new SimpleXMLTransfer();
      xUser->setName(strID);
      xUI->addChild(xUser);
    }
    else
      xUser = xUI->getChild(strID);
    group = xUser->attributeAsInt("max_group", 0);
    pic   = xUser->attributeAsInt("max_pic",   0);
    std::cout << "user@host: " << strName << "@" << strHost << ", " << strID << ", " << group << ", " << pic << "\n";
    return 0;
  }
  
  if (!fUserFound)
  {
    AddUser();
    return -1;
  }
  
  return -2;
}

SimpleXMLTransfer* User::GetThisUsersEntry()
{
  // find user@host
  SimpleXMLTransfer* xU = xData->getChild("user");
  SimpleXMLTransfer* xUser = 0;
  for (int n=0; n<xU->getChildCount(); n++)
  {
    xUser = xU->getChildAt(n);
    if (xUser->attribute("name").compare(strName) == 0 &&
        xUser->attribute("host").compare(strHost) == 0)
      return xUser;
  }
  return 0;
}

std::string User::GetStrID(int tmp, bool fGroup)
{
  std::string id;
  
  if (fGroup)
    id = strU(strID);
  else
    id = strID;

  int stellen = (int)(ceil(log(tmp) / log(26.0)))+1;
  for (int stelle=stellen-1; stelle>=0; stelle--)
  {
    int wertigkeit = 1;
    for (int n=0; n<stelle; n++)
      wertigkeit *= 26;
    
    int hier = tmp / wertigkeit;
    
    // die führende Null wird nicht dargestellt
    if (hier != 0 || stelle != stellen-1)
    {
      tmp -= hier * wertigkeit;
      char bla = 'A' + hier;
      if (fGroup)
        bla = 'a' + hier;
      id.push_back(bla);
    }
  }
  
  return id;
}

std::string User::GetIDPic()
{
  return GetStrID(++pic, false);
}

std::string User::GetIDGroup()
{
  return GetStrID(++group, true);
}

void User::WriteBack()
{
  if (strID.length())
  {
    SimpleXMLTransfer* xID = xData->getChild("id." + strID);
    xID->setAttributeOverwrite("max_group", group);
    xID->setAttributeOverwrite("max_pic",   pic);
  }
}

void User::AddUser()
{
  // Add new user's data
  SimpleXMLTransfer* xU = xData->getChild("user");
  SimpleXMLTransfer* xUser = new SimpleXMLTransfer();
  xUser->setName("user");
  xUser->addAttribute("name", strName);
  xUser->addAttribute("host", strHost);
  xU->addChild(xUser);
}

bool User::AllowIDSelect()
{
  return (xData != 0) && (xData->attributeAsInt("allowIDSelect", 1) > 0);
}

std::vector<std::string> User::GetUserList()
{
  std::vector<std::string> vList;
  std::set<char> setIDs;
  
  try
  {
    SimpleXMLTransfer* xU = xData->getChild("user");
    for (int n=0; n<xU->getChildCount(); n++)
    {
      SimpleXMLTransfer* xI = xU->getChildAt(n);
      int idx = xI->indexOfAttribute("id");
      if (idx >= 0)
      {
        std::string strID = xI->attributeVal(idx);
        if (strID.length() > 0)
        {
          vList.push_back(strID.substr(0, 1) + ": " + xI->attribute("name") + " at " + xI->attribute("host"));
          setIDs.insert(strID[0]);
        }
      }
    }
    // find unused ids
    for (char cID='a'; cID<='z'; cID++)
    {
      if (setIDs.find(cID) == setIDs.end())
      {
        std::string strTmp;
        strTmp.push_back(cID);
        vList.push_back(strTmp + std::string(": unused"));
      }
    }
  }
  catch (XMLException e)
  {
    std::cerr << e.what() << "\n";
  }
  
  return vList;
}

void User::SetUserID(std::string strNewID)
{
  if (strID.length() == 0)
  {
    strID = strNewID;
    SimpleXMLTransfer* xUser = GetThisUsersEntry();
    xUser->addAttribute("id", strID);
    
    xUser = xData->getChild("id");
    int idx = xUser->indexOfChild(strID);
    if (idx < 0)
    {
      xData->setAttribute("id." + strID + ".max_group", "0");
      xData->setAttribute("id." + strID + ".max_pic",   "0");
    }
    else
    {
      xUser = xUser->getChildAt(idx);
      group = xUser->attributeAsInt("max_group", 0);
      pic   = xUser->attributeAsInt("max_pic",   0);
    }
  }
}
