/*
 * Copyright (C) 2011-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 * 
 */

#include <iostream>
#include <fstream>
#include <set>
#include "../base/treeview.h"
#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"


int recurse(TreeView* tree, 
            TreeGroupBase* group,
            std::set<std::string>& setFiles,
            std::set<std::string>& setFound)
{
  TreeItem* item = group->itemFirstChild;
  
  while (item != 0)
  {
    if (item->getType() == TreeItem::PIC && item->getLinkID() == 0)
    {
      TreePicData* tpd = dynamic_cast<TreePicData*>(item);
      std::string strFile = FileTools::name(tpd->getPath());
      
      std::set<std::string>::iterator it = setFound.find(strFile);
      if (it != setFound.end())
      {
        std::cout << "Kommt mehrfach vor: " << strFile << "\n";
      }
      
      it = setFiles.find(strFile);
      if (it != setFiles.end())
      {
        if (tpd->getID() == 0)
        {
          tpd->setID(tree->getIDMax()+1);
          tree->newLinkableItem(tpd);
        }
        
        // std::cout << strFile << ": " << tpd->getID() << "\n";
        std::cout << "<i a=\"q\" l=\"" << tpd->getID() << "\" />\n";
        
        setFound.insert(strFile);
        setFiles.erase(it);
      }
      // 
      
      
      // getID()
      // setID()
    }  
    else if (item->getType() == TreeItem::GROUP)
    {
      recurse(tree, dynamic_cast<TreeGroupBase*>(item), setFiles, setFound);
    }
    
    //
    item = item->itemNext;
  }
  return(0);
}

int main(int argc, char* argv[])
{
  if (argc != 4)
  {
    std::cerr << "usage:\n" << argv[0] << " ALBUMFILE_IN FILE_WITH_FILENAMES ALBUMFILE_OUT\n";
    std::cerr << "\n";
    std::cerr << "FILE_WITH_FILENAMES lists filenames which are searched for in ALBUMFILE_IN.\n";
    std::cerr << "In case one is found, it is given an ID and the code for a link to this\n";
    std::cerr << "file is written to standard output.\n";
    return -1;
  }
  
  try 
  {
    TreeView* myData = new TreeView(0);
    myData->loadFromFile(argv[1], "");
    
    std::ifstream ifList;
    ifList.open(argv[2]);
    
    std::cout << myData->getIDMax() << "\n";
    
    std::set<std::string> setFiles;
    std::set<std::string> setFound;
    while (!ifList.eof())
    {
      char aLine[500];
      ifList.getline(aLine, 499);
      std::string strLine = trim(aLine);
      if (strLine.length() > 0)
      {
        // std::cout << strLine << "\n";
        setFiles.insert(strLine);
      }
    }
    ifList.close();

    // ich nehme an, dass das Album erheblich umfangreicher ist als die Liste
    // der zu verlinkenden Dateien. Daher bietet es sich an, einmal durch das 
    // Album zu laufen und bei jedem Bild zu schauen, ob es in der Liste vorkommt,
    // und nicht umgekehrt.
    recurse(myData, myData->root, setFiles, setFound);
    
    if (setFiles.size())
    {
      std::cout << "Nicht gefunden wurden:\n";
      for (std::set<std::string>::iterator it = setFiles.begin(); it!=setFiles.end(); it++)
        std::cout << "  " << *it << "\n";
    }
    
    myData->saveAsNewProject(argv[3]);
            
    delete myData;
  }
  catch (XMLException e)
  {
    std::cerr << "xmlexception: " << e.what() << "\n";
    return(-1);
  }
    
  return(0);
}

