/*
 * Copyright (C) 2008-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 * 
 * As far as this code is concerned, 'new style' looks like
 *    toplevel.sub.key2 top2.bla.foo.bar p.uiea.eeei
 * which means: levels separated by '.' and keywords separated by ' '.
 */

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "../shared/SimpleXMLTransfer.h"
#include "../shared/lib_conversions.h"

void convert(SimpleXMLTransfer* gr, char act, std::string from, std::string to)
{
  SimpleXMLTransfer* it;
  std::string        kw;
  std::string        out = "";
  int                idx;
  int count = gr->getChildCount();  
  
  for (int n=0; n<count; n++)
  {
    it = gr->getChildAt(n);
    
    if (it->getChildCount())
      convert(it, act, from, to);
    
    idx = it->indexOfAttribute("k");
    if (idx >= 0)
    {
      kw = trim(it->attributeVal(idx));
      if (kw.length())
      {
        std::string::size_type pos, l;
        std::string::size_type start = 0;
        std::string token;
        bool fMore = true;
        
        out = "";
        
        if (act == 'n')
        {
          kw = trim(collapse_space(kw));
          pos = kw.find('{');
          std::string::size_type end;
          
          while (pos != std::string::npos)
          {
            end = kw.find('}', pos+1);
            if (end != std::string::npos)
            {
              std::string newstyle;
              newstyle = kw.substr(pos-1, end-pos+1);
              newstyle[1] = '.';
              fMore = false;
              kw.replace(pos-1, end-pos+2, newstyle);
            }
            pos = kw.find('{');
          }  
          out = kw;
          /*
          if (!fMore)
           std::cout << " -> " << out << "\n";
           */
        }
        else if (act == 'c')
        {
          out = kw;
          out = trim(collapse_space(out));
          pos = out.find(',');
          start = 0;
          
          while (pos != std::string::npos)
          {
            l = out.find(' ', start);
            if (l != std::string::npos && l < pos)
            {
              // don't know how to handle that...
              std::cerr << "Don't know how to handle \"" << kw << "\"\n";
              out = "";
              break;
            }
            out[pos] = ' ';
            while (pos < out.length()-1 && isspace(out[pos]))
              pos++;
            start = pos+1;
            
            pos = out.find(',', start);
          }  
          /*
          if (out.compare(kw) != 0 && out.length())
           std::cerr << kw << " -> " << out << "\n";
           */
        }
        else
        {
          while (fMore)
          {
            pos = kw.find(' ', start);
            if (pos == std::string::npos)
            {
              fMore = false;
              token = kw.substr(start);
            }
            else
            {
              l = pos-start;
              if (l == 0)
                token = "";
              else
                token = kw.substr(start, l);
              start = pos+1;
            }
            
            // std::cout << "  token: " << token << "\n";
            
            token = trim(token);
            if (token.length())
            {
              switch (act)
              {
                case 'd':
                  if (token.compare(from) != 0)
                    out += token;
                  break;
                  
                case 'r':
                  if (token.compare(from) == 0)
                    out += to;
                  else
                    out += token;
                  break;
                  
                case 'p':
                  if (token.find('.') == std::string::npos)
                    out += from + "." + token;
                  else
                    out += token;
                  break;
                  
                case 'l':
                    out += strL(token);
                  break;
                  
                default:
                  std::cerr << "unknown option: -" << act << "\n";
                  exit(-1);
              }
              
              out += " ";
            }
          }        
        }
        
        if (out.length())
          it->overwriteAttribute(idx, trim(collapse_space(out)));
      }
    }
  }
}

int main(int argc, char* argv[])
{
  SimpleXMLTransfer* df;
  std::string from, to;

  // lib_conversions_tests();
  
  if (argc < 3 || argv[1][0] != '-' || strlen(argv[1]) < 2)
  {
    std::cout << "options:\n"
      << "   -n                        convert old to new style\n"
      << "   -c                        comma separated to new style\n"
      << "   -l                        to lowercase\n"
      << "   -d STRING                 delete keyword\n"
      << "   -p PREFIX                 use prefix for every keyword which has no prefix\n"
      << "   -r FROMSTRING TOSTRING    replace keyword\n";
    return(-1);
  }
  
  from = argv[2];
  if (argc > 3)
    to = argv[3];
  
  try 
  {
    df = new SimpleXMLTransfer(argv[argc-1]);
  }
  catch (XMLException e)
  {
    std::cerr << "Error loading datafile " << argv[argc-1] << ": " << e.what() << "\n";
    return(-1);
  }
  
  convert(df->getChild("data"), argv[1][1], from, to);

  df->print(std::cout, 0, 0);
  
  delete df;
  
  return(0);
}

