/*
 * Copyright (C) 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 * 
 * As far as this code is concerned, 'new style' looks like
 *    toplevel.sub.key2,top2.bla.foo.bar,p.uiea.eeei
 * which means: levels separated by '.' and keywords separated by ','.
 * So keywords can contain spaces again.
 * 
 * Old style is similar, but uses ' ' to separate levels (keywords must
 * not contain space).
 */

#include <iostream>
#include <cstring>
#include "../shared/SimpleXMLTransfer.h"
#include "../shared/lib_conversions.h"

void convert(SimpleXMLTransfer* gr, char act, std::string from, std::string to)
{
  SimpleXMLTransfer* it;
  std::string        kw;
  int                idx;
  int count = gr->getChildCount();  
  
  for (int n=0; n<count; n++)
  {
    it = gr->getChildAt(n);
    
    if (it->getChildCount())
      convert(it, act, from, to);
    
    idx = it->indexOfAttribute("k");
    if (idx >= 0)
    {
      kw = trim(collapse_space(it->attributeVal(idx)));
      if (kw.length())
      {
        std::string::size_type c;
        
        for (c=0; c<kw.length(); c++)
          if (kw[c] == ' ')
            kw[c] = ',';
                
        it->overwriteAttribute(idx, kw);
      }
    }
  }
}

int main(int argc, char* argv[])
{
  SimpleXMLTransfer* df;
  std::string from, to;

  // lib_conversions_tests();
  
  if (argc < 3 || argv[1][0] != '-' || strlen(argv[1]) < 2)
  {
    std::cout << "options:\n"
      << "   -n                        convert old to new style\n";
    return(-1);
  }
  
  from = argv[2];
  if (argc > 3)
    to = argv[3];
  
  try 
  {
    df = new SimpleXMLTransfer(argv[argc-1]);
  }
  catch (XMLException e)
  {
    std::cerr << "Error loading datafile " << argv[argc-1] << ": " << e.what() << "\n";
    return(-1);
  }
  
  convert(df->getChild("data"), argv[1][1], from, to);

  df->print(std::cout, 0, 0);
  
  delete df;
  
  return(0);
}

