/*
 * Copyright (C) 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include <stdio.h>
#include <FL/Fl_Help_Dialog.H>
#include "addkwdlg.h"
#include "texttreeitem.h"
#include "misc.h"
#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"

void cb_browser(TextTreeBrowser* sender, AddKwDlg* searchdlg)
{
  searchdlg->AddKw(sender->selected_path.c_str());
}

void AddKwDlg::AddKw(const char* item)
{
  std::string itemstr = UTF8::trim(UTF8::collapse_space(item));
  if (itemstr.length())
  {
    std::string buf = UTF8::trim(UTF8::collapse_space(ti_words->value()));
    
    if (buf.length())
      buf += ",";
    buf += item;
    ti_words->value(buf.c_str());
  }
}

AddKwDlg::AddKwDlg(std::set<std::string>& kwset)
{
  TextTreeItem* root = TextTreeItem::CreateFromSet(kwset);
  
  { Fl_Double_Window* o = myWindow = new Fl_Double_Window(785, 550, "Add Keyword(s)");
    o->user_data((void*)(this));    
    { Fl_Input* o = ti_words = new Fl_Input(15, 30, 755, 25, "Keyword(s) to add (separated by \',\'):");
      o->tooltip("Enter \'..\' to complete token shown in browser");
      o->align(FL_ALIGN_TOP);
    }
    btn_cancel = new Fl_Button(615, 500, 155, 35, "Cancel");
    btn_add = new Fl_Return_Button(615, 455, 155, 35, "Add");
    { TextTreeBrowser* o = brw_kws = new TextTreeBrowser(15, 65, 585, 470, "", root);
      o->box(FL_NO_BOX);
      o->color(FL_BACKGROUND2_COLOR);
      o->selection_color(FL_SELECTION_COLOR);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->callback((Fl_Callback*)cb_browser);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_RELEASE_ALWAYS);
      Fl_Group::current()->resizable(o);
    }
    //btn_helphier = new Fl_Button(615, 230, 155, 100, "Help: use\nhierarchical\nkeywords");
    o->end();
  }
  
  ti_words->when(FL_WHEN_CHANGED);
  ti_words->callback((Fl_Callback*)callback_text_input_to_texttreebrowser_comma);
  ti_words->user_data((void*)brw_kws);
  brw_kws->user_data(this);
  brw_kws->textsize(14);
}

AddKwDlg::~AddKwDlg()
{
  Fl::delete_widget(myWindow);
}

int AddKwDlg::showDlg()
{
  int nRetCode;
  
  myWindow->set_modal();
  myWindow->show();
      
  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();
    
    if (!o) 
    {
      Fl::wait();
    }
    else if (o == btn_cancel || o == myWindow)
    {
      nRetCode = 0;
      break;
    }
    else if (o == btn_add)
    {
      nRetCode = 1;
      break;
    }
    /*
    else if (o == btn_helphier)
    {
      Fl_Help_Dialog* dlg = new Fl_Help_Dialog();
      
      dlg->load(GUIMisc::getHelpPath("", "fa_ht_search.htm#keyword1").c_str());
      dlg->show();
    }*/
  }
  
  myWindow->hide();
  return(nRetCode);
}
