// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2006  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "albumoptdlg.h"
#include "../base/treeview.h"

#include <iostream>
# include <FL/Fl_File_Chooser.H>


void cb_albset_dir(Fl_Widget* widget, void* userdata)     /*{{{*/
{
  AlbumOptDlg* dlg = (AlbumOptDlg*)userdata;

  char* destptr;
  
  destptr = fl_dir_chooser("Thumbnail directory", 0);
  
  if (destptr != (char*)0)
    dlg->td_dir->value(destptr);
}
/*}}}*/



AlbumOptDlg::AlbumOptDlg(TreeView* tv)
{
  myData = tv;
  
  myWindow = new Fl_Double_Window(390, 250, "Album Options");

  { Fl_Group* o = new Fl_Group(10, 10, 370, 160, "Thumbnail files in:");
    o->box(FL_ENGRAVED_FRAME);
    o->align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
    { Fl_Round_Button* o = rb_src = new Fl_Round_Button(35, 35, 315, 25, "Source directory");
      o->type(102);
      o->down_box(FL_ROUND_DOWN_BOX);
    }
    { Fl_Round_Button* o = rb_home = new Fl_Round_Button(35, 65, 310, 25, "$HOME/.Fotoalbum/");
      o->type(102);
      o->down_box(FL_ROUND_DOWN_BOX);
    }
    { Fl_Round_Button* o = rb_select = new Fl_Round_Button(35, 95, 270, 25, "Select directory:");
      o->type(102);
      o->down_box(FL_ROUND_DOWN_BOX);
    }
    btn_select = new Fl_Button(50, 120, 85, 35, "Select...");
    { Fl_Input* o = td_dir = new Fl_Input(145, 125, 225, 25);
      o->deactivate();
    }
    o->end();
  }
  btn_cancel = new Fl_Button(60, 190, 120, 40, "Cancel");
  btn_ok = new Fl_Return_Button(215, 190, 120, 40, "OK");

  myWindow->end();
  myWindow->user_data((void*)(this));

  btn_select->callback( (Fl_Callback*)cb_albset_dir, (void*)this );
  
  {
    TreeView::eThumbDir td;
    std::string         tds;
    myData->exchangeThumbSettings(td, tds, false);
    
    switch (td)
    {
     case TreeView::eSrc:
      rb_src->setonly();
      break;
     case TreeView::eHome:
      rb_home->setonly();
      break;
     default:
      rb_select->setonly();
      break;
    }
    td_dir->value(tds.c_str());
    td_dir->readonly(1);    
  }
}

AlbumOptDlg::~AlbumOptDlg()
{
  delete myWindow;
}

int AlbumOptDlg::showDlg()
{
  int nRetCode;

  myWindow->set_modal();
  myWindow->show();

  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();

    if (!o)
    {
      Fl::wait();
    }
    else if (o == btn_cancel || o == myWindow)
    {
      std::cout << "cancel\n";
      nRetCode = 0;
      break;
    }
    else if (o == btn_ok)
    {
      nRetCode = 1;
      break;
    }
  }

  if (nRetCode)
  {
    TreeView::eThumbDir td;
    std::string         tds;
    
    if (rb_src->value())
      td = TreeView::eSrc;
    else if (rb_home->value())
      td = TreeView::eHome;
    else
      td = TreeView::eSelect;    
    tds = td_dir->value();
    
    std::cout << "New settings: " << td << ", " << tds << "\n";
    
    myData->exchangeThumbSettings(td, tds, true);
  }
    
  myWindow->hide();
  return(nRetCode);
}
