// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2009, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "../../version.h"
#include "../config.h"
#include "../shared/platform.h"

UserInterface* createNewWin()                             
{
  UserInterface* GUI;
  
  // Load file into a new window.
  char** a;
  a    = (char**)malloc(sizeof(char*)*1);
  char   name[] = TITLE_START;
  a[0] = (char*)(&name);
    
  GUI = new UserInterface();
  Fl_Window* window = GUI->make_window(1, a);        
  window->show(1, a);
  GUI->AfterCreation();
  
  return(GUI);
}


UserInterface* getUIByMenuBar(Fl_Widget* widget)          
{
  UserInterface* GUI = (UserInterface*)0;
  
  // Von welchem Fenster kommt die Anforderung?
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->menuBar == widget)
      GUI = UserInterface::windowList[n];
  
  return(GUI);
}


void cb_ende(Fl_Widget* widget, void* gui)                
{
  UserInterface* GUI = (UserInterface*)gui;

  // don't close on escape key
  if (Fl::event_key() != FL_Escape && GUI->askBeforeClosing() == 1)
  {
    GUI->saveOptions();
    GUI->window->hide();
    
    bool fVis = false;
    for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    {
      if (UserInterface::windowList[n]->window->visible())
        fVis = true;
    }
    if (!fVis && initHelpDlg != (Fl_Help_Dialog*)0)
      initHelpDlg->hide();
  }
}


void cb_new(Fl_Widget*, void* gui)                        
{
  createNewWin();
}


void cb_View(Fl_Widget* sender, void* action)             
{
  UserInterface* GUI = getUIByMenuBar(sender);
    
  //
  std::cout << "void cb_View(Fl_Widget* sender, void* action = " << (intptr_t)action << ")\n";
  
  switch ((intptr_t)action)
  {
   case UserInterface::FULLSCREEN_PICS:
    GUI->setPicFullscreen();
    break;
      
   case UserInterface::FULLSCREEN:
    GUI->setFullscreen();
    break;
  }
}

void cb_picbox(Fl_Widget* sender, void* nAktion)          
{
  // which GUI does the sender belong to?
  UserInterface* GUI = (UserInterface*)0;
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->mainpane_pic == sender)
      GUI = UserInterface::windowList[n];
  
  PicBox*       pPicBox    = (PicBox*)sender;
  ItPicInGroup* currentPic = pPicBox->getPic();
  intptr_t      aktion     = (intptr_t)nAktion;
  
  switch (aktion)
  {
    case PicBox::PROPERTIES:
      GUI->setLastDisplayedAsLastPopupSingle();
      GUI->popupActionPic(PA_PROPERTIES);
      break;
    case PicBox::ADD_KEYWORDS:
      GUI->setLastDisplayedAsLastPopupSingle();
      GUI->popupActionPic(PA_ADD_KEYWORD);
      break;
    case PicBox::CONTEXT:
      std::cout << "callback: picbox context\n";
      GUI->showContextMenuPic(pPicBox->nMX, pPicBox->nMY, currentPic);
      break;
    case PicBox::GROUP:
      GUI->displayGroup(GUI->GetLastDisplayedGroup());
      break;
    case PicBox::PICTURE_PREV:
      {
        ItPicInGroup* p = pPicBox->PicPrev();
        if (p)
          GUI->displayItem(p);
        else
          GUI->displayGroup(GUI->GetLastDisplayedGroup());
      }
      break;
    case PicBox::PICTURE_NEXT:
      {
        ItPicInGroup* p = pPicBox->PicNext();
        if (p)
          GUI->displayItem(p);
        else
          GUI->displayGroup(GUI->GetLastDisplayedGroup());
      }
      break;
    case PicBox::TOGGLE_MARK:
      {
        GUI->toggleMarkCurrentPic();
      }
      break;
      
    default:
      {
        int nAktion = (int)(aktion);
        if (nAktion >= PicBox::RATE)
        {          
          GUI->setLastDisplayedAsLastPopupSingle();
          GUI->popupActionPic(PA_RATE + nAktion-(int)(PicBox::RATE));
        }
      }
      break;
  }
}

void cb_picbutton(Fl_Widget* sender, void* nAktion)
{
  PicButton*    pPicButton = dynamic_cast<PicButton*>(sender);
  ItPicInGroup* pPic = pPicButton->myPic;
  
  // which GUI does the sender belong to?
  UserInterface* GUI = 0;
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->ThisButtonIsMine(pPicButton))
      GUI = UserInterface::windowList[n];
  
  intptr_t aktion = (intptr_t)nAktion;
  switch (aktion)
  {
    case PicButton::CONTEXT:
      GUI->showContextMenuPic(pPicButton->nMX, pPicButton->nMY, pPic);
      break;
    case PicButton::SHOW:
      GUI->displayItem(pPic, true, true, pPicButton->nMX, pPicButton->nMY);
      break;
  }  
}


void cb_showGroup(Fl_Widget* sender, void* gui)            
{
  UserInterface* GUI = (UserInterface*)gui;  
  GUI->displayGroup(GUI->browser->itemSelected);
}


void cb_textbox(Fl_Widget* sender, void* gui)             
{
  UserInterface* GUI = (UserInterface*)gui;
  TextBox*       tb  = (TextBox*)sender;
  
  GUI->followLink(tb->getLinkText());
}


void loadfile(UserInterface* GUI,
              const char*    filename)                    
{
  if (GUI->hasData())
  {
    GUI = createNewWin();
  }      
  
  GUI->loadFile(filename);  
}


void cb_loadfile(Fl_Widget*, void* gui)                   
{
  UserInterface* GUI = (UserInterface*)gui;
  
  char*        newfile;
  std::string  pathToStartFrom;
    
  pathToStartFrom = GUI->myFilename;
  while (pathToStartFrom.length() > 0 && pathToStartFrom[pathToStartFrom.length()-1] != '/')
    pathToStartFrom.replace(pathToStartFrom.length()-1, 1, "");
  
  newfile = fl_file_chooser("Open file", "*.pg.xml", pathToStartFrom.c_str());
    
  if (newfile != (char*)0)
  {
    loadfile(GUI, newfile);
  }
}


void cb_loadrecentfile(Fl_Widget* widget, void* userdata) 
{
  UserInterface* GUI = getUIByMenuBar(widget);
  
  loadfile(GUI, GUI->opts->recentFile((intptr_t)userdata).c_str());  
}


void cb_popupPic(Fl_Widget* widget, void* userdata)          
{
  UserInterface* GUI = (UserInterface*)0;

  Fl_Tooltip::enable(1);
  
  // Von welchem Fenster kommt die Anforderung?
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->popup == widget)
      GUI = UserInterface::windowList[n];
  
  GUI->popupActionPic((intptr_t)userdata);
}

void cb_popupGroup(Fl_Widget* widget, void* userdata)          
{
  UserInterface* GUI = (UserInterface*)0;

  Fl_Tooltip::enable(1);
  
  // Von welchem Fenster kommt die Anforderung?
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->popup == widget)
      GUI = UserInterface::windowList[n];
  
  GUI->popupActionGroup((intptr_t)userdata);
}


void cb_savefile(Fl_Widget*, void* gui)                   
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->save(false);
}


void cb_savefileas(Fl_Widget*, void* gui)                 
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->save(true);
}


void cb_options(Fl_Widget*, void* gui)                    
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->optionsdlg();
}


void cb_albset(Fl_Widget*, void* gui)                    
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->albumsettingsdlg();
}


void cb_Help(Fl_Widget*, void* value)                     
{
  switch ((intptr_t)value)
  {
   case 1:
    {
      fl_message("Version Number: " FOTOALBUMVERSION
                 "\nJens W. Wulf 2005-2011"
                 "\njens_w2 at gmx dot net\n\n"
                 "install prefix: "                                   STR_INSTALL_PREFIX "\n"
                 "build type: "                                       STR_CMAKE_BUILD_TYPE "\n"
                 "libgraphicsmagick (thumbnail creation, rotation): " STR_USE_LIBGRAPHICSMAGICK "\n"
                 "integrated jpegtran (rotation): "                   STR_USE_INTEGRATED_JPEGTRAN "\n"
                 "libexif (EXIF data): "                              STR_USE_LIBEXIF "\n"
                 "libexiv2 (EXIF and IPTC data): "                    STR_USE_EXIV2 "\n"
                 "pthreads: "                                         STR_USE_PTHREADS "\n"
                 );
    }
    break;
   default:
      GUIMisc::OpenHelp("", "fa_index.htm");
    break;
  }
}

void cb_idle(void* foo)
{
  UserInterface* gui = (UserInterface*)(foo);  
  gui->Idle();
}
