// -*- mode: c; mode: fold -*-
/*
 * Copyright (C) 2006, 2009, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include <time.h>
#include <string>
#include <cstring>
#include <iostream>
#include <inttypes.h>

# include <FL/Fl_Button.H>
# include <FL/Fl.H>


#include "dateselect.h"
#include "config.h"
#include "../shared/calendar.h"
#include "../shared/lib_conversions.h"


#define WINWIDTH  160
#define WINHEIGHT 140

const char* weekDaysShort = "\tM\tT\tW\tT\tF\tSa\tSu";

const int monthViewCols[] = {
  22*SIZEMUL, 19*SIZEMUL,19*SIZEMUL,19*SIZEMUL,19*SIZEMUL,22*SIZEMUL,19*SIZEMUL,19*SIZEMUL,0
};


DateSelect::DateSelect(int* year, 
                       int* month, 
                       int* day) /*{{{*/
{
  time_t curTime;
  tm*    curTimeBr;
  
  // get current time
  curTime = time(0);
  curTimeBr = localtime(&curTime);
  
  // get currently set datetime
  nTYear  = curTimeBr->tm_year+1900;
  nTMonth = curTimeBr->tm_mon+1;
  nTDay   = curTimeBr->tm_mday;
  
  pnYear  = year;
  pnMonth = month;
  pnDay   = day;

  std::cout << "DateSelect(" << *year << "-" << *month << "-" << *day << "\n";
  
  Fl_Window* w;
    {
      Fl_Window* o = win = new Fl_Window(WINWIDTH*SIZEMUL,WINHEIGHT*SIZEMUL,"DateSelector");
      w = o;
      o->callback((Fl_Callback*)cb_Window, (void*)(this));
      w->hotspot(o);    
        {  
          Fl_Box* o = monthBox = new Fl_Box( 0*SIZEMUL,0*SIZEMUL, 20*SIZEMUL,15*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
        {  
          Fl_Box* o = yearBox = new Fl_Box( 20*SIZEMUL,0*SIZEMUL, 40*SIZEMUL,15*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);
        }
        {  
          Fl_Button* o = new Fl_Button((WINWIDTH-80)*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 20*SIZEMUL,15*SIZEMUL,"Y-");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Year, (void*)-1);
        }
        {  
          Fl_Button* o = new Fl_Button((WINWIDTH-60)*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 20*SIZEMUL,15*SIZEMUL,"M-");
          o->labelsize (TEXTSIZE1);          
          o->callback((Fl_Callback*)cb_Month, (void*)-1);
        }
        {
          Fl_Button* o = new Fl_Button((WINWIDTH-40)*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 20*SIZEMUL,15*SIZEMUL,"M+"); 
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Month, (void*)1);
        }
        {
          Fl_Button* o = new Fl_Button((WINWIDTH-20)*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 20*SIZEMUL,15*SIZEMUL,"Y+");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Year, (void*)1);
        }
        {
          monthView = new Fl_Select_Browser (0, 15*SIZEMUL, WINWIDTH*SIZEMUL, (WINHEIGHT - 30)*SIZEMUL);
          monthView->type (1);
          monthView->box (FL_FLAT_BOX);
          monthView->labeltype (FL_NO_LABEL);
          monthView->labelfont (1);
          monthView->labelsize (TEXTSIZE1);
          monthView->textsize (TEXTSIZE1);
          monthView->column_widths(monthViewCols);
          monthView->callback ((Fl_Callback *) cb_MonthView);
          monthView->align (FL_ALIGN_TOP);
          monthView->when(FL_WHEN_RELEASE|FL_WHEN_CHANGED|FL_WHEN_RELEASE_ALWAYS);
        }
        {
          Fl_Button* o = doneB   = new Fl_Button(0*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Done");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Done);
          if (*pnDay == -1)
            o->deactivate();
          else
            o->activate();
        }
        {
          Fl_Button* o = new Fl_Button( 35*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Cancel");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Cancel);
        }
      o->set_modal();
      o->end();
    }
  
  if (*pnDay == -1)
    {
      nDYear  = nTYear;
      nDMonth = nTMonth;
    }
  else
    {
      nDYear  = *pnYear;
      nDMonth = *pnMonth;
    }
  dispMonYear();
}
/*}}}*/

inline void DateSelect::cb_Window_i(Fl_Window*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void DateSelect::cb_Window(Fl_Window* o, void* v) { /*{{{*/
  ((DateSelect*)(o->user_data()))->cb_Window_i(o,v);
}
/*}}}*/


inline void DateSelect::cb_Year_i(Fl_Button*, void* inc) /*{{{*/
{
  nDYear += (intptr_t)inc;
  if (nDYear < 2000)
    nDYear = 2000;
  dispMonYear();
}
/*}}}*/

void DateSelect::cb_Year(Fl_Button* o, void* v) /*{{{*/
{
  ((DateSelect*)(o->parent()->user_data()))->cb_Year_i(o,v);
}
/*}}}*/


inline void DateSelect::cb_Month_i(Fl_Button*, void* inc) /*{{{*/
{
  nDMonth += (intptr_t)inc;
  if (nDMonth < 1)
    {
      nDYear--;
      if (nDYear < 2000)
        nDYear = 2000;      
      nDMonth = 12;
    }
  if (nDMonth > 12)
    {
      (nDYear)++;
      nDMonth = 1;
    }
  dispMonYear();
}
/*}}}*/

void DateSelect::cb_Month(Fl_Button* o, void* v) /*{{{*/
{
  ((DateSelect*)(o->parent()->user_data()))->cb_Month_i(o,v);
}
/*}}}*/


inline void DateSelect::cb_MonthView_i(Fl_Select_Browser*, void* inc) /*{{{*/
{
  if (!Fl::pushed())
    {
      int x;
      int row;
      int sum = monthViewCols[0];
      int col = -1;
      
      x = Fl::event_x();
      row = monthView->value();
      if (row > 0 && x > sum)
        {
          for (int n = 1; n<8; n++)
            {
              sum += monthViewCols[n];
              if (x < sum)
                {
                  col = n;
                  break;
                }
            }
          if (col != -1)
            {
              x = (col - dow_fdom) + (7*(row-2));
              if (x < 1)
                cb_Month_i((Fl_Button*)0, (void*)-1);                                  
              else
                {
                  if (x > MaxTag(nDMonth, nDYear))
                    cb_Month_i((Fl_Button*)0, (void*)1);                                  
                  else
                    {
                      *pnDay   = x;
                      *pnMonth = nDMonth;
                      *pnYear  = nDYear;
                      doneB->activate();
                      dispMonth();
                    }
                }
            }
        }
    }
}
/*}}}*/

void DateSelect::cb_MonthView(Fl_Select_Browser* o, void* v) /*{{{*/
{
  ((DateSelect*)(o->parent()->user_data()))->cb_MonthView_i(o,v);
}
/*}}}*/


inline void DateSelect::cb_Cancel_i(Fl_Button*, void*) /*{{{*/
{
  *pnDay = -1;
  win->hide();
}
/*}}}*/

void DateSelect::cb_Cancel(Fl_Button* o, void* v) /*{{{*/
{
  ((DateSelect*)(o->parent()->user_data()))->cb_Cancel_i(o,v);
}
/*}}}*/


inline void DateSelect::cb_Done_i(Fl_Button*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void DateSelect::cb_Done(Fl_Button* o, void* v) /*{{{*/
{
  ((DateSelect*)(o->parent()->user_data()))->cb_Done_i(o,v);
}
/*}}}*/


void DateSelect::show() /*{{{*/
{
  win->show();
}
/*}}}*/

void DateSelect::hide() /*{{{*/
{
  win->hide();
}
/*}}}*/

int DateSelect::visible() /*{{{*/
{
  return win->visible();
}
/*}}}*/

void DateSelect::dispMonYear() /*{{{*/
{
  std::string tmp;
  
  tmp = itoStr(nDYear, ' ', 4);
  strcpy(yStr, tmp.c_str());
  
  tmp = itoStr(nDMonth, ' ', 1);
  strcpy(mStr, tmp.c_str());
  
  monthBox->label(mStr);
  yearBox->label(yStr);
  monthBox->redraw();
  yearBox->redraw();
  
  dow_fdom = Wochentag(1, nDMonth, nDYear);
  week_fdom = Kalenderwoche(1, nDMonth, nDYear);
  
  dispMonth();
}
/*}}}*/

void DateSelect::dispMonth() /*{{{*/
  /*
   @. Print rest of line, don't look for more '@' signs 
   @@ Print rest of line starting with '@' 
   @b Use a bold font (adds FL_BOLD to font) 
   @i Use an italic font (adds FL_ITALIC to font) 
   @f or @t Use a fixed-pitch font (sets font to FL_COURIER) 
   @c Center the line horizontally 
   @r Right-justify the text 
   @B0, @B1, ... @B255 Fill the backgound with fl_color(n) 
   @C0, @C1, ... @C255 Use fl_color(n) to draw the text 
   @u or @_ Underline the text.
   @- draw an engraved line through the middle.  
   */
{
  std::string line;
  int    dow;
  int    d;
  int    md;
  int    week;
  
  md   = MaxTag(nDMonth, nDYear);
  week = week_fdom;
  dow  = dow_fdom;
  
  monthView->clear();
  monthView->add(weekDaysShort);
 
  line = itoStr(week, ' ', 1);
  
  for (int n= -1; n<dow_fdom; n++)
    line += "\t";
    
  for (d = 1; d <= md; d++)
    {
      if (d == *pnDay && nDMonth == *pnMonth && nDYear == *pnYear)
        line += "@u";
      if (d == nTDay && nDMonth == nTMonth && nDYear == nTYear)
        line += "@i";
      line += itoStr(d, ' ', 1);
      if (++dow == 7)
        {
          monthView->add(line.c_str());
          if (week < 50)
            week++;
          else
            week = Kalenderwoche(d+1, nDMonth, nDYear);          
          line = itoStr(week, ' ', 1);
          line += "\t";
          dow  = 0;
        }
      else
        line += "\t";      
    }
  if (dow != 0)
    monthView->add(line.c_str());
}
/*}}}*/

DateSelect::~DateSelect()
{
  Fl::delete_widget(win);
}
