// -*- mode: c; mode: fold -*-
//
/*
 * Copyright (C) 2005-2009, 2011-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "../config.h"
#include "gui.h"
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <string.h>
#include <iostream>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <FL/Fl.H>

# include <FL/fl_draw.H>
# include <FL/fl_ask.H>
# include <FL/Enumerations.H>
# include <FL/Fl_File_Chooser.H>
# include <FL/Fl_Tooltip.H>
# include <FL/Fl_Help_Dialog.H>

#ifdef USE_LIBGRAPHICSMAGICK
# include <magick/api.h>
#endif

#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"
#include "../base/basetools.h"
#include "../base/export.h"
#include "../base/user.h"
#include "picbutton.h"
#include "itempropdlg.h"
#include "useriddlg.h"
#include "workerdlg.h"
#include "searchdlg.h"
#include "kwreplacedlg.h"
#include "optsdlg.h"
#include "tips.h"
#include "exportdlg.h"
#include "exportmetadlg.h"
#include "albumoptdlg.h"
#include "misc.h"
#include "../base/treekwhandler.h"
#include "../shared/thread.h"
#include "../base/searchprog.h"

#include "../base/batch.h"
#include "../base/plugin.h"
#include "../base/group.h"
#include "addkwdlg.h"

#include "popup.h"

/**
 * *********************** statische Daten ****************************************
 */


BlockTaskManager UserInterface::blockingTasks;
BackTaskDlg*     UserInterface::backTaskDlg = 0;

/**
 * Liste der Fenster
 */
std::vector<UserInterface*> UserInterface::windowList;

T_optionen*                 UserInterface::opts = (T_optionen*)0;

char**                      UserInterface::arguments;

bool                        UserInterface::fGimpHintShown = false;

Fl_Help_Dialog*             initHelpDlg;

/**
 * ********************************************************************************
 */

#define MN_FILE     "&File"
#define MN_F_SAVE   "&Save"
#define MN_F_RECENT "&Recent Files"
#define MN_VIEW     "&View"
#define MN_FSCR_PIC "Fullscreen Pics"
#define MN_FSCR     "Fullscreen"

#include "callback.cpp"


/**
 * workaround for thread-and-"xterm -e" problem: why do I receive SIGHUP?
 */
void signal_handler(int signum)
{
#ifndef WIN32
  std::cerr << "Received signal: " << signum << "  " << strsignal(signum) << "\n";
#endif
}

void setWaitCursor()
{
  fl_cursor(FL_CURSOR_WAIT);
  Fl::wait(0.1);
}
  

int execute_no_thread(void * (*start_routine)(void *), void * arg)
{
  int nRetCode;
  
  setWaitCursor();
  // work
  nRetCode = (intptr_t)(start_routine)(arg);
  // reset cursor
  fl_cursor(FL_CURSOR_DEFAULT);
  
  return(nRetCode);
}

void BlockingTaskPeriodic(void*)
{
  int nStateChange = UserInterface::blockingTasks.Step();
  switch (nStateChange)
  {
    case -1:
      UserInterface::backTaskDlg->w->hide();
      UserInterface::backTaskDlg->SetStatus(0, 0);
      break;
      
    case 1:
      UserInterface::backTaskDlg->w->show();
      break;
      
    default:
      break;
  }
  
  int a, b;
  if (!UserInterface::blockingTasks.FooStatus(a, b))
  {
    UserInterface::backTaskDlg->SetStatus(a, b);
    UserInterface::blockingTasks.PauseUser(UserInterface::backTaskDlg->pPause->value() != 0);
    if (UserInterface::backTaskDlg->fCancel)
    {
      UserInterface::blockingTasks.Cancel();
      UserInterface::backTaskDlg->fCancel = false;
      UserInterface::backTaskDlg->pPause->value(0);
    }
  }
  
  Fl::repeat_timeout(0.2, BlockingTaskPeriodic);
}


int main(int argc, char **argv)                            
{
#ifdef WIN32
  std::ofstream file_cerr("_cerr.txt");
  std::ofstream file_cout("_cout.txt");
  std::basic_streambuf<char>* cerr_orig = std::cerr.rdbuf(file_cerr.rdbuf());
  std::basic_streambuf<char>* cout_orig = std::cout.rdbuf(file_cout.rdbuf());
#endif
  
#ifndef UTF8_A  
  Fl::lock();
#endif

#ifndef WIN32
  // workaround for thread-and-"xterm -e" problem: why do I receive SIGHUP?
  signal (SIGHUP, signal_handler);
#endif
  
#ifdef USE_LIBGRAPHICSMAGICK
  InitializeMagick(*argv);
#endif    

  
  if (false)
  {
    SearchProg* p;
    
    /*
    p = new SearchProg("\"foo\" \"bar\" ; rate.num > 0 : + ; rate.best > 2 rate.worst > 0 : *");
    std::cout << "\n------------------------------------------------------------------------------\n";
    p = new SearchProg("\"foo\" \"bar\"   ;   rate.num > 0 :  +  ;   rate.best  >  2  rate.worst  >   0  :   *  ");
    std::cout << "\n------------------------------------------------------------------------------\n";
    p = new SearchProg("\"foo\" \"bar\" ; rate.num > 0 :  + ; rate.best > 2 rate.worst > 0 : *");
    std::cout << "\n------------------------------------------------------------------------------\n";
    p = new SearchProg("\"foo\" \"bar\" ; rate.num > 0  :  + ; rate.best > 2 rate.worst > 0 : *");
    std::cout << "\n------------------------------------------------------------------------------\n";
     p = new SearchProg("  \"foo\"   \"bar\"   ;   rate.num   >   0   :  +  ;  rate.best  >  2  rate.worst  >  0  :  *  ");
     */
    
    /*
    p = new SearchProg("rate.best > 2 rate.worst > 0");
    std::cout << "\n------------------------------------------------------------------------------\n";
   
    p = new SearchProg(": key.l str.find(\"blubb\") pop ref.num 1 > *");
    std::cout << "\n------------------------------------------------------------------------------\n";
   
    p = new SearchProg(" : key.l str.find(\"blubb\") pop ref.num 1 > *  ");
    std::cout << "\n------------------------------------------------------------------------------\n";
   */
    p = new SearchProg("!c\"suche in keywords\"  ");
    std::cout << "\n------------------------------------------------------------------------------\n";
    
    p = new SearchProg("!c\"suche in keywords\" !K\"suche im Kommentar\" !c\"suche in keywords2\" !pc\"suche parentComment\"   ");
    std::cout << "\n------------------------------------------------------------------------------\n";

    p = new SearchProg("!cK\"!cK\" pc\"pc C\" !K\"!K\" !pc\"!pc B\" !c\"!c\" !pc\"!pc A\"   ");
    std::cout << "\n------------------------------------------------------------------------------\n";
    return 0;
  }
  
  
  try
  {
    UserInterface::opts = new T_optionen((const char*)0);
    
    Fl::scheme(UserInterface::opts->UIStyle.c_str());
    
    UserInterface* GUI = new UserInterface();
    
    //
    std::string strFileToLoad = "";
    std::string strDirToLoad  = "";
    for (int n=1; n<argc; )
    {
      if (argv[n][0] == '-' && strlen(argv[n]) > 1)
      {
        switch (argv[n][1])
        {
          case 'd':
            strDirToLoad = argv[n+1];
            break;
            
          default:
            break;
        }
        n += 2;
      }
      else
      {
        if (n == argc-1)
          strFileToLoad = argv[n];
        n++;
      }
    }
    
    Fl_Double_Window* window = GUI->make_window(argc, argv);
    
    fl_register_images();
    
    // Von den Übergaben der Parameter an show hängt ab, ob die
    // im Windows eingestellten Farben übernommen werden. Wenn
    // einfach meine Parameter übergebe, geht das in die Hose,
    // weil er mit denen nichts anfangen kann. Also tue ich in
    // jedem Fall so, als wären keine Parameter (außer dem Aufruf selbst)
    // übergeben worden.
    UserInterface::arguments    = (char**)malloc(sizeof(char*)*1);
    UserInterface::arguments[0] = argv[0];
    window->show(1, UserInterface::arguments);
    
    if (!strFileToLoad.empty())
      GUI->loadFile(strFileToLoad.c_str());
    else if (!strDirToLoad.empty())
      GUI->loadDir(strDirToLoad.c_str(), 0, ItPic::eEXIFFILEDATE);
    else if (UserInterface::opts->recentFilesCnt())
    {
      // load most recently used file
      GUI->loadFile(UserInterface::opts->recentFile(0).c_str());
    }
    
    if (UserInterface::opts->fShowInitialHelp)
    {
      std::string target;
      
      if (UserInterface::opts->nStartCnt < 0)
        target = "fa_initwarn.htm";
      else
      {
        target = StartupTips::getTipTarget(UserInterface::opts->nStartCnt);
      }
      
      initHelpDlg = new Fl_Help_Dialog();
      initHelpDlg->load(GUIMisc::getHelpPath("C", target).c_str());
      initHelpDlg->show();
    }
    else
    {
      initHelpDlg = (Fl_Help_Dialog*)0;
    }
    
    GUI->SetPermissions();
    
    UserInterface::backTaskDlg = new BackTaskDlg();
    UserInterface::backTaskDlg->w = UserInterface::backTaskDlg->make_window();
    UserInterface::backTaskDlg->w->hide();
    
    Fl::add_timeout(0.2, BlockingTaskPeriodic, 0);
    
    Fl::run();
    
    while (UserInterface::windowList.size())
    {
      GUI = UserInterface::windowList[0];
      if (GUI != (UserInterface*)0)
      {
        delete GUI;
      }
    }
    
    delete window;
    
    if (PicBox::pPicCache)
      delete PicBox::pPicCache;
    
  }
  catch (XMLException e)
  {
    std::cerr << "XMLException: " << e.what() << "\n";
  }
  catch (std::exception e)
  {
    std::cerr << "std::exception: " << e.what() << "\n";
  }

#ifdef USE_LIBGRAPHICSMAGICK
  DestroyMagick();
#endif      
    
#ifdef WIN32
  std::cerr.rdbuf(cerr_orig);
  std::cout.rdbuf(cout_orig);
  file_cerr.close();
  file_cout.close();
#endif
  return 0;
}


Fl_Double_Window* UserInterface::make_window(int       argc,
                                             char*     argv[])
{
  window = new Fl_Double_Window(opts->nWinWidth, opts->nWinHeight);
#ifdef NOBORDER
  window->border(0);
#endif

  //
  Fl_Tooltip::size(opts->nFontSize);

  myData = new TreeView();

  window->user_data((void*)(this));
  {
    int nHeight    = (opts->nFontSize * 16) >> 3;
    int nTreeWidth = opts->nTreeWidth;

    menuBar = new Fl_Menu_Bar(0, 0, window->w(), nHeight);
    menuBar->textsize (opts->nFontSize);

    tile     = new Fl_Tile(0, nHeight, window->w(), window->h()-nHeight, "tile");
    tile->labeltype(FL_NO_LABEL);

    if (nTreeWidth > tile->w() * 0.8)
      nTreeWidth = (int)(tile->w() * 0.8);

    browser  = new PicTreeBrowser(0, nHeight, nTreeWidth, tile->h(), "");
    browser->setData(myData);
    browser->pGUI = this;
    browser->labeltype(FL_NO_LABEL);
    browser->textsize(opts->nFontSize);
    browser->callback( (Fl_Callback*)cb_showGroup, (void*)this );

    mainpane_pic = (PicBox*)0;

    mainpane_group = new PicGroup(nTreeWidth, nHeight, tile->w() - nTreeWidth, tile->h(), "");
    mainpane_group->labelsize(opts->nFontSize);
    mainpane_group->user_data((void*)this);
    mainpane_group->end();

    tile->end();
    window->resizable(tile);
  }
  window->end();
  window->callback((Fl_Callback*)cb_ende, (void*)this);

  // Menü anlegen
  menuBar->add(MN_FILE, 0, 0, (void*)this, FL_SUBMENU);
  menuBar->insert(menuBar->size()-2, "New",                     0,           cb_new,       (void*)this);
  menuBar->insert(menuBar->size()-2, "&Open...",                FL_CTRL+'o', cb_loadfile,  (void*)this);
  menuBar->insert(menuBar->size()-2, MN_F_SAVE,                 FL_CTRL+'s', cb_savefile,  (void*)this);
  menuBar->insert(menuBar->size()-2, "Save as...",              0,           cb_savefileas,(void*)this);
  menuBar->insert(menuBar->size()-2, "Album Settings...",       0,           cb_albset,    (void*)this, FL_MENU_DIVIDER);
  menuBar->insert(menuBar->size()-2, MN_F_RECENT,               0,           0,            (void*)this, FL_SUBMENU | FL_MENU_DIVIDER);
  menuBar->insert(menuBar->size()-2, "Application &Options...", 0,           cb_options,   (void*)this, FL_MENU_DIVIDER);
  menuBar->insert(menuBar->size()-2, "&Quit",                   FL_CTRL+'x', cb_ende,      (void*)this);

  menuBar->add(MN_VIEW, 0, 0, (void*)this, FL_SUBMENU);
  menuBar->insert(menuBar->size()-2, MN_FSCR_PIC, 0, (Fl_Callback*)cb_View, (void*)(UserInterface::FULLSCREEN_PICS), FL_MENU_TOGGLE);
  menuBar->insert(menuBar->size()-2, MN_FSCR,     0, (Fl_Callback*)cb_View, (void*)(UserInterface::FULLSCREEN),      FL_MENU_TOGGLE);
                  
  menuBar->add("&Help", 0, 0, (void*)this, FL_SUBMENU);
  menuBar->insert(menuBar->size()-2, "Help",  0, (Fl_Callback*)cb_Help, (void*)0);
  menuBar->insert(menuBar->size()-2, "About", 0, (Fl_Callback*)cb_Help, (void*)1);
  
  menuBar->textsize(opts->nFontSize);

  // Recent Files behandeln
  updateRecentFilesMenu();
      
  // set options
  Fl_Menu_Item* pMI;
  if (opts->pool->getInt("view.Fullscreen"))
  {
    pMI = (Fl_Menu_Item*)(menuBar->find_item(MN_VIEW "/" MN_FSCR_PIC));
    pMI->set();
  }
  
  if (opts->pool->getInt("view.FullscreenWin", 0))
  {
    pMI = (Fl_Menu_Item*)(menuBar->find_item(MN_VIEW "/" MN_FSCR));
    pMI->set();
    window->fullscreen();
  }
    
  setWindowTitle("");
  
  return window;
}


UserInterface::UserInterface()                             
{
  nScrollPosInGroup = -1;
  pDisplayReq = 0;
  pDisplayReqGroup = 0;
  windowList.push_back(this);
  myFilename = "";

  {
    std::cout << "Fenster: " << windowList.size() << " Stück\n";
    for (unsigned int n=0; n<windowList.size(); n++)
      std::cout << "  " << windowList[n] << "\n";
  }

  myData             = (TreeView*)0;
  popup              = (Fl_Menu_Button*)0;
  windowed           = 0;
  xmlpluginlist      = 0;
}


UserInterface::~UserInterface()                            
{
  for (unsigned int n=0; n<windowList.size(); n++)
  {
    if (windowList[n] == this)
    {
      // Mich selbst aus der Liste löschen
      windowList.erase(windowList.begin() + n);
      // Damit die Schleife beendet wird
      n = windowList.size();
    }
  }

  if (windowList.size() == 0 && initHelpDlg != (Fl_Help_Dialog*)0)
    initHelpDlg->hide();

  {
    std::cout << "Fenster: " << windowList.size() << " Stück\n";
    for (unsigned int n=0; n<windowList.size(); n++)
      std::cout << "  " << windowList[n] << "\n";
  }

  if (opts != (T_optionen*)0)
  {
    SimpleXMLTransfer* tmp;
    tmp = opts->pool->getChild("view");
    tmp->setAttributeOverwrite("Fullscreen",    menuBar->find_item(MN_VIEW "/" MN_FSCR_PIC)->value());
    tmp->setAttributeOverwrite("FullscreenWin", menuBar->find_item(MN_VIEW "/" MN_FSCR)->value());
    delete opts;
    opts = (T_optionen*)0;
  }

  if (myData != (TreeView*)0)
  {
    delete myData;
    myData = (TreeView*)0;
  }
  
  if (mainpane_pic != 0)
    delete mainpane_pic;

  while (groupviewdata.size())
  {
    TGroupViewData gvd;
    
    gvd = groupviewdata[groupviewdata.size()-1];
    groupviewdata.pop_back();
    
    if (gvd.thumbnail != (Fl_Shared_Image*)0)
    {
      gvd.thumbnail->uncache();
      gvd.thumbnail->release();
    }
    free(gvd.tooltip);
  }
    
  if (xmlpluginlist)
    delete xmlpluginlist;  
  if (popup != 0)
    delete popup;  
}

void cb_SetUserID(void* pUser)
{
  UserIDDlg* dlg = new UserIDDlg((User*)pUser);
  Fl_Double_Window* w = dlg->make_window();
  while (w->visible())
  {
    Fl::wait();
  }
}

int  UserInterface::loadFile(const char* filename)         
{
  int         nRetCode;

  myFilename = filename;

  if (myData != (TreeView*)0)
    delete myData;
  myData = new TreeView();
  browser->setData(myData);
  
  bool fUTF8Done = false;
  std::string strFromSet="";
  do
  {
    nRetCode = myData->loadFromFile(filename, strFromSet, cb_SetUserID);
    if (nRetCode == TreeView::eLE_NotUTF8)
    {
      const char* charset = fl_input("Your album needs conversion to UTF8.\n"
                                     "Please enter the character set to convert from (like \"ISO8859-1\")",
                                     0);
      if (charset == 0)
      {
        fl_alert("Conversion canceled.");
        fUTF8Done = true;
      }
      else 
      {
        strFromSet = UTF8::trim(charset);
        if (strFromSet.empty())
        {
          fl_alert("Conversion canceled.");
          fUTF8Done = true;
        }          
      }
    }
    else
      fUTF8Done = true;
  } 
  while (!fUTF8Done);
 
  if (nRetCode == 0)
  {
    opts->addRecentFile(filename);
    updateRecentFilesMenu();
  }
  else
    std::cerr << "error (" << nRetCode << ") loading file: " << filename << "\n";
  
  if ((nRetCode & TreeView::eLE_LINK) != 0)
    fl_alert("Error in project file: link error (see console)");
  if ((nRetCode & TreeView::eLE_BASEPATH) != 0)
    fl_alert("Error: basepath");
  if ((nRetCode & TreeView::eLE_XML) != 0)
    fl_alert("Error in project file: XML structure (see console)");
  if ((nRetCode & TreeView::eLE_VERSION) != 0)
    fl_alert("Error: project file has been created with a more recent\nversion of fotoalbum; I can't handle that. Please update!");
  if ((nRetCode & TreeView::eLE_NOFILE) != 0)
  {
    fl_alert("Error: no such file");
    // remove that file from recent files menu
    opts->removeRecentFile(filename);
    updateRecentFilesMenu();
  }
  if (nRetCode == TreeView::eLE_UTFConv)
    fl_alert("Error converting to UTF8");

  if (nRetCode == 0)
  {
    browser->redraw();
    setWindowTitle(filename);
  }
  return(0);
}

void UserInterface::setWindowTitle(const char* filename)   
{
  strWindowTitle = TITLE_START;

  if (filename != (const char*)0)
    strWindowTitle += filename;
  
  // valgrind output indicates that this string is not copied, so I have to maintain
  // that buffer:
  window->label(strWindowTitle.c_str());
}

void UserInterface::updateRecentFilesMenu(bool fRecurse)
{
  int idx = menuBar->find_index(MN_FILE "/" MN_F_RECENT);
  if (idx != -1)
    menuBar->clear_submenu(idx);
  
  // Neue Menüs anlegen
  if (opts->recentFilesCnt() >= 0)
  {
    std::string  rftext;
    int          nSize = opts->recentFilesCnt();
    
    for (int n=0; n<nSize; n++)
    {
      rftext = opts->recentFile(n);

      if (rftext.length() > 50)
        rftext = "... " + rftext.substr(rftext.length() - 50);

      std::string rft = "";
      for (std::string::size_type p=0; p<rftext.length(); p++)
      {
        if (rftext[p] == '/' && p != 0)
          rft.append("\\/");
        else if (rftext[p] != '\\' ||
                 rftext[p] != '&' ||
                 rftext[p] != '_')
        {
          rft.push_back('\\');
          rft.push_back(rftext[p]);
        }
        else
          rft.push_back(rftext[p]);
      }
      
      int off = 2;
      if (n == 0)
        off = 1;
      intptr_t n_ = n;
      menuBar->insert(idx+off, rft.c_str(), 0, (Fl_Callback*)cb_loadrecentfile, (void*)n_);
    }
  }

  if (fRecurse)
    for (unsigned int n=0; n<windowList.size(); n++)
      if (windowList[n] != this)
        windowList[n]->updateRecentFilesMenu(false);
}

void UserInterface::enterFullscreen()                      
{
  Fl_Widget* mainpane;

  if (mainpane_group == 0)
    mainpane = mainpane_pic;
  else
    mainpane = mainpane_group;

  // save old state
  windowed = new TWindowedData();
  windowed->win_x = window->x();
  windowed->win_y = window->y();
  windowed->win_w = window->w();
  windowed->win_h = window->h();
  windowed->pane_x = mainpane->x();
  windowed->pane_y = mainpane->y();
  windowed->pane_w = mainpane->w();
  windowed->pane_h = mainpane->h();
  // switch to fullscreen mode
#if (1 == 0)
  printf("%i %i %i %i\n", Fl::x(), Fl::y(), Fl::w(), Fl::h());
  window->resize(Fl::x(), Fl::y(), Fl::w(), Fl::h());
#endif
  window->fullscreen();

  int x;
  int y;
  int w;
  int h;
  Fl::screen_work_area(x, y, w, h);
  window->resize(x, y, w, h);
  
  menuBar->hide();
  browser->hide();
}

int  UserInterface::displayGroup(ItGroup* item)
{
  if (pDisplayReq == 0 && pDisplayReqGroup == 0)
  {
    pDisplayReqGroup = item;
    
    Fl::add_idle(cb_idle, (void*)(this));      
  }
  return(0);
}

int  UserInterface::displayItem(ItPicInGroup* item, bool fAllowFullscreen, bool fPluginIfUseful, int mx, int my)
{
  if (pDisplayReq == 0 && pDisplayReqGroup == 0)
  {
    // Im Fall eines nicht direkt darstellbaren Elements soll gleich das plugin-menu geöffnet werden:
    std::string strSuff = strL(FileTools::getSuffix(item->pPic->GetPath()));    
    if (fPluginIfUseful && 
        BaseTools::SuffixIsMovie(strSuff) && 
        FileTools::fileExists(myData->getBasepath() + item->pPic->GetPath()))
    {
      showContextMenuPic(mx, my, item, true);
    }
    else
    {
      pDisplayReq = item;
      fDispReqFullscreen = fAllowFullscreen;
      
      Fl::add_idle(cb_idle, (void*)(this));
    }
  }
  return(0);
}

void UserInterface::setPicFullscreen()                     
{
  int nFullscreen = menuBar->find_item(MN_VIEW "/" MN_FSCR_PIC)->value();
  std::cout << "Pics Fullscreen = " << nFullscreen << "\n";

  if (nFullscreen && mainpane_group == 0)
  {
    int x;
    int y;
    int w;
    int h;

    enterFullscreen();
    Fl::screen_work_area(x, y, w, h);
    mainpane_pic->resize(0, 0, w, h);
  }
}

void UserInterface::setFullscreen()                     
{
  if (menuBar->find_item(MN_VIEW "/" MN_FSCR)->value())
  {
    window->fullscreen();    
  }
  else
  {
    // which size and offset will work on every system?
    int x = (2*window->w())/3;
    int y = (2*window->h())/3;
    window->fullscreen_off(x/4, y/4, x, y);
  }
}

bool UserInterface::hasData()                              
{
  return((myData != (TreeView*)0) && myData->hasData());
}


std::string UserInterface::stringMaxLen(std::string  in,
                                        unsigned int mlen) 
{
  if (in.length() > mlen)
  {
    in = in.substr(0, mlen-3) + "...";
  }
  return(in);
}

void UserInterface::Tree_ShowGroup(ItGroup* pGroup)
{
  myData->makeVisible(pGroup);
  browser->itemSelected = pGroup;
  if (pGroup->canExpand())
    pGroup->fExpanded = true;
  browser->display(pGroup);
  browser->redraw();
}

void UserInterface::followLink(std::string linktext)       
{
  std::cout << "void UserInterface::followLink(std::string linktext = " << linktext << ")\n";
  
  std::string::size_type p1 = linktext.find("<ID=");
  std::string::size_type p2 = linktext.rfind(">");
  bool fDone = false;

  if (p1 != std::string::npos && p2 != std::string::npos)
  {
    std::string strID = linktext.substr(p1+4, p2-p1-4);
    std::cout << "id = " << strID << "\n";
    ItGroup* pGroup;
    ItPicInGroup* pPic = myData->GetPicInGroupPtr(strID, pGroup);
    if (pPic)
    {
      displayItem(pPic);
      fDone = true;
      Tree_ShowGroup(pGroup);
    }
    else
    {
      pGroup = myData->GetGroupPtr(strID);
      if (pGroup)
      {
        displayGroup(pGroup);
        fDone = true;
        Tree_ShowGroup(pGroup);
      }
    }
    nScrollPosInGroup = -1;
  }
  
  if (!fDone)
  {
    std::string path = myData->resolveExternalLink(linktext, GetLastDisplayedGroup());
    if (path.length() > 0)
    {
      std::cout << "display file: " << path << "\n";
      GUIMisc::OpenHtml(path);
    }
  }
}


int  UserInterface::showContextMenuGroup(int      m_x,
                                         int      m_y,
                                         ItGroup* item)
{
  lastPopupGroup = item;  
  Fl_Tooltip::enable(0);

  if (popup != 0)
    delete popup;  
  popup = new Fl_Menu_Button(m_x, m_y, 10, 0, "popup");
  popup->type(0);
  
  bool bUserHasID = myData->pUser->GetUserID().length() > 0;
  
  // Info entry
  {
    std::string info = "(" + item->GetName() + ")";
    addToPopup(true, info.c_str(), PA_NONE, true);
  }

  bool fItemIsSearch      = item->SearchType() == 1;
  bool fItemIsBelowSearch = item->SearchType() == 2;
  
  //
  if (item != myData->pGOrphanedPics)
  {
    if (fItemIsSearch)
    {
      addToPopup(false, "Refresh search result",               PA_REFRESH,       true);
      if (GetLastDisplayedGroup() != 0 &&
          GetLastDisplayedGroup() != item &&
          GetLastDisplayedGroup()->SearchType() != 2)
        addToPopup(false, "Use marked group as search source", PA_MV_SEARCH_SRC, true);
      addToPopup(true, "Edit search...",                       PA_EDIT_SEARCH,   true);                
    }
    
    if (IsPermitted(PERM_EditMeta))
      addToPopup(true, "Properties...",               PA_PROPERTIES, true);
    
    if (bUserHasID)
    {
      if (IsPermitted(PERM_AddGroups) && !fItemIsBelowSearch && !fItemIsSearch)
        addToPopup(false, "Add/Group...",               PA_ADDGROUP, true);
      if (IsPermitted(PERM_AddGroups) && myData->isAddDelAllowed() && IsPermitted(PERM_AddFiles) && !fItemIsBelowSearch && !fItemIsSearch)
        addToPopup(false, "Add/Directory as group...",  PA_ADDDIRGROUP, true);
      if (myData->isAddDelAllowed() && IsPermitted(PERM_AddFiles) && !fItemIsBelowSearch && !fItemIsSearch)
      {
        addToPopup(false, "Add/Files...",             PA_ADDPICS, true);
        addToPopup(false, "Add/File...",              PA_ADDPIC, true);
      }
    }
  }
  if (IsPermitted(PERM_MoveItems))
  {
    addToPopup(false, "Order/chronologically",      PA_ORDER_C, true);
    addToPopup(false, "Order/by filename",          PA_ORDER_P, true);
    addToPopup(false, "Order/by filesize",          PA_ORDER_S, true);
  }
  addToPopup(true, "All files available?",        PA_CHECK_FILES, true);
  if (bUserHasID)
    addToPopup(false, "Search...",                  PA_SEARCH, true);
  if (IsPermitted(PERM_EditMeta))
    addToPopup(true, "Edit keywords...",            PA_KWREPLACE, true);
  
  if (myData->isAddDelAllowed() && IsPermitted(PERM_CreateThumbs))
    addToPopup(true,  "Create Thumbnails",          PA_THUMB_CREATE, true);
  
  if (item != myData->pGOrphanedPics && !fItemIsBelowSearch)
  {
    if (GetLastDisplayedGroup() != 0 && GetLastDisplayedGroup() != item)
    {
      std::vector<ItPicInGroup*> vMarkedPics;
      GetLastDisplayedGroup()->GetMarkedPics(vMarkedPics);
      if (vMarkedPics.size() && !fItemIsSearch)
      {
        addToPopup(false, ("Marked pictures/(" + itoStr(vMarkedPics.size(), ' ', 1) + " pictures)").c_str(), PA_NONE, true);
        addToPopup(false, "Marked pictures/Move into this group",  PA_PICS_MOVE, true);
        addToPopup(false, "Marked pictures/Copy into this group",  PA_PICS_COPY, true);
      }
      
      addToPopup(false, ("Marked group/(" + GetLastDisplayedGroup()->GetName() + ")").c_str(), PA_NONE, true);
      if (IsPermitted(PERM_MoveItems))
      {
        if (!fItemIsSearch)
          addToPopup(false, "Marked group/Move into this group",  PA_MOVE_INTO,  true);
        addToPopup(false, "Marked group/Move after this group", PA_MOVE_AFTER, true);
      }
      if (IsPermitted(PERM_AddLinks))
      {
        if (!fItemIsSearch)
          addToPopup(false, "Marked group/Link into this group",  PA_LINK_INTO,  true);
        addToPopup(false, "Marked group/Link after this group", PA_LINK_AFTER, true);
      }
    }
  }
  
  addToPopup(true,  "Import, Export/Export...",                 PA_EXPORT, true);
  addToPopup(true,  "Import, Export/Export metadata only...",   PA_EXPORT_META, true);
  if (IsPermitted(PERM_EditMeta))
    addToPopup(true,  "Import, Export/Import metadata",         PA_IMPORT_META, true);
  
  if (IsPermitted(PERM_DelEmptyGroups) && 
      ((item->pFirstChild == 0 && item->vPics.size() == 0) || item->SearchType() == 1) &&
      item != myData->pGOrphanedPics &&
      item->nRefCnt == 0 &&
      !fItemIsBelowSearch)
    addToPopup(false, "Remove from album",                      PA_REMOVE, true);
  
  popup->popup();

  return(0);
}


void UserInterface::addToPopup(bool        fSep,
                               const char* descr,
                               intptr_t    nAction,
                               bool        fGroup)
{
  std::string Descr;
  if (fSep)
    Descr = "_";
  else
    Descr = "";
  Descr.append(descr);
  if (fGroup)
    popup->add(Descr.c_str(), 0, (Fl_Callback*)cb_popupGroup, (void*)nAction);
  else
    popup->add(Descr.c_str(), 0, (Fl_Callback*)cb_popupPic, (void*)nAction);
}

std::string UserInterface::GetSourcepath(ItGroup* group)
{
  return(myData->GetAddPath());
}


void UserInterface::popupActionPic(int nAction)               
{
  std::cout << "Popup action picture " << nAction << "\n";

  // Liste der Elemente erstellen
  std::vector<ItPicInGroup*> vList;
  if (lastPopupSinglePic)
    vList.push_back(lastPopupSinglePic);
  else
    GetLastDisplayedGroup()->GetMarkedPics(vList);
  
  switch (nAction)
  {
    case PA_NONE:
      break;
      
    case PA_PROPERTIES:
      {
        // may be triggered by keyboard-shortcut, therefore permissions have to be checked:
        if (IsPermitted(PERM_EditMeta))
        {
          ItemPropertyDlg*  dlg = new ItemPropertyDlg(lastPopupSinglePic->pPic, myData);
          dlg->make_window();
        }
      }
      break;
      
    case PA_TIMESHIFT:
      {
        const char* newnameptr = fl_input("Seconds to shift (positive/negative)", "-3600");
        if (newnameptr != (const char*)0)
        {
          std::string newname = newnameptr;
          if (newname.length() > 0)
          {
            int nShift = atoi(newname.c_str());
            BatchProc::timeshift(vList, nShift);
          }
        }
      }
      break;
      
    case PA_ADD_KEYWORD:
      // may be triggered by keyboard-shortcut, therefore permissions have to be checked:
      if (IsPermitted(PERM_AppendMeta))
      {
        std::set<std::string> kwset;        
        myData->GetUserRoot()->kw_setGroup(kwset);
        AddKwDlg* dlg = new AddKwDlg(kwset);
        if (dlg->showDlg())
        {
          const char* kwstringptr = dlg->ti_words->value();
          if (kwstringptr != (const char*)0)
          {
            std::set<std::string> kwset = Keywords::KeywordList_Set(kwstringptr);        
            if (!kwset.empty())
              BatchProc::kw_add(vList, kwset);
          }
        }
        delete dlg;
      }
      break;
      
    case PA_ROT_CCW:
      for (std::vector<ItPicInGroup*>::iterator it = vList.begin(); it!=vList.end(); it++)
        blockingTasks.Add(new BlockingTask_Rotate(myData->getBasepath() + (*it)->pPic->GetPath(), 90));
      break;
      
    case PA_ROT_CW:
      for (std::vector<ItPicInGroup*>::iterator it = vList.begin(); it!=vList.end(); it++)
        blockingTasks.Add(new BlockingTask_Rotate(myData->getBasepath() + (*it)->pPic->GetPath(), -90));
      break;
      
    case PA_EDIT_GIMP:
      {
        if (!fGimpHintShown)
        {
          fGimpHintShown = true;
          // Last time I used it, it worked, although gimp had not been running.
          // I remember that it did not work someday...
          fl_message("This may only work if Gimp is already running.");
        }
        setWaitCursor();
        BatchProc::editGimp(vList, myData->getBasepath());
        fl_cursor(FL_CURSOR_DEFAULT);
      }
      break;
      
    case PA_EXPORT: 
      {
        ExportDlg* dlg = new ExportDlg(opts, false);
        if (dlg->showDlg())
        {
          Export ex;
          ex.destdir = opts->pool->getString("export.dir");
          if (!ex.destdir.empty())
          {
            ex.setOpts(opts->pool);            
            ex.data = myData;
            
            // fixed values because files are only created, so there is nothing to merge/overwrite
            ex.fKwInherit        = true;
            ex.fKwMerge          = false;
            ex.fCommentOverwrite = true;
            
            ex.run(vList, 0); // todo
            ExportRun(ex);
          }
        }
        delete dlg;
      }        
      break;

    case PA_EXPORT_META:
      ExportMeta(0, vList, GetLastDisplayedGroup());
      break;
      
    case PA_UNLINK:
      {
        int nC = fl_choice("Really remove?", 0, "Cancel", "Remove");
        if (nC == 2)
        {
          ItGroup* pG = GetLastDisplayedGroup();
          for (unsigned int n=0; n<vList.size(); n++)
          {
            ItPic* pP = vList[n]->pPic;
            if (pG->Unlink(vList[n]) == 0)
              myData->pGOrphanedPics->vPics.push_back(new ItPicInGroup(pP));
          }                              
          redrawViews();
        }
      }
      break;
        
    case PA_REMOVE:
    case PA_WIPE:
      {
        bool fDelFiles = (nAction == PA_WIPE);
        int  nC;
        
        if (fDelFiles)
          nC = fl_choice("Really delete? Image files will be deleted, too!", 0, "Cancel", "Delete");
        else
          nC = fl_choice("Really remove from album?", 0, "Cancel", "Remove");
        
        if (nC == 2)
        {
          bool fDelErr = false;
          ItGroup* pG = GetLastDisplayedGroup();
          for (unsigned int n=0; n<vList.size(); n++)
          {
            ItPic* pP = vList[n]->pPic;
            int nErr = 0;
            if (fDelFiles)
            {
              for (unsigned int n=0; TreeView::uThumbSizes[n] > 0; n++)
              {
                std::string strPath = myData->getPicPath(pP, TreeView::uThumbSizes[n]);
                if (FileTools::fileExists(strPath))
                  nErr |= FileTools::rm(strPath);
              }
              if (nErr == 0)
                nErr |= FileTools::rm(myData->getPicPath(pP, 0));
            }
            if (nErr == 0)
            {
              pG->Unlink(vList[n]);
              myData->Delete(pP);
            }
            else
              fDelErr = true;
          }
          if (fDelErr)
            fl_alert("Error deleting file");
          redrawViews();
        }
      }
      break;
      
    case PA_UNMARK:
      for (unsigned int n=0; n<vList.size(); n++)
        vList[n]->fMarked = false;
      mainpane_group->redraw();
      break;
      
    case PA_MOVE_AFTER:
      {
        vList.clear();
        GetLastDisplayedGroup()->GetMarkedPics(vList);
        GetLastDisplayedGroup()->MoveAfter(lastPopupSinglePic, vList);
        redrawViews();
      }
      break;

    case PA_FILE_REPLACE:
      {
        char* picdir = fl_file_chooser("Insert file", NULL, (myData->getBasepath() + vList[0]->pPic->GetPath()).c_str());
        if (picdir != (char*)0 && strlen(picdir) > 0)
        {
          std::cout << "alt: " << vList[0]->pPic->GetPath() << "\n";
          
          
          std::string strPicFilename = picdir;
          myData->updateBasepath(FileTools::path(strPicFilename).c_str());
          vList[0]->pPic->FileReplace(strPicFilename.substr(myData->getBasepath().length()));
          
          std::cout << "neu: " << vList[0]->pPic->GetPath() << "\n";          
        }
      }
      break;
      
    default:
      if (nAction >= PA_PLUGIN_BASE)
      {
        std::string strErr;
        SimpleXMLTransfer* plugin = xmlpluginlist->getChild("item")->getChildAt(nAction - PA_PLUGIN_BASE);
        if (plugin->attributeAsInt("blocking", 0))
        {
          std::list<std::string> vCmds;
          strErr = Plugin::getCmds(vList, plugin, myData, vCmds);
          if (strErr.empty())
            for (std::list<std::string>::iterator it = vCmds.begin(); it!=vCmds.end(); it++)
              blockingTasks.Add(new BlockingTask_System(*it));
        }
        else
          strErr = Plugin::run(vList, plugin, myData);
        
        if (strErr.length())
          fl_alert(strErr.c_str());        
      }
      else if (nAction >= PA_RATE)
      {
        if (myData->pUser->GetUserID().length() > 0)
        {
          for (unsigned int n=0; n<vList.size(); n++)
            vList[n]->pPic->SetRating(nAction-PA_RATE);
          if (mainpane_group)
            mainpane_group->redraw();
          else
            mainpane_pic->UpdateTextBox();
        }
      }
      else
        fl_message("Not implemented yet");
      break;
  }
}

void UserInterface::popupActionGroup(int nAction)               
{
  switch (nAction)
  {
    case PA_NONE:
      break;
      
    case PA_ADDDIRGROUP:
      {
        char* picdir = fl_dir_chooser("Add directory as group", GetSourcepath(lastPopupGroup).c_str());
        
        if (picdir != (char*)0)
        {
          ItGroup* pGroup = lastPopupGroup->addNewGroup(FileTools::name(picdir).c_str(), myData);
          loadDir(picdir, pGroup, ItPic::eEXIFFILEDATE);
          browser->redraw();
        }
      }
      break;
      
    case PA_ADDGROUP:
      {
        const char* newnameptr = fl_input("Name of new group", "new group");
        if (newnameptr != (const char*)0)
        {
          std::string newname = newnameptr;
          if (newname.length() > 0)
          {
            lastPopupGroup->addNewGroup(newname.c_str(), myData);
            browser->redraw();
          }
        }
      }
      break;
      
    case PA_ADDPICS:
      {
        char* picdir = fl_dir_chooser("Insert files from directory", GetSourcepath(lastPopupGroup).c_str());
        if (picdir != (char*)0)
          loadDir(picdir, lastPopupGroup, ItPic::eEXIFFILEDATE);
      }
      break;
      
    case PA_ADDPIC:
      {
        char* picdir = fl_file_chooser("Insert file", NULL, GetSourcepath(lastPopupGroup).c_str());
        if (picdir != (char*)0 && strlen(picdir) > 0)
          myData->addFile(picdir, lastPopupGroup, ItPic::eEXIFFILEDATE);
      }
      break;
      
    case PA_REMOVE:
      {
        int nC = fl_choice("Really remove from album?", 0, "Cancel", "Remove");
        
        if (nC == 2)
        {
          if (lastPopupGroup == GetLastDisplayedGroup())
            browser->itemSelected = 0; // todo: das geht noch besser
          lastPopupGroup->pParent->Unlink(lastPopupGroup);
          
          // It must be possible to delete a search group even though it has children.
          // However, its children have to be deleted, too:
          ItGSearch* pS = dynamic_cast<ItGSearch*>(lastPopupGroup);
          if (pS)
            pS->DeleteMyContent();
            
          myData->Delete(lastPopupGroup);
                    
          redrawViews();
        }
      }
      break;

    case PA_PICS_MOVE:
    case PA_PICS_COPY:
      {
        ItGroup* pSrc = GetLastDisplayedGroup();
        
        if (nAction == PA_PICS_COPY && pSrc == myData->pGOrphanedPics)
          nAction = PA_PICS_MOVE;
        
        std::vector<ItPicInGroup*> vMarkedPics;
        pSrc->GetMarkedPics(vMarkedPics);
        for (unsigned int n=0; n<vMarkedPics.size(); n++)
        {
          ItPic* pP = vMarkedPics[n]->pPic;
          lastPopupGroup->vPics.push_back(new ItPicInGroup(pP));
          if (nAction == PA_PICS_MOVE)
            pSrc->Unlink(vMarkedPics[n]);
          pP->IncRefCnt();
          if (pSrc == myData->pGOrphanedPics)
            pP->IncRefCnt();
        }
        redrawViews();
      }
      break;
      
    case PA_MOVE_INTO:
      {
        ItGroup* pGroup = GetLastDisplayedGroup();
        pGroup->pParent->Unlink(pGroup);
        lastPopupGroup->AddGroup(pGroup);
        redrawViews();
      }
      break;
      
    case PA_LINK_INTO:
      {
        ItGLinkRef* ng = new ItGLinkRef(lastPopupGroup->getLastChild(), lastPopupGroup, GetLastDisplayedGroup());
        myData->AddGroup(ng);
        redrawViews();
      }
      break;

    case PA_MOVE_AFTER:
      {
        lastPopupGroup->pParent->MoveAfter(lastPopupGroup, GetLastDisplayedGroup());
        redrawViews();
      }
      break;
      
    case PA_ORDER_C:
      {
        setWaitCursor();
        lastPopupGroup->SortChildrenChrono();
        fl_cursor(FL_CURSOR_DEFAULT);
      }
      break;
      
    case PA_ORDER_P:
      {
        setWaitCursor();
        lastPopupGroup->SortChildrenPath();
        fl_cursor(FL_CURSOR_DEFAULT);
      }
      break;
      
    case PA_PROPERTIES:
      {
        // may be triggered by keyboard-shortcut, therefore permissions have to be checked:
        if (IsPermitted(PERM_EditMeta))
        {
          ItemPropertyDlg* dlg = new ItemPropertyDlg(lastPopupGroup, myData);
          dlg->make_window();
        }
      }
      break;

    case PA_KWREPLACE:
      {
        KwReplaceDlg* dlg = new KwReplaceDlg(lastPopupGroup, myData->GetUserRoot());
        dlg->showDlg();
        delete dlg;
      }
      break;
      
    case PA_EXPORT:
      {
        ExportDlg* dlg = new ExportDlg(opts, true);
        if (dlg->showDlg())
        {
          Export ex;
          ex.destdir = opts->pool->getString("export.dir");
          if (!ex.destdir.empty())
          {
            ex.setOpts(opts->pool);            
            ex.data = myData;
            
            // fixed values because files are only created, so there is nothing to merge/overwrite
            ex.fKwInherit        = true;
            ex.fKwMerge          = false;
            ex.fCommentOverwrite = true;
            
            std::vector<ItGroup*> listGroups;
            listGroups.push_back(lastPopupGroup);
            ex.run(listGroups);
            ExportRun(ex);
          }
        }
        delete dlg;
      }        
      break;
      
    case PA_EXPORT_META:
      {
        std::vector<ItPicInGroup*> vList;
        ExportMeta(lastPopupGroup, vList, 0);
      }        
      break;
      
    case PA_CHECK_FILES:
      {
        setWaitCursor();
        unsigned int uMF = lastPopupGroup->countMissingFiles(myData->getBasepath());
        fl_cursor(FL_CURSOR_DEFAULT);
        
        if (uMF)
          fl_alert("At least one file is missing! See console...");
        else
          fl_message("All files found.");
      }
      break;
      
    case PA_SEARCH:
      {
        std::set<std::string> kwset;        
        lastPopupGroup->kw_setGroup(kwset);        
        SearchDlg* dlg = new SearchDlg(kwset, myData, lastPopupGroup);
        
        dlg->cb_flat->value(0);
        dlg->ti_expr->value("\"\"");
        dlg->ti_expr->position(1);
        dlg->inp_name->value("new search");
              
        if (dlg->showDlg())
        {          
          if (strlen(dlg->ti_expr->value()))
          {
            ItGSearch* ng = new ItGSearch(lastPopupGroup->getLastChild(), lastPopupGroup, dlg->pSrcGroup, dlg->ti_expr->value(), dlg->cb_flat->value());
            ng->SetName(dlg->inp_name->value());
            ng->SetSearchProg(new SearchProg(ng->strExpr));
            myData->AddGroup(ng);
            ng->Refresh(myData->getBasepath());
            browser->redraw();
          }
        }
        delete dlg;
      }
      break;
      
    case PA_REFRESH:
      dynamic_cast<ItGSearch*>(lastPopupGroup)->Refresh(myData->getBasepath());
      displayGroup(lastPopupGroup);
      browser->redraw();           
      break;     
      
    case PA_EDIT_SEARCH:
      {
        ItGSearch* sg = dynamic_cast<ItGSearch*>(lastPopupGroup);
        
        std::set<std::string> kwset;
        if (sg->pSrc)
          sg->pSrc->kw_setGroup(kwset);
        SearchDlg* dlg = new SearchDlg(kwset, myData, sg->pSrc);

        dlg->cb_flat->value(sg->nFlat);
        dlg->ti_expr->value(sg->strExpr.c_str());
        dlg->inp_name->value(sg->GetName().c_str());
        
        if (dlg->showDlg())
        {          
          if (strlen(dlg->ti_expr->value()))
          {
            sg->nFlat   = dlg->cb_flat->value();
            sg->strExpr = dlg->ti_expr->value();
            sg->pSrc    = dlg->pSrcGroup;
            sg->SetName(dlg->inp_name->value());
            sg->SetSearchProg(new SearchProg(sg->strExpr));
            sg->Refresh(myData->getBasepath());
            displayGroup(lastPopupGroup);
            browser->redraw();
          }
        }
        delete dlg;        
      }
      break;     
      
    case PA_MV_SEARCH_SRC:
      {
        ItGSearch* sg = dynamic_cast<ItGSearch*>(lastPopupGroup);        
        sg->pSrc = GetLastDisplayedGroup();
        sg->Refresh(myData->getBasepath());
        browser->redraw();
      }
      break;     
      
    default:      
      fl_message("Not implemented yet");
      break;      
  }
  
#if 0 == 1 // todo: popupActionGroup
  std::cout << "Popup action " << nAction << "\n";

  switch (nAction)
  {
   case PA_ORDER_P:
    {
//todo      ((TreeGroupBase*)lastPopupSingleItem)->sortChildrenPath();
    }
    break;

   case PA_ORDER_S:
    {
//todo      ((TreeGroupBase*)lastPopupSingleItem)->sortChildrenSize(myData->getBasepath());
    }
    break;

    case PA_IMPORT_META:
      {
        std::set<TreeItem*> set;
        std::set<TreeItem*>::iterator it;
        TreeView::RecursiveList(list, set);
        it = set.begin();
        while (it != set.end())
        {
          TreePicData* p = dynamic_cast<TreePicData*>(*it);
          if (p)
            p->ImportMetadata(myData->getBasepath());
          it++;
        }
      }
      break;

  }
#endif
}


int  UserInterface::save(bool fNewName)                    
{
  int nRetCode = 0;

  if (myData->getProjectfilePath().length() == 0)
    fNewName = true;

  if (fNewName)
  {
    char*        newfile;

    newfile = fl_file_chooser("Save as...", "*.pg.xml", "");

    if (newfile != (char*)0)
    {
      std::string filename = newfile;

      if (filename.find(".pg.xml") != filename.length()-7)
      {
        filename += ".pg.xml";
      }

      nRetCode = myData->saveAsNewProject(filename);

      if (nRetCode)
      {
        fl_alert("Error saving to new file");
        std::cerr << "Error saving to new file: " << nRetCode << "\n";
      }
      else
      {
        setWindowTitle(filename.c_str());      
        opts->addRecentFile(filename.c_str());
        updateRecentFilesMenu();
      }
    }
  }
  else
  {
    nRetCode = myData->saveToProjectFile();
    if (nRetCode)
    {
      fl_alert("Error saving to file");
    }
  }
  
  SetPermissions();
  
  return(nRetCode);
}


int  UserInterface::askBeforeClosing()                     
{
  bool fUnsavedData = true;

  if (hasData())
  {
    setWaitCursor();
    
    std::cout << "hasData() == true\n";
    if (myData->getProjectfilePath().length() != 0)
    {
      std::cout << "projectfilePath\n";
      try 
      {
        if (myData->dataSaved())
          fUnsavedData = false;
      }
      catch (XMLException e)
      {
        std::cerr << "XMLException in int  UserInterface::askBeforeClosing(): " << e.what() << "\n";
        fl_alert((std::string("Error: ") + e.what() + std::string("\nPlease try again.")).c_str());
        fUnsavedData = true;
      }
    }
    
    fl_cursor(FL_CURSOR_DEFAULT);
  }
  else
  {
    std::cout << "hasData() == false\n";
    fUnsavedData = false;
  }

  if (fUnsavedData)
  {
    if (fl_choice("There is unsaved data. Exit anyway?", NULL, "Cancel", "Exit") == 2)
      fUnsavedData = false;
  }

  if (fUnsavedData)
    return(-1);
  else
    return(1);
}

void UserInterface::redrawViews()                             
{
  browser->redraw();
  if (mainpane_group != (PicGroup*)0 && GetLastDisplayedGroup() != 0)
  {
    displayGroup(GetLastDisplayedGroup());
  }
}


void UserInterface::saveOptions()                             
{
  opts->nWinWidth  = window->w();
  opts->nWinHeight = window->h();
  opts->nTreeWidth = browser->w();
}


void UserInterface::optionsdlg()                              
{
  OptsDlg* dlg = new OptsDlg(opts);
  dlg->showDlg();
  delete dlg;
}


void UserInterface::setLastDisplayedAsLastPopupSingle()       
{  
  lastPopupSinglePic = mainpane_pic->getPic();
}


void UserInterface::toggleMarkCurrentPic() 
{
  // toggle data
  mainpane_pic->getPic()->fMarked = !mainpane_pic->getPic()->fMarked;
  // make it visible in the picture
  mainpane_pic->redrawTextBox();
}


void UserInterface::albumsettingsdlg() 
{
  AlbumOptDlg* dlg = new AlbumOptDlg(myData);
  dlg->showDlg();
  delete dlg;
}

bool UserInterface::IsPermitted(unsigned int uMask)
{
  return(myData->perm->IsPermitted(uMask));
}

void UserInterface::SetPermissions()
{
  Fl_Menu_Item* pMI = (Fl_Menu_Item*)(menuBar->find_item(MN_FILE "/" MN_F_SAVE));
  if (!IsPermitted(PERM_OverwriteAlbum))
    pMI->deactivate();
  else
    pMI->activate();
}

void UserInterface::Idle()
{
  if (pDisplayReqGroup != 0)
  {
    bool bScroll = false;
    if (mainpane_group == 0)
    {
      bScroll = true;

      int x, y;
      bool bPicWasFullscreen = windowed != 0 && menuBar->find_item(MN_VIEW "/" MN_FSCR_PIC)->value();
      
      if (windowed != 0)
      {
        if (windowed != 0 && menuBar->find_item(MN_VIEW "/" MN_FSCR)->value() == 0)
          window->fullscreen_off(windowed->win_x, windowed->win_y, windowed->win_w, windowed->win_h);
        menuBar->show();
        browser->show();
        x = windowed->pane_x;
        y = windowed->pane_y;
        delete windowed;
        windowed = 0;
      }
      else
      {
        x = mainpane_pic->x();
        y = mainpane_pic->y();
      }
      
      tile->remove(mainpane_pic);
      
      if (bPicWasFullscreen)
      {
        tile->position(1, 0, x, 0);                
      }
      mainpane_group = new PicGroup(x, y, window->w()-x, window->h()-y, "");
      mainpane_group->labelsize(opts->nFontSize);
      mainpane_group->user_data((void*)this);
      mainpane_group->end();
      
      tile->add(mainpane_group);
    }

    ItGroup*         pGroup = pDisplayReqGroup;
    int              nX    = mainpane_group->x() + 5;
    int              nY    = mainpane_group->y() + 5;
    int              nSize = opts->uThumbSize;
    TGroupViewData   gvd;
    std::string      tooltipString;
    std::string      strGroupdescr = pGroup->GetName();

    mainpane_group->scrollbar.linesize(nSize/2);
      
    // Wenn es eine nicht-ausgeführte Suche ist, soll sie ausgeführt werden
    {
      ItGSearch* pS = dynamic_cast<ItGSearch*>(pGroup);
      if (pS && pS->vPics.size() == 0 && pS->pFirstChild == 0)
      {
        pS->Refresh(myData->getBasepath());
        if (pS->canExpand())
          pS->fExpanded = true;
        browser->redraw();
      }
    }        
    
    if (strGroupdescr.length())
      strGroupdescr = "<p>" + strGroupdescr + "</p>";
    
    {
      JWWDateTime min, max;
      
      min.nTime = 0;
      min.nDate = 0;
      max = min;
      
      int nCnt = pGroup->getMinMaxDateTime(min, max, false);
      std::string minstr = min.getDateTimeAsString();
      std::string maxstr = max.getDateTimeAsString();
      
      strGroupdescr += "<p>" + itoStr(nCnt, ' ', 1) + " Pictures</p>";
      
      if (minstr.compare(maxstr) == 0)
      {
        if (maxstr.length())
          strGroupdescr += "<p>" + maxstr + "</p>";
      }
      else
        strGroupdescr += "<p>" + maxstr + " to " + minstr + "</p>";
    }
    
    {
      std::string strCommentHtml = BaseTools::GetAsHTML(pGroup->GetComment());
      if (strCommentHtml.length())
        strGroupdescr += strCommentHtml;
    }
    
    // --- clear group display --------------------------------------
    mainpane_group->clear();
    
    while (groupviewdata.size())
    {
      gvd = groupviewdata[groupviewdata.size()-1];
      groupviewdata.pop_back();
      
      if (gvd.thumbnail != (Fl_Shared_Image*)0)
      {
        gvd.thumbnail->uncache();
        gvd.thumbnail->release();
      }
      free(gvd.tooltip);
    }
    
    // --- make sure thumbnails exist -------------------------------
    if (opts->fCreateThumbs && IsPermitted(PERM_CreateThumbs))
    {
      WorkThumbCreate work;
      
      for (unsigned int n=0; n<pGroup->vPics.size(); n++)
      {
        ItPic*       pPic   = pGroup->vPics[n]->pPic;
        std::string  ref    = myData->getPicPath(pPic, 0);
        std::string  result = myData->getPicPath(pPic, opts->uThumbSize);
        
        if (FileTools::isFileUpToDate(ref, result) != 1)
        {
          work.filelist.push_back(ref);
          work.filelist.push_back(result);
        }
      }
      
      // Create thumbs
      if (work.filelist.size())
      {
        work.nMaxW  = opts->uThumbSize;
        work.nMaxH  = opts->uThumbSize;
        work.flQual = opts->pool->getDouble("thumbnail.quality");
        work.nType  = opts->pool->getInt("thumbnail.mode");
        
        WorkerData   wd;
        wd.init(work.filelist.size()/2, (void*)&work);
        
        int nRetCode;
        
        if (SimpleThread::Create(createThumbs, (void*)&wd))
        {
          std::cerr << "error creating thread\n";
          std::cerr.flush();
          nRetCode = -1;
        }
        else
        {
          blockingTasks.Pause(true);
          WorkerDlg* wdlg = new WorkerDlg(&wd);
          wdlg->showDlg();
          while (wd.nRun) { Fl::wait(0.2); };
          nRetCode = wd.nRetCode;
          delete wdlg;
          blockingTasks.Pause(false);
        }
        if (nRetCode)
        {
          fl_alert("Error while creating thumbnails (see console)");
        }
      }
    }
    
    // --- display --------------------------------------------------
    setWaitCursor();
    groupviewdata.reserve(pGroup->vPics.size());
    for (unsigned int n=0; n<pGroup->vPics.size(); n++)
    {
      ItPic*     pPic   = pGroup->vPics[n]->pPic;
      PicButton* button = new PicButton(nX, nY, nSize+6, nSize+6);        
      button->align(FL_ALIGN_CENTER);
      {
        tooltipString = pPic->getDescr(4, true, myData->getBasepath());
        gvd.tooltip = (char*)malloc(tooltipString.length()+1);
        strcpy(gvd.tooltip, tooltipString.c_str());
        button->tooltip(gvd.tooltip);
      }
      button->item(pGroup->vPics[n]);
      button->callback( (Fl_Callback*)cb_picbutton, (void*)this );
      gvd.thumbnail = Fl_Shared_Image::get(myData->getPicPath(pPic, nSize).c_str());
      if (gvd.thumbnail != (Fl_Shared_Image*)0)
      {
        button->image  (gvd.thumbnail);
        button->deimage(gvd.thumbnail);
      }
      else
      {
        button->setLabel(FileTools::name(pPic->GetPath()));
      }
      mainpane_group->add(button);
      
      gvd.pic = pGroup->vPics[n];
      groupviewdata.push_back(gvd);
      
      nY += nSize + 6 + 5;
    }
    
    mainpane_group->setDescr(strGroupdescr);
    mainpane_group->resize(mainpane_group->x(), mainpane_group->y(), mainpane_group->w(), mainpane_group->h());
    if (bScroll && nScrollPosInGroup >= 0)
      mainpane_group->scroll_to(0, nScrollPosInGroup);
    mainpane_group->redraw();
    fl_cursor(FL_CURSOR_DEFAULT);
    
    
    pDisplayReqGroup = 0;
  }
  else if (pDisplayReq != 0)
  {
    ItPicInGroup* item = pDisplayReq;
    bool fAllowFullscreen = fDispReqFullscreen;
    
    // If one is following a link:
    fl_cursor(FL_CURSOR_DEFAULT, FL_FOREGROUND_COLOR, FL_BACKGROUND_COLOR);
    
    if (mainpane_group != 0)
    {
      nScrollPosInGroup = mainpane_group->yposition();
      
      int x, y, w, h;
      bool bFullscreen = menuBar->find_item(MN_VIEW "/" MN_FSCR_PIC)->value() && fAllowFullscreen;
      
      if (bFullscreen)
      {
        enterFullscreen();
        Fl::screen_work_area(x, y, w, h);
        x = y = 0;
      }
      else
      {
        x = mainpane_group->x();
        y = mainpane_group->y();
        w = mainpane_group->w();
        h = mainpane_group->h();
      }
      
      tile->remove(mainpane_group);
      Fl::delete_widget(mainpane_group);
      mainpane_group = (PicGroup*)0;
      
      if (mainpane_pic == 0)
        mainpane_pic = new PicBox(x, y, w, h);
      else
        mainpane_pic->resize(x, y, w, h);
      mainpane_pic->labelsize(opts->nFontSize);
      mainpane_pic->callback((Fl_Callback*)cb_picbox);
      mainpane_pic->boxcallback((Fl_Callback*)cb_textbox, (void*)this);
      mainpane_pic->align(FL_ALIGN_CENTER);
      tile->add(mainpane_pic);
      if (bFullscreen)
      {
        // Ohne diesen Aufruf kann man während der Vollbildanzeige die Tile-Grenze
        // mit der Maus im Bild finden (man sieht es am Zeiger) und verschieben.
        // Mit diesem Aufruf ist die Grenze wirklich ganz links.
        // Auf newx=0 darf man es wohl nicht setzen, weil es dann nicht rückgängig
        // gemacht werden kann.
        tile->position(windowed->pane_x, 0, 1, 0);
      }
    }
    
    // load and display pic
    blockingTasks.Pause(true);
    if (mainpane_pic->setPic(item, myData, opts, this))
    {
      blockingTasks.Pause(false);
      // todo: Fehler absetzen?
      pDisplayReq = 0;
      return;
    }
    blockingTasks.Pause(false);
    
    pDisplayReq = 0;
  }
  Fl::remove_idle(cb_idle, (void*)(this));      
}

void UserInterface::AfterCreation()
{
  updateRecentFilesMenu();
  SetPermissions();
}

int UserInterface::loadDir(const char* dirname, ItGroup* pGroup, ItPic::eDateTimeSource dtmethod)
{
  if (pGroup == 0)
    pGroup = myData->GetUserRoot();
  setWaitCursor();
  myData->addPicsFromDir(dirname, pGroup, dtmethod);
  fl_cursor(FL_CURSOR_DEFAULT);

  return(0);
}

int UserInterface::showContextMenuPic(int           m_x,
                                      int           m_y,
                                      ItPicInGroup* item,
                                      bool fOnlyPlugins)
{
  //  std::cout << "UserInterface::showContextMenuPic(" << m_x << ", " << m_y << ", ...)\n";
  Fl_Tooltip::enable(0);

  bool fItemIsBelowSearch = GetLastDisplayedGroup()->SearchType() == 2;
  
  // Liste der Elemente erstellen
  std::vector<ItPicInGroup*> vMarkedPics;
  GetLastDisplayedGroup()->GetMarkedPics(vMarkedPics);
  bool fThereAreMarkedItemsButItemIsNotOneOfThem = false;
  if (!item->fMarked)
  {
    if (vMarkedPics.size() > 0)
      fThereAreMarkedItemsButItemIsNotOneOfThem = true;
    vMarkedPics.clear();
    vMarkedPics.push_back(item);
    lastPopupSinglePic = item;
  }
  else
  {
    if (vMarkedPics.size() == 1)
      lastPopupSinglePic = vMarkedPics[0];
    else
      lastPopupSinglePic = 0;
  }
  
  if (popup != 0)
    delete popup;  
  popup = new Fl_Menu_Button(m_x, m_y, 10, 0, "popup");
  popup->type(0);

  // Info entry
  {
    std::string info;
    if (vMarkedPics.size() == 1)
      info = "(One item)";
    else
      info = "(" + itoStr(vMarkedPics.size(), ' ', 1) + " items)";
    addToPopup(true, info.c_str(), PA_NONE);
  }
  
  // Real entries
  if (fOnlyPlugins == false)
  {
    if (vMarkedPics.size() == 1 && IsPermitted(PERM_EditMeta))
      addToPopup(true, "Properties...",               PA_PROPERTIES);

    if (item->fMarked)
      addToPopup(true, "Unmark",                      PA_UNMARK);
  
    if (myData->pUser->GetUserID().length() > 0)
    {
      addToPopup(false, "Rate/Unrated",         PA_RATE+0);
      addToPopup(false, "Rate/1, X, delete!",   PA_RATE+1);
      addToPopup(false, "Rate/2",               PA_RATE+2);
      addToPopup(false, "Rate/3",               PA_RATE+3);
      addToPopup(false, "Rate/4",               PA_RATE+4);
      addToPopup(false, "Rate/5, best",         PA_RATE+5);
    }
  
    if (myData->isAddDelAllowed() && IsPermitted(PERM_EditFiles))
    {
      addToPopup(false, "Rotate counterclockwise",    PA_ROT_CCW);
      addToPopup(false, "Rotate clockwise",           PA_ROT_CW);
      addToPopup(false, "Open with GIMP",             PA_EDIT_GIMP);
    }
    if (IsPermitted(PERM_EditMeta))
      addToPopup(false, "Shift time...",              PA_TIMESHIFT);
    if (IsPermitted(PERM_AppendMeta))
      addToPopup(true,  "Add keyword...",             PA_ADD_KEYWORD);
  }
  
  // Insert Plugins menu
  {
    if (xmlpluginlist)
      delete xmlpluginlist;
    
    std::string loadErrorString = Plugin::getPluginList(xmlpluginlist);
    
    if (loadErrorString.length())
      fl_alert(loadErrorString.c_str());
    else if (xmlpluginlist)
    {
      SimpleXMLTransfer* g;
      unsigned int       size;
      
      try
      {
        SimpleXMLTransfer*  i;
        std::string         label;
        
        g = xmlpluginlist->getChild("item");
        size = g->getChildCount();
        
        for (unsigned int cnt=0; cnt<size; cnt++)
        {
          i = g->getChildAt(cnt);
          
          if (Plugin::match(vMarkedPics, i) && 
              (i->attributeAsInt("readonly", 1)==1 || IsPermitted(PERM_EditFiles)))
          {
            if (fOnlyPlugins)
              label = "";
            else
              label = "Plugins/";
            label += i->attribute("descr");              
            addToPopup((cnt == size-1), label.c_str(), PA_PLUGIN_BASE + cnt);
          }                            
        }
      }
      catch (XMLException e)
      {
        std::string msg = "Error in plugins.xml: ";
        msg += e.what();
        fl_alert(msg.c_str());
      }  
    }
  }

  if (fOnlyPlugins == false)
  {
    if (fThereAreMarkedItemsButItemIsNotOneOfThem && IsPermitted(PERM_MoveItems) && !fItemIsBelowSearch)
      addToPopup(false, "Move marked after this item",            PA_MOVE_AFTER);
    
    addToPopup(true,  "Import, Export/Export...",                 PA_EXPORT);
    addToPopup(true,  "Import, Export/Export metadata only...",   PA_EXPORT_META);
    if (IsPermitted(PERM_EditMeta))
      addToPopup(true,  "Import, Export/Import metadata",         PA_IMPORT_META);
    
    if (!fItemIsBelowSearch)
    {
      if (GetLastDisplayedGroup() == myData->pGOrphanedPics)
      {
        if (IsPermitted(PERM_DelItems))
          addToPopup(false, "Remove from album",            PA_REMOVE);  
        if (IsPermitted(PERM_DelFiles))
          addToPopup(false, "Remove (incl. picture files)", PA_WIPE);
      }
      else
      {
        if (true) // todo: permission bit "unlink picture"
          addToPopup(false, "Remove",                       PA_UNLINK);
      }
    }
    
    if (lastPopupSinglePic)
    {
      std::string strP = strL(FileTools::getSuffix(lastPopupSinglePic->pPic->GetPath()));
      if (BaseTools::SuffixIsMovie(strP))
        addToPopup(true, "Replace file...",                 PA_FILE_REPLACE);
    }
  }
  popup->popup();  
  
  return 0;
}

bool UserInterface::ThisButtonIsMine(PicButton* pButton)
{
  PicGroup* pPicGroup = dynamic_cast<PicGroup*>(pButton->parent());

  return(pPicGroup != 0 && mainpane_group == pPicGroup);
}

void UserInterface::ExportRun(Export& ex)
{
  if (ex.error.length())
  {
    fl_alert(ex.error.c_str());
  }
  else
  {
    opts->pool->setAttributeOverwrite("export.dir", ex.destdir);
    
    WorkThumbCreate work;
    WorkerData      wd;
    int             nRetCode;
    
    // --- Part 1/4: shrink list --------------------------
    {
      SimpleXMLTransfer* shr = opts->getShrink();
      work.nMaxW  = shr->attributeAsInt("w", 800);
      work.nMaxH  = shr->attributeAsInt("h", 600);
      work.flQual = opts->pool->getDouble("shrink.quality");
      if (opts->pool->getInt("export.fast", 0) != 0)
        work.nType = 2;
      else
        work.nType = 0;
      for (unsigned int cnt=0; cnt<ex.shrink.size(); cnt++)
      {
        work.filelist.push_back(ex.shrink[cnt]);
      }
    }
    
    wd.init(work.filelist.size()/2, (void*)&work);
    
    nRetCode = SimpleThread::Create(createThumbs, (void*)&wd);
    if (nRetCode)
    {
      std::cerr << "error creating thread\n";
      nRetCode = -1;
    }
    else
    {
      blockingTasks.Pause(true);
      WorkerDlg* wdlg = new WorkerDlg(&wd);
      wdlg->showDlg();
      while (wd.nRun) { Fl::wait(0.2); };
      nRetCode = wd.nRetCode;
      delete wdlg;
      blockingTasks.Pause(false);
    }
    if (nRetCode)
    {
      fl_alert("Error");
    }
    
    // --- Part 2/4: thumb list --------------------------
    {
      work.nMaxW  = opts->pool->getDouble("export.thumbsize");
      work.nMaxH  = work.nMaxW;
      work.flQual = opts->pool->getDouble("export.thumbqual");
      if (opts->pool->getInt("export.fast", 0) != 0)
        work.nType = 2;
      else
        work.nType = 0;
      work.filelist.clear();
      for (unsigned int cnt=0; cnt<ex.vThumbs.size(); cnt++)
      {
        work.filelist.push_back(ex.vThumbs[cnt]);
      }
    }
    
    wd.init(work.filelist.size()/2, (void*)&work);
    
    nRetCode = SimpleThread::Create(createThumbs, (void*)&wd);
    if (nRetCode)
    {
      std::cerr << "error creating thread\n";
      nRetCode = -1;
    }
    else
    {
      blockingTasks.Pause(true);
      WorkerDlg* wdlg = new WorkerDlg(&wd);
      wdlg->showDlg();
      while (wd.nRun) { Fl::wait(0.2); };
      nRetCode = wd.nRetCode;
      delete wdlg;
      blockingTasks.Pause(false);
    }
    if (nRetCode)
    {
      fl_alert("Error");
    }
    
    
    
    // --- Part 3/4: copy list ----------------------------
    work.filelist.clear();
    {
      for (unsigned int cnt=0; cnt<ex.copy.size(); cnt++)
      {
        work.filelist.push_back(ex.copy[cnt]);
      }
    }
    
    wd.init(work.filelist.size()/2, (void*)&work);
    
    nRetCode = SimpleThread::Create(copyFiles, (void*)&wd);
    if (nRetCode)
    {
      std::cerr << "error creating thread\n";
      nRetCode = -1;
    }
    else
    {
      blockingTasks.Pause(true);
      WorkerDlg* wdlg = new WorkerDlg(&wd);
      wdlg->showDlg();
      while (wd.nRun) { Fl::wait(0.2); };
      nRetCode = wd.nRetCode;
      delete wdlg;
      blockingTasks.Pause(false);
    }
    if (nRetCode)
    {
      fl_alert("Error");
    }
    
    
    // --- Part 4/4: annotate list ------------------------
    setWaitCursor();
    ex.run_annotate();
    fl_cursor(FL_CURSOR_DEFAULT);              
  }
}

void UserInterface::ExportMeta(ItGroup* pGroup, std::vector<ItPicInGroup*> vListPics, ItGroup* pPicParent)
{
  ExportMetaDlg* dlg = new ExportMetaDlg();
  if (dlg->showDlg())
  {
    Export ex;
    
    ex.type        = Export::eMetaOnly;
    ex.structure   = Export::eFileSys;
    ex.fRecursive  = true;
    ex.fThumbs     = false;
    ex.fDatafile   = false;
    ex.fAddToJpeg  = true;
    ex.fPrefix     = false;
    ex.fHtml       = false;          
    
    ex.fKwInherit        = (dlg->sw_kwinherit   ->value() != 0);
    ex.fKwMerge          = (dlg->rb_kwmerge     ->value() != 0);
    ex.fCommentOverwrite = (dlg->sw_comoverwrite->value() != 0);
    
    ex.destdir     = myData->getBasepath();
    ex.data        = myData;
        
    std::vector<ItGroup*> listGroups;
    listGroups.push_back(lastPopupGroup);
    ex.run(vListPics, pPicParent, listGroups);
    
    if (ex.error.length())
    {
      fl_alert(ex.error.c_str());
    }
    else
    {
      // --- Part 1/1: annotate list ------------------------
      setWaitCursor();
      ex.run_annotate();
      fl_cursor(FL_CURSOR_DEFAULT);              
    }
  }
  delete dlg;
}
