// -*- mode: c; mode: fold -*-
// 
/*
 * Copyright (C) 2005, 2008-2009, 2011-2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "itempropdlg.h"

#include <iostream>
#include <string.h>
#include <FL/fl_ask.H>

#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"
#include "gui.h"
#include "dateselect.h"
#include "addkwdlg.h"
#include "../base/basetools.h"
#include "../base/pic.h"
#include "../base/group.h"

void kill_selection(Fl_Text_Editor* e) 
{
  if (e->buffer()->selected()) 
  {
    e->insert_position(e->buffer()->primary_selection()->start());
    e->buffer()->remove_selection();
  }
}

int kf_backspace_word(int, Fl_Text_Editor* e)
{
  if (!e->buffer()->selected())
  {
    Fl_Text_Buffer* buf = e->buffer();
    
    bool fGo = true;
    int nStep = 1;
    int nPosEnd = e->insert_position();
    // Delete until some word-character is left of cursor, or easier
    // with regard to UTF8: delete while there is space or punctuation 
    // left of cursor.
    e->move_left();
    int pos = e->insert_position();
    while (fGo && nStep)
    {
      int nChar = buf->byte_at(pos);
      if ((nChar & 0x80) == 0 && (isspace(nChar) || ispunct(nChar)))
        nStep = e->move_left();
      else
        fGo = false;
      pos = e->insert_position();
    }
    // Delete until some none-word-character is left of cursor, or 
    // easier with regard to UTF8: delete until there is space or
    // punctuation left of cursor.
    fGo = true;
    nStep = 1;
    while (fGo && nStep)
    {
      int nChar = buf->byte_at(pos);
      if ((nChar & 0x80) == 0 && (isspace(nChar) || ispunct(nChar)))
        fGo = false;
      else
        nStep = e->move_left();
      pos = e->insert_position();
    }
    if (nStep)
      e->move_right();
     
    e->buffer()->select(e->insert_position(), nPosEnd);
  }
  
  kill_selection(e);
  
  e->show_insert_position();
  e->set_changed();
  if (e->when()&FL_WHEN_CHANGED)
    e->do_callback();
  return 1;
}

void cb_edit(Fl_Widget* widget, void* userdata)
{
  const int nColMax = 80;

  Fl_Text_Editor* e = (Fl_Text_Editor*)(userdata);
  Fl_Text_Buffer* b = e->buffer();

  int pos = e->insert_position() - 1;
  if (pos < nColMax)
    return;
  
  //debug std::cout << "(" << e->insert_position() << ")  ";

  int linepos = 0;
  int nIndent = 0;
  {
    // search start of line
    while (pos > 0 && b->byte_at(pos) != '\n')
      pos--;
    if (pos > 0)
      pos++;
    if (b->byte_at(pos) != '|') // don't break table lines
    {
      int curpos = e->insert_position();
      bool fIndent = true;
      while (pos < curpos)
      {
        if (fIndent && UTF8::isspace(b->byte_at(pos)))
          nIndent++;
        else
          fIndent = false;
        if ((b->byte_at(pos) & 0x80) == 0)
        {
          //debug std::cout << itoHexStr(b->byte_at(pos) & 0xFF, 8) << " ";
          pos++;
        }
        else
        {
          //debug std::cout << itoHexStr(b->byte_at(pos) & 0xFF, 8);
          pos++; // skip first byte
          while ((b->byte_at(pos) & 0xC0) == 0x80)
          {
            //debug std::cout << itoHexStr(b->byte_at(pos) & 0xFF, 8);
            pos++;
          }
          //debug std::cout << " ";
        }
        linepos++;
      }
      //debug std::cout << "{" <<linepos<<"}";
    }    
  }
  
  //debug std::cout << "\n";
  if (linepos > nColMax)
  {
    pos = e->insert_position() - 1;
    while (pos > 0 && b->byte_at(pos) != '\n' && b->byte_at(pos) != ' ')
      pos--;
    if (b->byte_at(pos) == ' ')
      b->replace(pos, pos+1, "\n");
    for (; nIndent>0; nIndent--)
      b->insert(pos+1, " ");         
  }
}

void cb_close(Fl_Widget* widget, void* userdata)
{
  // force the user to click "cancel" or "apply"
}

void cb_ipd_cancel(Fl_Widget* widget, void* userdata)
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  
  std::cout << "ItemPropertyDlg: cancel (" << dlg << ")\n";
  
  dlg->window->hide();
  delete dlg;
}

void cb_ipd_addkw(Fl_Widget* widget, void* userdata) 
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  dlg->AddKw();
}

void cb_ipd_ok(Fl_Widget* widget, void* userdata)     
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  std::cout << "ItemPropertyDlg: ok (" << dlg << ")\n";
  if (dlg->saveValues() == 0)
  {
    dlg->checkErrors();
    dlg->window->hide();
    delete dlg;
  }
}

void cb_ipd_date(Fl_Widget* widget, void* userdata)   
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  std::cout << "ItemPropertyDlg: date (" << dlg << ")\n";
  
  int y, m, d;
  
  // das aktuelle Datum übergeben, falls es eins gibt:
  ItPic* pic = dlg->pPic;
  
  if (pic->hasDate())
  {
    JWWDateTime dt;
    
    dt.setDate(dlg->inp_date->value());
    
    d = dt.getDay();
    m = dt.getMonth();
    y = dt.getYear();    
  }
  else    
    d = -1;
  
  DateSelect* dateselect = new DateSelect(&y, &m, &d);
  
  dateselect->show();
  while (dateselect->visible())
    Fl::wait();
  
  if (d != -1)
  {    
    dlg->inp_date->value(JWWDateTime::getSomeDateAsString(JWWDateTime::assembleDate(y, m, d)).c_str());
  }
  delete dateselect;
}

ItemPropertyDlg::ItemPropertyDlg(ItGroup* myItem, TreeView* myTreeView)
{
  pPic     = 0;
  pGroup   = myItem;
  treeview = myTreeView;
  pMeta = dynamic_cast<ItMeta*>(pGroup);
}

ItemPropertyDlg::ItemPropertyDlg(ItPic* myItem, TreeView* myTreeView)
{
  pPic     = myItem;
  pGroup   = 0;
  treeview = myTreeView;
  pMeta = dynamic_cast<ItMeta*>(pPic);
}

ItemPropertyDlg::~ItemPropertyDlg()              
{
  std::cout << "ItemPropertyDlg: destructor (" << this << ")\n";
  
  inp_keywords->buffer(0);
  delete buf_keywords;
  
  inp_comment->buffer(0);
  delete buf_comment;
  
  Fl::delete_widget(window);
}

void ItemPropertyDlg::AddKw()
{
  std::set<std::string> kwset;        
  treeview->GetUserRoot()->kw_setGroup(kwset);
  AddKwDlg* dlg = new AddKwDlg(kwset);
  if (dlg->showDlg())
  {
    const char* kwstringptr = dlg->ti_words->value();
    if (kwstringptr != (const char*)0)
    {
      std::set<std::string> newkwlist = Keywords::KeywordList_Set(kwstringptr);
      
      if (!newkwlist.empty())
      {
        std::set<std::string>           oldkwlist = Keywords::KeywordList_Set(buf_keywords->text());
        std::set<std::string>::iterator it;  
        
        oldkwlist.insert(newkwlist.begin(), newkwlist.end());
        buf_keywords->remove(0, buf_keywords->length()); // clear buffer
        for (it = oldkwlist.begin(); it != oldkwlist.end(); it++)
        {
          buf_keywords->append((*it).c_str());
          buf_keywords->append(",\n");
        }                       
      }
    }
  }

  delete(dlg);
}

void ItemPropertyDlg::make_window()              
{
  int nY  = 0;
  int nTH = (UserInterface::opts->nFontSize*3)>>1;
  int nCH;
  const int nW = 750;
  
  unsigned int inside = 0;
  unsigned int insideandbelow = 0;
  
  ItGLinkRef* pGroupLink = dynamic_cast<ItGLinkRef*>(pGroup);
  
  {
    int nRows;
    
    if (pPic)
    {
      nRows = 15;
    }
    else
    {
      nRows = 14;
//todo      ((TreeGroupBase*)treeitem)->countNonGroupItems(inside, insideandbelow);
    }
    
    if (pGroupLink)
      nRows++;
    
    nCH = 550 - nRows * nTH;
  }
  
  window = new Fl_Double_Window(750, 550);
  window->label("Properties");
  
  if (pGroupLink)
  {
    std::string info = "This item is a link. It points to ";
    info += pGroupLink->pLink->GetID();
    out_linkinfo = new Fl_Output(0, nY, nW, nTH);
    out_linkinfo->value(info.c_str());
    nY += nTH;
  }
  
  //
  inp_title = new Fl_Input(0, nY+nTH, nW, nTH);
  if (pPic)
  {
    inp_title->value(pPic->GetPath().c_str());
    inp_title->label("Path:");
    inp_title->readonly(1);
  }
  else
  {
    inp_title->value(pGroup->GetName().c_str());
    namelabel = "Name:";
    
    namelabel += "         (";
    namelabel += itoStr(inside, ' ', 1);
    namelabel += "/";
    namelabel += itoStr(insideandbelow, ' ', 1);
    namelabel += ")";
    
    inp_title->label(namelabel.c_str());
    
    std::cout << "group " << pGroup->GetID() << " \"" << pGroup->GetName() << "\" has nRefCnt=" << pGroup->nRefCnt << "\n";
  }
  inp_title->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  nY += 2*nTH;
  
  // 
  buf_comment  = new Fl_Text_Buffer();
  buf_comment->append(pMeta->GetComment().c_str());
  inp_comment  = new Fl_Text_Editor(0, nY+nTH, nW, nCH);
  inp_comment->label("Comment:");
  inp_comment->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  inp_comment->buffer(buf_comment);  
  nY += nTH + nCH;    
  
  // 
  buf_keywords = new Fl_Text_Buffer();
  {
    std::vector<std::string> kwlist;
    pMeta->kw_list(kwlist);
    for (unsigned int n=0; n<kwlist.size(); n++)
      buf_keywords->append((kwlist[n] + ",\n").c_str());
  }
  inp_keywords = new Fl_Text_Editor(0, nY+nTH, nW-100, 6*nTH);
  inp_keywords->label("Keywords:");
  inp_keywords->buffer(buf_keywords);
  inp_keywords->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  btn_kwadd    = new Fl_Button(nW-100, nY+nTH, 100, nTH, "Add...");
  btn_kwadd->callback( (Fl_Callback*)cb_ipd_addkw, (void*)this );
  nY += (15*nTH)>>1;
  
  if (pPic)
  {
    int nDTW = nW / 5;
    
    btn_date    = new Fl_Button(0, nY, nDTW, nTH, "Date:");
    btn_date->callback( (Fl_Callback*)cb_ipd_date, (void*)this );
    
    //
    inp_date    = new Fl_Input(nDTW, nY, nDTW, nTH);
    inp_date->value(pPic->getDateString().c_str());
    
    //
    inp_time    = new Fl_Input(nW>>1, nY, nDTW, nTH);
    inp_time->value(pPic->getTimeString().c_str());
    inp_time->label("Time:");
    inp_time->align(FL_ALIGN_LEFT);
    nY += (3*nTH)>>1;
  }
  
  //
  btn_ID = new Fl_Button(nW>>2, nY, nW>>1, nTH, "ID");
  strcpy(IDString, "ID: ");
  strcat(IDString, pMeta->GetID().c_str());
  btn_ID->label(IDString);
  nY += nTH;
  
  window->end();
  window->resizable(inp_comment);
  
  //
  nY += nTH;
  btn_Cancel    = new Fl_Button(0, nY, 100, nTH, "Cancel");
  btn_Cancel->callback( (Fl_Callback*)cb_ipd_cancel, (void*)this );
  
  //
  btn_OK        = new Fl_Button(100, nY, 100, nTH, "&Apply");
  btn_OK->callback( (Fl_Callback*)cb_ipd_ok, (void*)this );
  
  window->add(btn_Cancel);
  window->add(btn_OK);
  
  window->resizable(inp_comment);
  
  window->set_modal();
  window->show();
  window->callback((Fl_Callback*)cb_close, (void*)this );
  
  if (pPic)
    Fl::focus(inp_comment);
  else
    Fl::focus(inp_title);
  
  checkErrors();
  
  {
    inp_comment->remove_key_binding(FL_BackSpace, FL_TEXT_EDITOR_ANY_STATE);
    inp_comment->add_key_binding(FL_BackSpace, 0, Fl_Text_Editor::kf_backspace);
    inp_comment->add_key_binding(FL_BackSpace, FL_CTRL, kf_backspace_word);
    
    inp_comment->callback( (Fl_Callback*)cb_edit, (void*)(inp_comment));
    inp_comment->when(FL_WHEN_CHANGED);
  }
}

int  ItemPropertyDlg::saveValues()               
{
  const char* text;
  int         nRetVal = 0;
      
  if (pGroup)
    pGroup->SetName(inp_title->value());
  
  text = buf_comment->text();
  pMeta->SetComment(text);
  //  free(text); // see docs! should be done!

  if (pPic)
  {
    if (pPic->SetDate(inp_date->value()))
    {
      fl_alert("Error in date format");
      nRetVal = -1;
    }
    if (pPic->SetTime(inp_time->value()))
    {
      fl_alert("Error in time format");
      nRetVal = -1;
    }
  }    
  
  {
    // clean up keywords
    std::string                     stringbuf = "";
    std::set<std::string>           kwlist    = Keywords::KeywordList_Set(buf_keywords->text());
    std::set<std::string>::iterator it;  
        
    for (it = kwlist.begin(); it != kwlist.end(); it++)
    {
      stringbuf.append(*it);
      stringbuf.append(std::string(","));
    }                       
    pMeta->SetKeywords(stringbuf.c_str());    
  }
  //  free(text); // see docs! should be done!    
  
  return(nRetVal);
}

void ItemPropertyDlg::checkErrors()              
{
#if 0 == 1 // todo! auf kaputte Links im Kommentar prüfen
  std::vector<unsigned int> ids;
  std::vector<unsigned int> idserr;
  LinkInfo*                 li = treeview->getLinkInfoList();
  
  treeitem->appendCommentIDs(ids);
  
  for (unsigned int n=0; n<ids.size(); n++)
  {
    if (li->getItemWithID(ids[n]) == (TreeItem*)0)
      idserr.push_back(ids[n]);
  }
  
  if (idserr.size())
  {
    std::string msg = "Link IDs to which no item exists:\n";
    for (unsigned int n=0; n<idserr.size(); n++)
    {
      msg += itoStr(idserr[n], ' ', 1) + "\n";
    }
    fl_alert(msg.c_str());
  }
#endif
}

