/*
 * Copyright (C) 2008-2009, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "kwreplacedlg.h"
#include <FL/fl_ask.H>
#include <cstring>
#include "texttreeitem.h"
#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"
#include "misc.h"
#include "../base/group.h"

void cb_browser0(TextTreeBrowser* sender, KwReplaceDlg* searchdlg)
{
  searchdlg->SetKw(0, sender->selected_path.c_str());
}

void cb_browser1(TextTreeBrowser* sender, KwReplaceDlg* searchdlg)
{
  searchdlg->SetKw(1, sender->selected_path.c_str());
}

KwReplaceDlg::KwReplaceDlg(ItGroup* group, ItGroup* root)
{
  this->group = group;
  this->root  = root;
  
  std::set<std::string> kwset;
  group->kw_setGroup(kwset);
  TextTreeItem* kwgroup = TextTreeItem::CreateFromSet(kwset);

  kwset.clear();
  root->kw_setGroup(kwset);
  TextTreeItem* kwroot  = TextTreeItem::CreateFromSet(kwset);
    
  { Fl_Double_Window* o = myWindow = new Fl_Double_Window(785, 550, "Replace keyword");
    o->user_data((void*)(this));
    { Fl_Input* o = ti_wordold = new Fl_Input(10, 30, 350, 25, "from");
      o->tooltip("Enter \'..\' to complete token shown in browser");
      o->align(FL_ALIGN_TOP);
    }
    { Fl_Input* o = ti_wordnew = new Fl_Input(370, 30, 405, 25, "to");
      o->tooltip("Enter \'..\' to complete token shown in browser");
      o->align(FL_ALIGN_TOP);
    }
    { TextTreeBrowser* o = brw_kwfrom = new TextTreeBrowser(10, 65, 350, 380, "", kwgroup);
      o->box(FL_NO_BOX);
      o->color(FL_BACKGROUND2_COLOR);
      o->selection_color(FL_SELECTION_COLOR);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->callback((Fl_Callback*)cb_browser0);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_RELEASE_ALWAYS);
    }
    { TextTreeBrowser* o = brw_kwto = new TextTreeBrowser(370, 65, 405, 380, "", kwroot);
      o->box(FL_NO_BOX);
      o->color(FL_BACKGROUND2_COLOR);
      o->selection_color(FL_SELECTION_COLOR);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->callback((Fl_Callback*)cb_browser1);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_RELEASE_ALWAYS);
      Fl_Group::current()->resizable(o);
    }
    { Fl_Check_Button* o = cb_wholeword = new Fl_Check_Button(10, 450, 245, 25, "Whole word");
      o->down_box(FL_DOWN_BOX);
    }
    btn_replace = new Fl_Button(615, 455, 160, 35, "Replace");
    btn_close = new Fl_Button(615, 500, 160, 35, "Close");
    o->end();
  }

  ti_wordold->when(FL_WHEN_CHANGED);
  ti_wordold->callback((Fl_Callback*)callback_text_input_to_texttreebrowser_comma);
  ti_wordold->user_data((void*)brw_kwfrom);
  ti_wordnew->when(FL_WHEN_CHANGED);
  ti_wordnew->callback((Fl_Callback*)callback_text_input_to_texttreebrowser_comma);
  ti_wordnew->user_data((void*)brw_kwto);  
  
  cb_wholeword->value(1);
  brw_kwfrom->user_data(this);
  brw_kwfrom->textsize(14);
  brw_kwto->user_data(this);
  brw_kwto->textsize(14);
}

KwReplaceDlg::~KwReplaceDlg()
{
  Fl::delete_widget(myWindow);
}

void KwReplaceDlg::showDlg()
{
  myWindow->set_modal();
  myWindow->show();
      
  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();
    
    if (!o) 
    {
      Fl::wait();
    }
    else if (o == btn_close || o == myWindow)
    {
      break;
    }
    else if (o == btn_replace)
    {
      bool fDo = false;
      std::string oldstr = UTF8::trim(ti_wordold->value());
      std::string newstr = UTF8::trim(ti_wordnew->value());
      
      if (oldstr.find(',') != std::string::npos || newstr.find(',') != std::string::npos)
      {
        fl_alert("A keyword must not contain ','.");
      }
      else
      {      
        if (newstr.length() == 0)
        {
          if (fl_choice("Really replace with nothing?", NULL, "Cancel", "Replace") == 2)
            fDo = true;
        }
        else if (oldstr.find(newstr) == 0)
        {
          if (fl_choice("Really replace with something less specific?", NULL, "Cancel", "Replace") == 2)
            fDo = true;
        }
        else
          fDo = true;
        
        if (fDo)
        {
          // replace
          group->kw_replaceGroup(oldstr, UTF8::trim(newstr), cb_wholeword->value());
          
          // update tree views
          std::set<std::string> kwset;
          group->kw_setGroup(kwset);
          brw_kwfrom->delData();
          brw_kwfrom->setData(TextTreeItem::CreateFromSet(kwset));
          
          kwset.clear();
          root->kw_setGroup(kwset);
          brw_kwto->delData();
          brw_kwto->setData(TextTreeItem::CreateFromSet(kwset));                
        }
      }
    }    
  }
  
  myWindow->hide();
}

void KwReplaceDlg::SetKw(int w, const char* item)
{
  Fl_Input* ti;
  if (w==0)
    ti = ti_wordold;
  else
    ti = ti_wordnew;
  
  if (strlen(item))
  {
    ti->value(item);
  }
}

