// -*- mode: c; mode: fold -*-
//
/*
 * Copyright (C) 2006, 2008, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "misc.h"

#include <iostream>
#include "../config.h"
#include "../shared/filetools.h"
#include "../shared/utf8.h"
#include "../shared/platform.h"
#include "texttreeitem.h"
#include <FL/Fl_Help_Dialog.H>

void GUIMisc::OpenHtml(std::string strPath)
{
  bool fDone = false;
  
  if (WIN32_VAL != 0 && !fDone)
  {
    if (Platform::system(strPath) == 0)
      fDone = true;
  }
  
  if (WIN32_VAL == 0 && !fDone)
  {
    if (system(("x-www-browser " + strPath).c_str()) == 0)
      fDone = true;
  }
  
  if (WIN32_VAL == 0 && !fDone)
  {
    if (system(("xdg-open " + strPath).c_str()) == 0)
      fDone = true;
  }
  
  if (!fDone)
  {
    Fl_Help_Dialog* dlg = new Fl_Help_Dialog();            
    dlg->load(strPath.c_str());
    dlg->show();
  }
}

int GUIMisc::OpenHelp(std::string strLang, std::string strFile)
{
  OpenHtml(GUIMisc::getHelpPath(strLang, strFile));
    
  return 0;
}

std::string GUIMisc::getHelpPath(std::string lang, std::string name)
{
  const char* installpaths[] = { "../share/doc/fotoalbum/user",          // win32
                                 STR_INSTALL_PREFIX "/" STR_DOC_PATH,    // unix
                                 "/usr/local/"          STR_DOC_PATH,    // unix, fallback
                                 "/opt/"                STR_DOC_PATH,    // unix, fallback
                                 "/usr/"                STR_DOC_PATH };  // unix, fallback
  
  if (lang.compare("C") == 0 || lang.length() == 0)
    lang = "en";
  
  std::string fpath;
  std::string anchor = "";
  
  {
    std::string::size_type pos = name.find("#");
    if (pos != std::string::npos)
    {
      anchor = name.substr(pos);
      name   = name.substr(0, pos);
    }
  }
  
  for (int m=0; m<2; m++)
  {
    if (m==1)
      lang = "en";
    
    for (unsigned int n=0; n<sizeof(installpaths)/sizeof(const char*); n++)
    {
      fpath = std::string(installpaths[n]) + "/" + lang + "/" + name;
      std::cout << "trying " << fpath << "\n";
      if (FileTools::fileExists(fpath))
        return(fpath + anchor);
    }
  }
  
  return(name + anchor);
}

void callback_text_input_to_texttreebrowser(Fl_Input* sender, TextTreeBrowser* tree, char sep)
{
  std::string::size_type pos, len;
  std::string            buf    = sender->value();
  int                    cursor = sender->position();
  bool                   fReplace = false;
  
  
  pos = buf.rfind(sep, cursor-1);
  if (pos != std::string::npos)
    buf = buf.substr(pos+1);
    
  pos = buf.find(sep);
  if (pos != std::string::npos)
    buf = buf.substr(0, pos);
  
  // leading whitespace is never part of keyword
  while (buf.length() && UTF8::isspace(buf[0]))
    buf = buf.substr(1);
  
  // if input ends in two dots: replace dots by matching part of tree
  len = buf.length();
  if (len > 2 && buf[len-1] == '.' && buf[len-2] == '.')
  {
    fReplace = true;
    buf = buf.substr(0, len-2);
  }
  
  if (buf.length())
    tree->show(buf);
  
  if (fReplace)
  {
    TextTreeItem* p = tree->getSelectedItem();
    if (p)
    {
      std::string::size_type pos2;
      
      std::string part = p->getText();
      buf = sender->value();
      // remove dots
      buf.replace(cursor-2, 2, "");
      cursor -= 2;
      //
      pos  = buf.rfind('.', cursor-1);
      pos2 = buf.rfind(',', cursor-1);
      if (pos != std::string::npos && pos2 != std::string::npos && pos2 > pos)
        pos = pos2;
      
      if (pos == std::string::npos)
        buf.replace(0, cursor, part);
      else
        buf.replace(pos+1, cursor-pos-1, part);
      sender->value(buf.c_str());
      // set cursor and mark
      if (pos == std::string::npos)
        sender->position(part.length());
      else
        sender->position(pos+1+part.length());
    }
  }
}

void callback_text_input_to_texttreebrowser_space(Fl_Input* sender, TextTreeBrowser* tree)
{
  callback_text_input_to_texttreebrowser(sender, tree, ' ');
}

void callback_text_input_to_texttreebrowser_comma(Fl_Input* sender, TextTreeBrowser* tree)
{
  callback_text_input_to_texttreebrowser(sender, tree, ',');
}

void callback_text_input_to_texttreebrowser_search(Fl_Input* sender, TextTreeBrowser* tree)
{
  char sep = '"';
  // Schauen, ob man innerhalb eines Strings ist (also eine ungerade Anzahl
  // von " vor dem Cursor steht). Wenn das so ist, kann item eingefügt werden.
  std::string strExpr = sender->value();
  std::string::size_type pOpen  = strExpr.rfind(sep, sender->position()-1);
  std::string::size_type pClose = strExpr.find(sep,  sender->position());
  
  int cnt = 0;
  for (std::string::size_type p=0; p<sender->position(); p++)
    if (strExpr[p] == sep)
      cnt++;
  if ((cnt & 1) != 0)
  {
    std::string::size_type len;
    std::string            buf    = sender->value();
    int                    cursor = sender->position();
    bool                   fReplace = false;
    
    buf = buf.substr(pOpen+1);
    if (pClose != std::string::npos)
      buf = buf.substr(0, pClose-pOpen-1);

    // leading whitespace is never part of keyword
    while (buf.length() && UTF8::isspace(buf[0]))
      buf = buf.substr(1);
    
    // if input ends in two dots: replace dots by matching part of tree
    len = buf.length();
    if (len > 2 && buf[len-1] == '.' && buf[len-2] == '.')
    {
      fReplace = true;
      buf = buf.substr(0, len-2);
    }
    
    if (buf.length())
      tree->show(buf);
    
    if (fReplace)
    {
      TextTreeItem* p = tree->getSelectedItem();
      if (p)
      {
        std::string::size_type pos;
      
        std::string part = p->getText();
        // remove dots
        strExpr.replace(cursor-2, 2, "");
        cursor -= 2;
        //
        pos  = strExpr.rfind('.', cursor-1);
        if (pos != std::string::npos && pOpen > pos)
          pos = pOpen;
      
        if (pos == std::string::npos)
          strExpr.replace(0, cursor, part);
        else
          strExpr.replace(pos+1, cursor-pos-1, part);
        sender->value(strExpr.c_str());
        // set cursor and mark
        if (pos == std::string::npos)
          sender->position(part.length());
        else
          sender->position(pos+1+part.length());
      }
    }
  }
}
