// -*- mode: c; mode: fold -*-
//
/*
 * Copyright (C) 2005-2007, 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "optionen.h"

#include <stdio.h>
#include <fstream>
#include <iostream>
#include <set>

#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"

const char* T_optionen::UIStyles[] = {"none", "plastic", "gtk+"};



T_optionen::T_optionen(const char* mypath)        
{
  SimpleXMLTransfer* cf;

  // --- 'Sichere' Vorgaben setzen -------------------------------------------- 
  pool = (SimpleXMLTransfer*)0;
  nWinWidth        = 600;
  nWinHeight       = 400;
  nFontSize        = 14;
  nTreeWidth       = 200;
  uThumbSize       = 150;
  flAspectRatio    = 1.0;
  fCreateThumbs    = true;
  nStartCnt        = -1;
  fShowInitialHelp = true;
  UIStyle          = "none";  
    
  // --- Versuchen, die Datei zu öffnen --------------------------------------- 
  if (mypath != (const char*)0)
    path = mypath;
  else
    path = "";

  // Wenn man die Datei nicht öffnen kann, den Default-Path benutzen.
  cf = (SimpleXMLTransfer*)0;
  try
  {
    cf = new SimpleXMLTransfer(path.c_str());
  }
  catch (XMLException e)
  {
    path = FileTools::getHomeDir("Fotoalbum") + ".options_c.xml";
    if (cf != (SimpleXMLTransfer*)0)
      delete cf;
    cf = (SimpleXMLTransfer*)0;
  }

  // Wenn sie mit dem ersten Pfad nicht geöffnet werden konnte, versuche ich es
  // jetzt mit dem zweiten.
  if (cf == (SimpleXMLTransfer*)0)
  {
    try
    {
      cf = new SimpleXMLTransfer(path.c_str());
    }
    catch (XMLException e)
    {
      // Es gibt keine gültige Datei
      cf = (SimpleXMLTransfer*)0;
    }
  }
  

  // --- Optionen lesen ------------------------------------------------------- 
  if (cf != (SimpleXMLTransfer*)0)
  {
    try
    {
      SimpleXMLTransfer* item;

      // Angaben aus der Datei lesen.
      item = cf->getChild("liveplotapp2.options.recentFiles");
      for (int n=0; n<item->getChildCount(); n++)
        recentFiles.push_back(item->getChildAt(n)->attribute("name"));

      nWinWidth     =  cf->getInt("windowstate.width");
      nWinHeight    =  cf->getInt("windowstate.height");
      nFontSize     =  cf->getInt("windowstate.fontsize");
      nTreeWidth    =  cf->getInt("windowstate.treewidth");
      uThumbSize    =  cf->getInt("base.nThumbnailSize");
      fCreateThumbs = (cf->getInt("base.fCreateThumbs") != 0);
      flAspectRatio = (float)(cf->getDouble("base.flAspectRatio"));

      // Some options have been introduced lately, they may not be there:
      if (cf->indexOfChild("misc") >= 0)
      {
        nStartCnt        =  cf->getInt("misc.nStartCnt");
        fShowInitialHelp = (cf->getInt("misc.fShowInitialHelp") != 0);
        UIStyle          =  cf->getString("misc.UIStyle");
      }

      // extract pool
      {
        int index = cf->indexOfChild("pool");
        if (index >= 0)
        {
          pool = cf->getChildAt(index);
          cf->removeChildAt(index);
        }
      }

      delete cf;
    }
    catch (XMLException e)
    {
      delete cf;
      cf     = (SimpleXMLTransfer*)0;
    }
  }

  
  // --- Some checks ----------------------------------------------------------
  if (nTreeWidth < 50)
    nTreeWidth = 50;  

  // --- pool muss vorhanden und gefüllt sein --------------------------------- 
  if (pool == (SimpleXMLTransfer*)0)
  {
    pool = new SimpleXMLTransfer();
    pool->setName("pool");

    pool->setAttribute("shrink.index",    "0");
    pool->setAttribute("shrink.quality",  "0.75"); // todo: which default to use?
    pool->setAttribute("shrink.size.w",   "800");
    pool->setAttribute("shrink.size.h",   "600");
  }
  
  {
    std::set<std::string> setSizes;
    std::set<std::string>::iterator it;
    setSizes.insert("800x600");
    setSizes.insert("1024x768");
    setSizes.insert("1280x1024");
    
    SimpleXMLTransfer* xSet = pool->getChild("shrink");
    for (int n=0; n<xSet->getChildCount(); n++)
    {
      SimpleXMLTransfer* xEntry = xSet->getChildAt(n);
      it = setSizes.find(xEntry->attribute("w") + "x" + xEntry->attribute("h"));
      if (it != setSizes.end())
        setSizes.erase(it);
    }
    for (it=setSizes.begin(); it!=setSizes.end(); it++)
    {
      std::string strEntry = *it;
      std::string::size_type pos = strEntry.find('x');
      SimpleXMLTransfer* tmp = new SimpleXMLTransfer();
      tmp->setName("size");
      tmp->setAttribute("w", strEntry.substr(0, pos));
      tmp->setAttribute("h", strEntry.substr(pos+1));
      xSet->addChild(tmp);
    }
  }  
  
  pool->makeSureAttributeExists("export.type",       "1");
  pool->makeSureAttributeExists("export.structure",  "2");
  pool->makeSureAttributeExists("export.recurse",    "1");
  pool->makeSureAttributeExists("export.followlinks","1");
  pool->makeSureAttributeExists("export.thumbs",     "0");
  pool->makeSureAttributeExists("export.datafile",   "0");
  pool->makeSureAttributeExists("export.addtojpeg",  "1");
  pool->makeSureAttributeExists("export.dir",        "");
  pool->makeSureAttributeExists("export.html",       "0");
  pool->makeSureAttributeExists("export.thumbsize",  "200");
  pool->makeSureAttributeExists("export.thumbqual",  "0.75"); // todo: what to use? 0.6 is bad.

  // Default is '0'. This is fast enough, nice and doesn't depend on
  // external programs:
  pool->makeSureAttributeExists("shrink.mode",       "0");
  pool->makeSureAttributeExists("thumbnail.mode",    "0");
  pool->makeSureAttributeExists("thumbnail.quality", "0.75"); // todo: what to use? 0.6 is bad.
  
  pool->makeSureAttributeExists("view.Fullscreen",   "1");
  pool->makeSureAttributeExists("view.PicsInTree",   "1");
}


T_optionen::~T_optionen()                         
{
  // Angaben in die Datei zurückschreiben
  SimpleXMLTransfer* cf = new SimpleXMLTransfer();
  SimpleXMLTransfer* group;
  SimpleXMLTransfer* item;

  cf->setAttribute("liveplotapp2.options.recentFiles.dummy", "0");
  group = cf->getChild("liveplotapp2.options.recentFiles");

  for (unsigned int n=0; n<recentFiles.size(); n++)
  {
    item = new SimpleXMLTransfer();
    item->setName("recentFile");
    item->setAttribute("name", recentFiles[n]);
    group->addChild(item);
  }

  cf->setAttribute("windowstate.width",     nWinWidth);
  cf->setAttribute("windowstate.height",    nWinHeight);
  cf->setAttribute("windowstate.fontsize",  nFontSize);
  cf->setAttribute("windowstate.treewidth", nTreeWidth);
  cf->setAttribute("base.nThumbnailSize",   uThumbSize);
  cf->setAttribute("base.flAspectRatio",    ftoStr(flAspectRatio, 1, 3));
  if (fCreateThumbs)
    cf->setAttribute("base.fCreateThumbs", "1");
  else
    cf->setAttribute("base.fCreateThumbs", "0");

  nStartCnt = (nStartCnt+1) & 0xFFFF;
  cf->setAttribute("misc.nStartCnt", nStartCnt);
  if (fShowInitialHelp)
    cf->setAttribute("misc.fShowInitialHelp", "1");
  else
    cf->setAttribute("misc.fShowInitialHelp", "0");
  cf->setAttribute("misc.UIStyle", UIStyle);

  cf->addChild(pool);

  // Datei schreiben
  cf->SaveToFile(path);
  delete cf;
}


void T_optionen::addRecentFile(const char* fname) 
{
  removeRecentFile(fname);

  // Wenn es mehr als zehn Einträge sind, den letzten löschen:
  while (recentFiles.size() > 10)
    recentFiles.pop_back();

  // Der erste soll der neueste sein:
  recentFiles.insert(recentFiles.begin(), fname);
}


void T_optionen::removeRecentFile(const char* fname) 
{
  // Wenn es die Datei gibt, löschen:
  for (unsigned int n=0; n<recentFiles.size(); n++)
    if (recentFiles[n].compare(fname) == 0)
    {
      recentFiles.erase(recentFiles.begin() + n);
      n = recentFiles.size();
    }
}


std::string T_optionen::recentFile(unsigned int nNr)       
{
  if (recentFiles.size() > nNr)
    return(recentFiles[nNr]);
  else
    return("");
}


int T_optionen::getNrOfStyles()
{
  return(sizeof(T_optionen::UIStyles) / sizeof(const char*));
}

SimpleXMLTransfer* T_optionen::getShrink()
{
  SimpleXMLTransfer* tmp = pool->getChild("shrink");
  return(tmp->getChildAt(tmp->attributeAsInt("index", 0)));
}
