/*
 * Copyright (C) 2005-2006, 2009, 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "optsdlg.h"

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "../shared/lib_conversions.h"
#include "../base/treeview.h"
#include "../config.h"


Fl_Double_Window* OptsDlg::make_window() {
  Fl_Double_Window* w;
  { Fl_Double_Window* o = new Fl_Double_Window(410, 380, "Options");
    w = o;
    o->user_data((void*)(this));
    { sw_initialHelp = new Fl_Check_Button(10, 10, 150, 25, "Show initial help");
      sw_initialHelp->down_box(FL_DOWN_BOX);
    } // Fl_Check_Button* sw_initialHelp
    { inp_fontsize = new Fl_Int_Input(10, 62, 74, 29, "Font size");
      inp_fontsize->align(FL_ALIGN_TOP_LEFT);
    } // Fl_Int_Input* inp_fontsize
    { cb_UIStyle = new Fl_Choice(175, 62, 172, 30, "UI Style");
      cb_UIStyle->down_box(FL_BORDER_BOX);
      cb_UIStyle->align(FL_ALIGN_TOP_LEFT);
    } // Fl_Choice* cb_UIStyle
    { inp_aspectratio = new Fl_Float_Input(10, 119, 74, 29, "Aspect Ratio");
      inp_aspectratio->align(FL_ALIGN_TOP_LEFT);
    } // Fl_Float_Input* inp_aspectratio
    { cb_DispSpeedQual = new Fl_Choice(175, 119, 172, 30, "Display speed/quality");
      cb_DispSpeedQual->down_box(FL_BORDER_BOX);
      cb_DispSpeedQual->align(FL_ALIGN_TOP_LEFT);
    } // Fl_Choice* cb_DispSpeedQual
    { Fl_Group* o = new Fl_Group(10, 175, 390, 125, "Thumbnails");
      o->box(FL_ENGRAVED_FRAME);
      o->align(FL_ALIGN_TOP_LEFT);
      { sw_thumbs = new Fl_Check_Button(20, 175, 140, 40, "Create Thumbnails");
        sw_thumbs->down_box(FL_DOWN_BOX);
      } // Fl_Check_Button* sw_thumbs
      { cb_thumbsize = new Fl_Choice(20, 260, 155, 30, "Thumbnail size");
        cb_thumbsize->down_box(FL_BORDER_BOX);
        cb_thumbsize->align(FL_ALIGN_TOP_LEFT);
      } // Fl_Choice* cb_thumbsize
      { cb_shrinkmode = new Fl_Choice(230, 205, 160, 30, "Shrink mode");
        cb_shrinkmode->down_box(FL_BORDER_BOX);
        cb_shrinkmode->align(FL_ALIGN_TOP_LEFT);
      } // Fl_Choice* cb_shrinkmode
      { sl_shrinkqual = new Fl_Value_Slider(230, 260, 160, 30, "jpeg quality");
        sl_shrinkqual->align(FL_ALIGN_TOP_LEFT);
      } // Fl_Value_Slider* sl_shrinkqual
      o->end();
    } // Fl_Group* o
    { btn_cancel = new Fl_Button(10, 330, 140, 40, "Cancel");
    } // Fl_Button* btn_cancel
    { btn_ok = new Fl_Return_Button(260, 330, 140, 40, "OK");
    } // Fl_Return_Button* btn_ok
    o->end();
  } // Fl_Double_Window* o
  return w;
}

OptsDlg::OptsDlg(T_optionen* myopts)
{
  opts = myopts;

  myWindow = make_window();
  
  if (opts->fShowInitialHelp)
    sw_initialHelp->value(1);
  else
    sw_initialHelp->value(0);
  
  inp_fontsize->value(itoStr(opts->nFontSize, ' ', 1).c_str());

  {
    int num = opts->getNrOfStyles();
    int index = -1;
    for (int cnt=0; cnt<num; cnt++)
    {
      cb_UIStyle->add(T_optionen::UIStyles[cnt]);
      if (opts->UIStyle.compare(T_optionen::UIStyles[cnt]) == 0)
        index = cnt;
    }
    if (index >= 0)
      cb_UIStyle->value(index);
  }

  inp_aspectratio->value(ftoStr(opts->flAspectRatio, 1, 5).c_str());

  cb_DispSpeedQual->add("Fast; low quality");
  cb_DispSpeedQual->add("Medium speed; mostly good quality");
  cb_DispSpeedQual->add("Slow; good quality");
  cb_DispSpeedQual->value(opts->pool->getInt("display.QualOverSpeed", 1));

  if (opts->fCreateThumbs)
    sw_thumbs->value(1);
  else
    sw_thumbs->value(0);

  {  
    int nIndex = -1;
    for (unsigned int n=0; TreeView::uThumbSizes[n] > 0; n++)
    {
      cb_thumbsize->add(itoStr(TreeView::uThumbSizes[n], ' ', 1).c_str());
      if (opts->uThumbSize == TreeView::uThumbSizes[n])
        nIndex = n;
    }      
    if (nIndex >= 0)
      cb_thumbsize->value(nIndex);
  }  

  cb_shrinkmode->add("Slow; high quality");
  cb_shrinkmode->add("Faster; good quality");
  cb_shrinkmode->add("Fast; medium quality");
  cb_shrinkmode->value(opts->pool->getInt("thumbnail.mode"));

  sl_shrinkqual->step(0.01);
  sl_shrinkqual->bounds(0.4, 1.0);
  sl_shrinkqual->value(opts->pool->getDouble("thumbnail.quality"));
  sl_shrinkqual->type(FL_HOR_NICE_SLIDER);  
}

OptsDlg::~OptsDlg()
{
  delete myWindow;
}

int OptsDlg::showDlg()
{
  int nRetCode;
  
  myWindow->set_modal();
  myWindow->show();
      
  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();
    
    if (!o) 
    {
      Fl::wait();
    }
    else if (o == btn_cancel || o == myWindow)
    {
      std::cout << "cancel\n";
      nRetCode = 0;
      break;
    }
    else if (o == btn_ok)
    {
      nRetCode = 1;
      break;
    }    
  }

  if (nRetCode)
  {
    opts->nFontSize        = atoi(inp_fontsize->value());
    opts->flAspectRatio    = atof(inp_aspectratio->value());
    opts->uThumbSize       = TreeView::uThumbSizes[cb_thumbsize->value()];
    opts->fCreateThumbs    = (sw_thumbs->value() != 0);
    opts->fShowInitialHelp = (sw_initialHelp->value() != 0);
    opts->UIStyle          = T_optionen::UIStyles[cb_UIStyle->value()];
    
    opts->pool->setAttributeOverwrite("thumbnail.quality", doubleToString(sl_shrinkqual->value()));
    opts->pool->setAttributeOverwrite("thumbnail.mode",    cb_shrinkmode->value());
    opts->pool->setAttributeOverwrite("display.QualOverSpeed", cb_DispSpeedQual->value());
    
    Fl::scheme(opts->UIStyle.c_str());
  }
  
  myWindow->hide();
  return(nRetCode);
}
