/*
 * Copyright (C) 2005, 2008, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "picbutton.h"

#include <iostream>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include "picgroup.h"
#include "../base/treeview.h"
#include "../base/pic.h"
#include "../shared/lib_conversions.h"

PicButton::PicButton(int X,int Y,int W,int H,const char *l)
  : Fl_Button(X,Y,W,H,l)
{
}

PicButton::~PicButton()
{
//  std::cout << "PicButton::~PicButton()\n";
}

void PicButton::draw()
{
  if (myPic->fMarked)
    fl_draw_box(FL_FLAT_BOX,   x(), y(), w(), h(), FL_SELECTION_COLOR);
  else    
    fl_draw_box(FL_BORDER_BOX, x(), y(), w(), h(), FL_BACKGROUND2_COLOR);
  draw_label();

  std::string strInfo = "";
  if (myPic->pPic->nRefCnt != 1)
    strInfo = itoStr(myPic->pPic->nRefCnt, ' ', 1);
  if (!strInfo.empty())
  {
    int nx = x() + w() - fl_width(strInfo.c_str()) - 3;
    int ny = y() + h() - (int)(fl_descent());
    if (1 == 1)
    {
      fl_color(FL_WHITE);
      fl_draw(strInfo.c_str(), nx+0, ny+0);
      fl_draw(strInfo.c_str(), nx+2, ny+0);
      fl_draw(strInfo.c_str(), nx+0, ny+2);
      fl_draw(strInfo.c_str(), nx+2, ny+2);
    }
    else
    {
      int off = 2;
      fl_draw_box(FL_ROUNDED_BOX, x()+off, y()+off, fl_width(strInfo.c_str())+4+off, fl_height()+2+off, FL_WHITE);
      nx += off;
      ny += off;
    }    
    fl_color(FL_BLACK);
    fl_draw(strInfo.c_str(), nx+1, ny+1);
  }
  
  if (myPic->pPic->vRating.size())
  {
    std::vector<int>& vRating = myPic->pPic->vRating;
    if (vRating[0] != ItPic::eRatingNone)
    {
      if (vRating[0] == ItPic::eRatingWorst)
        strInfo = 'X';
      else
        strInfo = itoStr(vRating[0], ' ', 1);
      int nx = x();
      int ny = y() + (int)(fl_height()) - (int)(fl_descent());
      fl_color(FL_WHITE);
      fl_draw(strInfo.c_str(), nx+0, ny+0);
      fl_draw(strInfo.c_str(), nx+2, ny+0);
      fl_draw(strInfo.c_str(), nx+0, ny+2);
      fl_draw(strInfo.c_str(), nx+2, ny+2);
      fl_color(FL_BLACK);
      fl_draw(strInfo.c_str(), nx+1, ny+1);
    }
    
    int nCnt = 0;
    int nWorst = ItPic::eRatingBest;
    int nBest  = ItPic::eRatingWorst;
    for (unsigned int n=1; n<vRating.size(); n++)
      if (vRating[n] != ItPic::eRatingNone)
      {
        nCnt++;
        if (nWorst > vRating[n])
          nWorst = vRating[n];
        if (nBest < vRating[n])
          nBest = vRating[n];
      }
    
    if (nCnt > 0)
    {
      strInfo = itoStr(nCnt, ' ', 1) + ",";
      if (nWorst == ItPic::eRatingWorst)
        strInfo += 'X';
      else
        strInfo += itoStr(nWorst, ' ', 1);
      strInfo += ",";
      if (nBest == ItPic::eRatingWorst)
        strInfo += 'X';
      else
        strInfo += itoStr(nBest, ' ', 1);
      {    
        int nx = x() + w() - fl_width(strInfo.c_str()) - 3;
        int ny = y() + (int)(fl_height()) - (int)(fl_descent());
        fl_color(FL_WHITE);
        fl_draw(strInfo.c_str(), nx+0, ny+0);
        fl_draw(strInfo.c_str(), nx+2, ny+0);
        fl_draw(strInfo.c_str(), nx+0, ny+2);
        fl_draw(strInfo.c_str(), nx+2, ny+2);
        fl_color(FL_BLACK);
        fl_draw(strInfo.c_str(), nx+1, ny+1);
      }
    }
  }
}

int PicButton::handle(int event)
{
  int X,Y,W,H; 
  
  X = x();
  Y = y();
  W = w();
  H = h();
  
  if (!Fl::event_inside(X,Y,W,H))
    return 0;
  
  switch (event) 
  {
   case FL_ENTER:
    return 1;
    break;
    
   case FL_PUSH:
    return 1;
    break;
    
   case FL_RELEASE:
    if (Fl::event_button() == FL_RIGHT_MOUSE)
    {
      // context menu      
      nMX = Fl::event_x();
      nMY = Fl::event_y();
      do_callback(this, CONTEXT);
    }
    else
    {
      if ((Fl::event_state() & FL_CTRL) != 0)
      {
        if (myPic != 0)
        {
//          std::cout << "toggle marked\n";
          myPic->fMarked = !myPic->fMarked;
          ((PicGroup*)parent())->ClickCtrl(myPic);
          redraw();
        }
      }
      else if ((Fl::event_state() & FL_SHIFT) != 0)
      {
        if (myPic != 0)
        {
//          std::cout << "toggle marked range\n";
          ((PicGroup*)parent())->ClickShift(myPic);
          redraw();
        }
      }
      else
      {
        nMX = Fl::event_x();
        nMY = Fl::event_y();
        do_callback(this, SHOW);
      }
    }
//    std::cout << "PicButton: end of handle\n";
    return 1;
    break;
  }
  
  return Fl_Button::handle(event);
}

void PicButton::item(ItPicInGroup* p)
{
  myPic = p;
}

void PicButton::redrawIf(ItPicInGroup* ptr)
{
  if (ptr == myPic)
    redraw();
}

void PicButton::setLabel(std::string somestring)
{
  mylabel = somestring;
  label(mylabel.c_str());
}
