// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2009, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */


#include "picgroup.h"

#include <iostream>
# include <FL/Fl.H>


#include "callback.h"
#include "picbutton.h"
#include "../base/treeview.h"
#include "../base/pic.h"


#define SCROLLBARWIDTH 20  // todo

PicGroup::PicGroup(int X,int Y,int W,int H,const char *l)
  : Fl_Scroll(X,Y,W,H,l)
{
  textbox  = (TextBox*)0;
  type(Fl_Scroll::VERTICAL_ALWAYS);
}

PicGroup::~PicGroup()
{
}

void PicGroup::resize(int x, int y, int w, int h) 
{
  std::cout << "void PicGroup::resize(" << x << ", " << y << ", " << w << ", " << h << ")\n";
  
  int nTBH = 0;
  
  if (textbox != (TextBox*)0)
  {
    textbox->recalcLayoutInto(x, y, w-SCROLLBARWIDTH, h);
    nTBH = textbox->h();
  }
  

  int        nSize         = children();  
  int        nPixelSize    = 0;
  Fl_Widget* widget;
  
  for (int n=0; n<nSize && nPixelSize == 0; n++)
  {
    widget = child(n);
    if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      nPixelSize = widget->w();
  }
  
  std::cout << "     PicGroup::resize:  nPixelSize  = " << nPixelSize << "\n";

  if (nSize > 0 && nPixelSize > 0)
  {    
    int nGap            = 5;
    int nScrollbarWidth = SCROLLBARWIDTH; // todo
    int nPicsPerRow     = (w - nScrollbarWidth)/ (nPixelSize+nGap);
    int m  = 0;
    int ly = nGap + y + nTBH;
      
    std::cout << "     PicGroup::resize:  nPicsPerRow = " << nPicsPerRow << "\n";
    std::cout << "     PicGroup::resize:  nTBH        = " << nTBH << "\n";
    
    for (int n=0; n<nSize; n++)
    {
      widget = child(n);
      
      if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      {
        widget->resize(x + nGap + m * (nPixelSize+nGap), ly, nPixelSize, nPixelSize);
        m++;
        if (m == nPicsPerRow)
        {
          m = 0;
          ly += nPixelSize + nGap;
        }
      }
    }
  }
    
  Fl_Scroll::resize(x, y, w, h);
}


void PicGroup::draw()                             
{
  Fl_Scroll::draw();
//  textbox->draw();
}


void PicGroup::setDescr(std::string newdescr)     
{
  if (textbox == (TextBox*)0)
  {
    textbox = new TextBox(x(), y(), w()-SCROLLBARWIDTH, h());
    textbox->labelsize(labelsize());
    // set up callback
    textbox->callback( (Fl_Callback*)cb_textbox, user_data());
  }

  textbox->setDescr(newdescr.c_str());
  textbox->recalcLayoutInto(x(), y(), w()-SCROLLBARWIDTH, h());
  insert(*textbox, 0);
}


void PicGroup::clear()                            
{
  Fl_Scroll::clear();
  textbox     = (TextBox*)0;
  pClickCtrl  = 0;
  pClickShift = 0;
}

void PicGroup::ClickShift(ItPicInGroup* ptr)         
{
  if (pClickCtrl == 0)
    return;
  
  int        nSize    = children();  
  int        nOK      = 0;
  Fl_Widget* widget;
  
  // Can I find both items?
  int nIdxShiftNew = -1;
  int nIdxShiftOld = -1;
  int nIdxCtrl     = -1;
  for (int n=0; n<nSize; n++)
  {
    widget = child(n);
    
    if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
    {
      if (((PicButton*)widget)->myPic == ptr)
      {
        nIdxShiftNew = n;
        nOK++;
      }
      else if (((PicButton*)widget)->myPic == pClickCtrl)
      {
        nIdxCtrl = n;
        nOK++;
      }
      else if (((PicButton*)widget)->myPic == pClickShift)
      {
        nIdxShiftOld = n;
        nOK++;
      }
    }
  }

  if (nOK == 3)
  {
    // unmark previous range
    int nA = nIdxCtrl;
    int nB = nIdxShiftOld;
    if (nA > nB)
    {
      nB = nIdxCtrl;
      nA = nIdxShiftOld;
    }
    for (int n=nA; n<=nB; n++)
    {
      widget = child(n);
      if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      {
        ItPicInGroup* i = ((PicButton*)widget)->myPic;        
        i->fMarked = false;        
        widget->redraw();
      }
    }    
  }
    
  if (nOK > 1)
  {
    pClickShift = ptr;
    
    // mark new range
    int nA = nIdxCtrl;
    int nB = nIdxShiftNew;
    if (nA > nB)
    {
      nB = nIdxCtrl;
      nA = nIdxShiftNew;
    }
    for (int n=nA; n<=nB; n++)
    {
      widget = child(n);
      if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      {
        ItPicInGroup* i = ((PicButton*)widget)->myPic;        
        i->fMarked = true;
        widget->redraw();
      }
    }    
  }
}

void PicGroup::ClickCtrl(ItPicInGroup* pPic)
{
  pClickCtrl  = pPic;
  pClickShift = 0;
}
