// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005,2008, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "pictreebrowser.h"


#include "../base/group.h"
#include "gui.h"

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <iostream>


PicTreeBrowser::PicTreeBrowser(int X,int Y,int W,int H,const char*l)
: Fl_Browser_(X,Y,W,H,l) 
{
  type(FL_HOLD_BROWSER);
  
  itemSelected = 0;
}

void* PicTreeBrowser::item_first() const                    
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->pGRoot));
  else
    return((void*)0);
}


void* PicTreeBrowser::item_next(void *l) const              
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->browserItemNext((ItGroup*)l)));
  else
    return((void*)0);
}


void* PicTreeBrowser::item_prev(void *l) const              
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->browserItemPrev((ItGroup*)l)));
  else
    return((void*)0);
}


int   PicTreeBrowser::item_height(void *l) const            
{
  // Do not display root group
  if ( ((ItGroup*)l)->pParent == 0)
    return 0;
  else
    return(nItemHeight);
}


void  PicTreeBrowser::textsize(uchar s)                     
{
  Fl_Browser_::textsize(s);
  
  // calculate things  
  nItemHeight = (10*s) >> 3;
  
  // Das Quadrat sollte eine ungerade Höhe haben,
  // damit das Kreuz darin wirklich in der Mitte ist.
  nSizeBox = s;
  if ((nSizeBox & 1) == 0)
    nSizeBox--;
  
  nLengthDash = (s * 3) >> 3;  
  if ((nLengthDash & 1) == 0)
    nLengthDash--;
  nD          = (( (10*s) >> 3) - nSizeBox) >> 1;
  nTextsize   = s;
  nGroupAdd   = (s * 3) >> 1;
  
  std::cout << "PicTreeBrowser layout:\n";
  std::cout << "  nTextsize   = " << nTextsize << "\n";
  std::cout << "  nItemHeight = " << nItemHeight << "\n";
  std::cout << "  nSizeBox    = " << nSizeBox << "\n";
  std::cout << "  nLengthDash = " << nLengthDash << "\n";
  std::cout << "  nD          = " << nD << "\n";
  std::cout << "  nGroupAdd   = " << nGroupAdd << "\n";
}


int   PicTreeBrowser::handle(int e)                         
{
  ItGroup *l;
  
  // get box INSIDE the scrollbars
  int X,Y,W,H;  
  bbox(X,Y,W,H);
  
  switch (e) 
  {
    case FL_PUSH:
      // to make the Fl_Browser_ handle the scrollbars:
      if (
          !Fl::event_inside(X,Y,W,H)
         )
        break;
      
      return 1;
      break;
      
      // only to make sure Fl_Browser_ will not handle it
    case FL_DRAG:    
      return(1);    
      
    case FL_RELEASE:
      l = (ItGroup*)find_item(Fl::event_y());
      if (l != (ItGroup*)0)
      {
        int  nXRelease = Fl::event_x() + hposition() - leftedge();
        bool fRedraw   = false;
        bool fContext  = false;
        int  nXBoxMin  = 3 + (l->treeDepth()-1) * 2 * nTextsize;
        int  nXBoxMax  = nXBoxMin + nTextsize;
        
        if (Fl::event_button() == FL_RIGHT_MOUSE)
        {
          fContext = true;
        }
        else
        {
          if (l->pParent != (ItGroup*)0 &&
              nXRelease >= nXBoxMin &&
              l->canExpand())
          {
            if (nXRelease >= nXBoxMax)
            {
              l->fExpanded = true;
            }
            else
            {
              l->fExpanded = !l->fExpanded;
            }
            fRedraw = true;
          }        
        }            
                
        if (nXRelease >= nXBoxMax && !fContext)
        {
          // select
          // Mark item as selected
          itemSelected = l;
          //          
          do_callback();
          fRedraw = true;
        }
        
        if (fRedraw)
          redraw();
        
        if (fContext)
          pGUI->showContextMenuGroup(Fl::event_x(), Fl::event_y(), l);
        
        return(1);
      }
  }
  return Fl_Browser_::handle(e);
}


int   PicTreeBrowser::item_width(void *v) const             
{
  ItGroup* item     = (ItGroup*)v;
  int       X        = 3 + (item->treeDepth()-1) * 2 * nTextsize;

  X += nGroupAdd;
    
  return( (int)(fl_width( item->GetName().c_str() ))         + X);
}


void  PicTreeBrowser::item_draw(void *v, 
                                int X, int Y, 
                                int, int) const             
{
  ItGroup* item = (ItGroup*)v;
  int nTD = item->treeDepth() - 1;
  // do not display root group
  if (nTD >= 0)
  {
    Fl_Color lcol = textcolor();
    {
      if (item == itemSelected)
      {
        fl_draw_box(FL_FLAT_BOX, X, Y, w(), nTextsize+fl_descent(), FL_SELECTION_COLOR);
        lcol = fl_contrast(lcol, FL_SELECTION_COLOR);
      }
      else    
        fl_draw_box(FL_FLAT_BOX, X, Y, w(), nTextsize+fl_descent(), FL_BACKGROUND2_COLOR);
    }    
    fl_color(lcol);
    
    X += 3;
    
    fl_line_style(0);
    
    X += nTD * 2 * nTextsize;
    
    if (true)
    {
      ItGroup* g        = (ItGroup*)item;        
      
      int  nMitte   = (nSizeBox>>1);
      int  nAbstand = (nSizeBox - nLengthDash)>>1;        
      
      fl_rect(X, Y+nD, nSizeBox, nSizeBox);
      
      if (g->canExpand())
      {
        if (g->fExpanded)
          fl_line(X+nAbstand, Y+nD+nMitte,   X+nAbstand+nLengthDash-1, Y+nD+nMitte);
        else
        {
          fl_xyline(X+nAbstand, Y+nD+nMitte,   X+nAbstand+nLengthDash-1);
          fl_yxline(X+nMitte,   Y+nD+nAbstand, Y+nD+nAbstand+nLengthDash-1);
        }
      }
      X += (nTextsize * 3) >> 1;
    }
    
    if (dynamic_cast<ItGLinkRef*>(item) != 0 || item->SearchType() > 0)
      fl_font(textfont() | FL_ITALIC, nTextsize);
    else
      fl_font(textfont(), nTextsize);
    fl_draw(item->GetName().c_str(),
            X, Y + nTextsize - ((int)(fl_descent())>>2));
  }
}
