/*
 * Copyright (C) 2005, 2008-2009, 2012  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include <stdio.h>
#include <cstring>
#include <iostream>
#include <FL/fl_ask.H>
#include "searchdlg.h"
#include "texttreeitem.h"
#include "misc.h"
#include "../base/searchprog.h"
#include "../base/group.h"
#include "../base/treeview.h"

void cb_browser(KWSRCHBROWSER* sender, SearchDlg* searchdlg)
{
  printf("search dialog is %p\n", searchdlg);
  printf("browser is       %p\n", sender);
  
  searchdlg->AddKw(sender->selected_path.c_str());
}

void cb_SrcGroup(void* bla, SearchDlg* searchdlg)
{
  const char* newnameptr = fl_input("ID of source group", searchdlg->pSrcGroup->GetID().c_str());
  if (newnameptr != (const char*)0)
    searchdlg->SetSource(newnameptr);
}

void cb_Help(void* bla, SearchDlg* searchdlg)
{
  GUIMisc::OpenHelp("", "fa_ht_search.htm");
}

void SearchDlg::SetSource(const char* srcID)
{
  ItGroup* p = pTreeView->GetGroupPtr(srcID);
  if (p)
  {
    pSrcGroup = p;
    pNameStr = (char*)malloc(p->GetName().length()+1);
    strcpy(pNameStr, p->GetName().c_str());
    btn_src->label(pNameStr);
  }
  else
  {
    fl_alert("No such group");
  }
}

void SearchDlg::AddKw(const char* item)
{
  if (strlen(item))
  {
    // Schauen, ob man innerhalb eines Strings ist (also eine ungerade Anzahl
    // von " vor dem Cursor steht). Wenn das so ist, kann item eingefügt werden.
    std::string strExpr = ti_expr->value();
    std::string::size_type pOpen  = strExpr.rfind('"', ti_expr->position()-1);
    std::string::size_type pClose = strExpr.find('"',  ti_expr->position());
    
    int cnt = 0;
    for (std::string::size_type p=0; p<ti_expr->position(); p++)
      if (strExpr[p] == '"')
        cnt++;
    if ((cnt & 1) != 0)
    {
      std::string buf = strExpr.substr(0, pOpen+1) + item;
      
      if (pClose != std::string::npos)
        buf += strExpr.substr(pClose);
        
      ti_expr->value(buf.c_str());
      ti_expr->position(pOpen + 1 + strlen(item));
    }
  }
}

SearchDlg::SearchDlg(std::set<std::string>& kwset, TreeView* pTreeView, ItGroup* pSrc)
{
  pLog = 0;
  this->pTreeView = pTreeView; 
  TextTreeItem* root = TextTreeItem::CreateFromSet(kwset);
  
  printf("Baum erzeugt\n");
  
  { myWindow = new Fl_Double_Window(785, 545, "Search");
    myWindow->user_data((void*)(this));
    { ti_expr = new Fl_Input(15, 30, 755, 25, "Search expression");
      ti_expr->align(Fl_Align(FL_ALIGN_TOP_LEFT));
      ti_expr->tooltip("Enter \'..\' to complete token shown in browser");
    } // Fl_Input* ti_expr
    { brw_kws = new TextTreeBrowser(15, 65, 450, 470, "", root);
      brw_kws->box(FL_NO_BOX);
      brw_kws->color(FL_BACKGROUND2_COLOR);
      brw_kws->selection_color(FL_SELECTION_COLOR);
      brw_kws->labeltype(FL_NORMAL_LABEL);
      brw_kws->labelfont(0);
      brw_kws->labelsize(14);
      brw_kws->labelcolor(FL_FOREGROUND_COLOR);
      brw_kws->callback((Fl_Callback*)cb_browser);
      brw_kws->align(Fl_Align(FL_ALIGN_BOTTOM));
      brw_kws->when(FL_WHEN_RELEASE_ALWAYS);
      Fl_Group::current()->resizable(brw_kws);
    } // Fl_Select_Browser* brw_kws
    { inp_name = new Fl_Input(470, 335, 300, 25, "Name of search");
      inp_name->align(Fl_Align(FL_ALIGN_TOP_LEFT));
    } // Fl_Input* inp_name
    { btn_src = new Fl_Button(470, 375, 300, 35, "src group");
    } // Fl_Button* btn_src
    { cb_flat = new Fl_Check_Button(470, 420, 300, 25, "Flat result, no duplicates");
      cb_flat->down_box(FL_DOWN_BOX);
    } // Fl_Check_Button* cb_flat
    { btn_help = new Fl_Button(470, 455, 145, 35, "Help");
    } // Fl_Button* btn_help
    { btn_explain = new Fl_Button(625, 455, 145, 35, "Explain");
    } // Fl_Button* btn_explain
    { btn_cancel = new Fl_Button(470, 500, 145, 35, "Cancel");
    } // Fl_Button* btn_cancel
    { btn_search = new Fl_Return_Button(625, 500, 145, 35, "Search");
    } // Fl_Return_Button* btn_search
    myWindow->end();
  } // Fl_Double_Window* myWindow
  
  btn_src->callback((Fl_Callback*)cb_SrcGroup, this);
  btn_help->callback((Fl_Callback*)cb_Help, this);
  
  ti_expr->when(FL_WHEN_CHANGED);
  ti_expr->callback((Fl_Callback*)callback_text_input_to_texttreebrowser_search);
  ti_expr->user_data((void*)brw_kws);
  
  brw_kws->user_data(this);
  brw_kws->textsize(14);
  
  cb_flat->value(0);  
  
  pSrcGroup = pSrc;
  if (pSrc)
  {
    pNameStr = (char*)malloc(pSrc->GetName().length()+1);
    strcpy(pNameStr, pSrc->GetName().c_str());
  }
  else
  {
    pNameStr = (char*)malloc(1);
    strcpy(pNameStr, "");
  }
  btn_src->label(pNameStr);
}

SearchDlg::~SearchDlg()
{
  delete myWindow;
  free(pNameStr);
}

int SearchDlg::showDlg()
{
  int nRetCode;
  
  myWindow->set_modal();
  myWindow->show();
      
  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();
    
    if (!o) 
    {
      Fl::wait();
    }
    else if (o == btn_cancel || o == myWindow)
    {
      nRetCode = 0;
      break;
    }
    else if (o == btn_explain || o == btn_search)
    {
      // check expression
      SearchProg* sp = new SearchProg(ti_expr->value());
      std::string strErr = sp->GetErr();
      std::string strLog = sp->GetLog();
      delete sp;
      if (strErr.length() == 0 && o == btn_search)
      {
        nRetCode = 2;
        break;
      }
      else
      {
        if (strErr.length())
          strErr = "Error: " + strErr;
        
        int xx = myWindow->x();
        int yy = myWindow->y();
        int ww = myWindow->w();
        int hh = myWindow->h();
        myWindow->hide(); // necessary to let the help dialog receive events although the modal search window is there
        // show compile log
        if (pLog == 0)
          pLog = new Fl_Help_Dialog();
        
        std::cout << strLog << "\n";
        
        pLog->value((strLog + "<p>" + strErr + "</p>").c_str());
        pLog->show();
        pLog->resize(xx, yy, ww, hh);
        do
        {
          Fl::readqueue();
          Fl::wait();
        }
        while (pLog->visible());
        //
        myWindow->show();
      }
    }    
  }
  
  if (pLog)
    pLog->hide();
  
  myWindow->hide();
  return(nRetCode);
}
