// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2008, 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "texttreeitem.h"
#include <iostream>
#include "../shared/lib_conversions.h"
#include "../shared/utf8.h"

TextTreeItem* TextTreeItem::CreateFromSet(std::set<std::string>& kwset)
{
  TextTreeItem* root = new TextTreeItem(0, 0, "Keyword hierarchy browser");
  
  std::set<std::string>::iterator i = kwset.begin();
  while (i != kwset.end())
  {
    std::string item = (*i);
    i++;
    root->Add(item);
  }
  
  // root->print(std::cout);
  
  return(root);
}

void TextTreeItem::print(std::ostream& out)
{
  for (int n=0; n<treeDepth(); n++)
    out << "  ";
  out << getText();
  {
    TextTreeItem* p = (TextTreeItem*)item_prev();
    if (p)
      out << " p " << p->getText();
  }
  {
    TextTreeItem* p = (TextTreeItem*)item_next();
    if (p)
      out << " n " << p->getText();
  }
  out << "\n";
  
  TextTreeItem* p = itemFirstChild;
  
  while (p)
  {
    p->print(out);
    p = p->itemNext;
  }
}

void TextTreeItem::Add(std::string path)
{
  if (path.length())
  {
    std::string::size_type pos = path.find('.');
    TextTreeItem* ptr = itemFirstChild;
    
    if (pos != std::string::npos)
    {
      std::string firstpart = path.substr(0, pos);
      std::string rest      = path.substr(pos + 1);
      bool fDone = false;
      
      if (itemFirstChild == 0)
      {
        itemFirstChild = new TextTreeItem(0, this, firstpart);
        itemFirstChild->Add(rest);
      }
      else
      {
        while (ptr)
        {
          if (ptr->getText().compare(firstpart) == 0)
          {
            ptr->Add(rest);
            fDone = true;
            break;
          }
          else
          {
            ptr = ptr->itemNext;
          }
        }
        
        if (!fDone)
        {
          ptr = itemFirstChild;
          while (ptr->itemNext)
            ptr = ptr->itemNext;
          TextTreeItem* item = new TextTreeItem(ptr, this, firstpart);
          item->Add(rest);
        }
      }
    }
    else
    {
      if (itemFirstChild == 0)
      {
        itemFirstChild = new TextTreeItem(0, this, path);
      }
      else
      {
        while (ptr->itemNext)
          ptr = ptr->itemNext;
        new TextTreeItem(ptr, this, path);
      }
    }
  }
}

TextTreeItem::TextTreeItem(TextTreeItem* prev, TextTreeItem* parent, std::string mytext)
{
  itemPrev       = prev;
  itemNext       = 0;
  itemParent     = parent;
  itemFirstChild = 0;
  
  if (0)
  {
    std::cout << "creating " << mytext << "  ";
    {
      TextTreeItem* p = parent;
      while (p)
      {
        std::cout << p->getText() << " . ";
        p = p->itemParent;
      }
    }
    std::cout << "\n";
  }
  
  if (itemPrev != 0)
    itemPrev->itemNext = this;
  if (itemParent != 0 && itemPrev == 0)
    itemParent->itemFirstChild = this;

  text = mytext;
  
  if (parent == 0)
    fExpanded = true;
  else
    fExpanded = false;
}

TextTreeItem::~TextTreeItem()
{
  if (itemFirstChild)
    delete itemFirstChild;
  
  if (itemNext)
    delete itemNext;
}

int  TextTreeItem::treeDepth()                                           
{
  int           nNum = 0;
  TextTreeItem* item = itemParent;
  
  while (item != (TextTreeItem*)0)
  {
    item = item->itemParent;
    nNum++;
  }
  
  return(nNum);
}

void* TextTreeItem::item_next(bool fIncludingChildren)
{
  if (itemFirstChild != 0 && fExpanded == true && fIncludingChildren)
  {
    // The group is expanded. So it does contain at least one item which can be displayed.
    return(itemFirstChild);
  }
  else if (itemNext)
    return(itemNext);
  else if (itemParent)
    return(itemParent->item_next(false));
  else
    return(0);
}

void* TextTreeItem::item_prev()
{
  if (itemPrev == 0)
    return(itemParent);
  else
  {
    TextTreeItem* p = itemPrev;
    
    while (p->itemFirstChild && p->fExpanded)
    {
      p = p->itemFirstChild;
      
      while (p->itemNext)
        p = p->itemNext;    
    }
    
    return(p);
  }
}

std::string TextTreeItem::getPath()
{
  TextTreeItem* p = itemParent;
  std::string tmp = getText();
  
  while (p)
  {
    if (p->itemParent)
      tmp = p->getText() + "." + tmp;
    p = p->itemParent;
  }
  return(tmp);
}

TextTreeItem* TextTreeItem::find(std::string path)
{
  std::string::size_type pos = path.find('.');
  std::string name, tmpstr;
  
  if (path.length() == 0)
  {
    if (itemFirstChild)
      fExpanded = true;
    return(this);
  }
  
  if (pos == std::string::npos)
    name = path;
  else
    name = path.substr(0, pos);
  
  // std::cout << "I am *" << text << "*, asked *" << path << "*, *" << name << "*\n";
  
  // do I have a child with that name?
  TextTreeItem* p = itemFirstChild;
  while (p)
  {
    tmpstr = UTF8::to_lower(p->getText());
    if (tmpstr.find(name) == 0)
    {
      if (pos == std::string::npos)
        return(p);
      else if (tmpstr.compare(name) == 0)
      {
        p = p->find(path.substr(pos+1));
        return(p);
      }        
    }
    p = p->itemNext;
  }
  
  return(p);  
}
