/*
 * Copyright (C) 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TEXTTREEITEM_H
# define TEXTTREEITEM_H

# include <string>
# include <set>

class TextTreeItem
{
public:
  
  static TextTreeItem* CreateFromSet(std::set<std::string>& kwset);

  /**
   * 
   */
  TextTreeItem(TextTreeItem* prev, TextTreeItem* parent, std::string mytext);
  
  
  ~TextTreeItem();
  
  /**
   *
   */
  std::string getText() { return(text); };
  std::string getPath();
  
  /**
   * Previous item in the same group.
   */
  TextTreeItem* itemPrev;
  
  /**
   * Next item in the same group.
   * This is public to allow for fast group operations.
   */
  TextTreeItem* itemNext;   
  
  /**
   * Pointer to the TreeGroupBase to which I belong.
   * this is public to allow for a fast @ref treeDepth() (without 
   * recursive method calls).
   */
  TextTreeItem* itemParent;
  
  
  TextTreeItem* itemFirstChild;
  

  void* item_next(bool fIncludingChildren = true);
  void* item_prev();
  
  /**
   * Jedes Element muss wissen, wie weit es eingerückt werden
   * muss. Dazu muss man die Anzahl der Schritte über @ref itemParent bis zum
   * Null-Element zählen. Es werden nicht viele gleichzeitig dargestellt, daher
   * kann man das jedesmal zählen. Alternativ könnte sich auch jede @ref TreeGroupBase
   * diese Entfernung merken.
   */
  int treeDepth();
  
  bool fExpanded;

  void Add(std::string path);

  void print(std::ostream& out);

  TextTreeItem* find(std::string path);
  
private:
  std::string text;
};

#endif
