/*
 * Copyright (C) 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */
#include "asciidoc.h"

#include "utf8.h"

std::string AsciiDoc::ToHtml(std::string in)
{
  std::string out = "";
  int nState = 0;
  int nNewline = 0;
  int nSpaces;
  bool fFollowLineBreak = false;
    
  for (std::string::size_type n=0; n<in.length(); n++)
  {
    if (nState == 0 && UTF8::isspace(in[n]) == false) 
    {
      nState = 1;
      out += "<p>";
    }
    
    switch (nState)
    {
      case 0:
        // see above
        break;
        
      case 1:
        if (in[n] == '\n')
        {
          nNewline = 1;
          nSpaces = 0;
          nState = 2;
        }
        break;
        
      case 2:        
        if (in[n] == '|')
        {
          out += "</p>\n<table cellspacing=\"0\" cellpadding=\"2\" rules=\"all\" frame=\"border\">\n";
          // read first line: "|======================"
          while (in[n] != '\n')
            n++;
          
          int nTabState = 1;
          for (; n<in.length() && nTabState >= 0; n++)
          {
            switch (nTabState)
            {
              case 1:
                if (in[n] == '|')
                {
                  out += "<tr><td>";
                  nTabState = 2;
                }
                break;
                
              case 2:
                if (in[n] == '\n')
                {
                  nTabState = 3;
                }
                else
                {
                  if (in[n] == '|')
                    out += "</td>\n<td>";
                  else
                    out += in[n];
                }
                break;
                
              case 3:
                if (in[n] == '|')
                {
                  out += "</td></tr>\n";                  
                  nTabState = 4;
                }
                else
                {
                  out += " ";
                  out += in[n];                  
                  nTabState = 2;
                }
                break;
                
              case 4:
                if (in[n] == '=')
                {
                  while (in[n] == '=' && n < in.length())
                    n++;                  
                  nTabState = -1;
                }
                else
                {
                  out += "<tr><td>";
                  nTabState = 2;
                }
                break;
            }
          }
                    
          out += "</table>\n<p>\n";
          if (n<in.length() && UTF8::isspace(in[n]) == false) 
            nState = 1;
          else
          {
            nNewline = 1;
            nState = 2;
          }
        }
        else if (in[n] == '\n')
        {
          nNewline++;
          nSpaces = 0;
        }
        else if (UTF8::isspace(in[n]) == false) 
        {
          if (fFollowLineBreak)
          {            
            if (nNewline >= 2)
              out += "</p>\n<p>\n";
            else
              out += "<br/>\n";
            fFollowLineBreak = false;
          }          
          else if (nNewline >= 2)
            out += "</p>\n<p>\n";
          else
            out += " ";
          
          if (nSpaces > 0)
            fFollowLineBreak = true;
          
          nState = 1;
        }
        else
          nSpaces++;
        break;      
    }
        
    if (nState == 1)
      out += in[n];
  }
  
  if (nState != 0)
    out += "</p>\n";

  return out;
}
