// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2006  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "calendar.h"

#include <time.h>

int Tag_des_Jahres(int tag, int monat, int jahr) 
{
  const int DaysToMonth[12] = {0, 31, 59, 90, 120, 151, 
                               181, 212, 243, 273, 304, 334};
  
  int Tage = DaysToMonth[monat-1] + tag;
  
  if (monat > 2 && (((jahr % 4 == 0) && (jahr % 100 != 0)) || jahr % 400 == 0))
    Tage++;
  
  return Tage;
} 


int CurrentDoomDay(int jahr) 
{
   const int CenturyDoomDay[4] = {5,4,2,0};
   int CC             = (jahr/100);
   int YY             = jahr%100;
   int CCDoomDay      = CenturyDoomDay[CC%4];
   int YYDoomDay      = 0;

   if (YY == 0)
    YYDoomDay = CCDoomDay;
   else
    {
      if (YY%12 == 0)
        YYDoomDay = (CCDoomDay + YY/12 - 1) % 7;
      else YYDoomDay = ( CCDoomDay
                         + ((YY/12)
                            + (YY%12)
                            + (((YY-1)%12)/4))
                         ) % 7;
    }
  if ((CC%4 == 0) && (YY != 0))
    YYDoomDay = YYDoomDay + 1;
  
  return YYDoomDay;
} 


int Wochentag(int tag, int monat, int jahr) 
{
  return((CurrentDoomDay(jahr)+((Tag_des_Jahres(tag,monat,jahr) - 1)%7))%7);
}


int Kalenderwoche(int tag, int monat, int jahr) 
{
  int Woche      = 0;
  int Wchtag1Jan = CurrentDoomDay(jahr);
  int Tage       = Tag_des_Jahres(tag,monat,jahr)-1;

  if (Wchtag1Jan > 3)
    Tage = Tage - (7 - Wchtag1Jan);
  else 
    Tage = Tage + Wchtag1Jan;

  if (Tage < 0)
    {
      if ((Wchtag1Jan == 4) || (CurrentDoomDay(jahr-1) == 3))
        Woche = 53;
      else
        Woche = 52;
    }
  else 
    Woche = (Tage/7) + 1;

  if ((Tage > 360) && (Woche > 52)) 
    {
      if (Wchtag1Jan == 3)
        Woche = 53;
      else
        {
          if (CurrentDoomDay(jahr+1) == 4)
            Woche = 53;
          else 
            Woche = 1;
        }
    }

  return Woche;
} 


int MaxTag(int monat, int jahr) 
{
  const int DaysOfMonth[12] = {31, 28, 31, 30, 31, 30, 
                               31, 31, 30, 31, 30, 31};
  
  // Wenn es kein Februar ist, den Wert aus der Tabelle nehmen, ansonsten
  // prüfen, ob es ein Schaltjahr ist.
  // Schaltjahrdefinition: siehe Mail in DE/SCI/ELECTRONICS vom 6.11.:
  // ein Jahr ist ein Schaltjahr, wenn es 
  //   durch 4, aber nicht durch 100
  // oder
  //   durch 400
  // ganzzahlig teilbar ist.
 
 if (monat != 2) return DaysOfMonth[monat-1];
 else 
  {
    if (((jahr % 4 == 0) && (jahr % 100 != 0)) || jahr % 400 == 0) return 29;
    else return 28;
  } 
}


int daysOfYear(int y) 
{
    if (((y % 4 == 0) && (y % 100 != 0)) 
	|| y % 400 == 0) 
	return 366;
    else
	return 365;
}


int getNumberOfDay(int tag, int monat, int jahr) 
{
  int            WTagOffset;

  monat--;
  {
    WTagOffset = tag;
    // Anzahl der Tage seit 1.1.1999 ausrechnen
    const int DaysToMonth[12] = {0, 31, 59, 90, 120, 151, 
				 181, 212, 243, 273, 304, 334};
    
    WTagOffset += ((jahr-1999)*365) + DaysToMonth[monat]; 
    
    // Jetzt muß noch für jedes Schaltjahr seit 1999 ein Tag addiert werden.    
    for (int JahrCount = 1999; JahrCount < jahr; JahrCount++) 
      if (((JahrCount % 4 == 0) && (JahrCount % 100 != 0)) 
	  || JahrCount % 400 == 0) WTagOffset++;
    
    if ((monat > 1) && 
	(((jahr % 4 == 0) && (jahr % 100 != 0)) ||
	 (jahr % 400 == 0))) WTagOffset++;    
  } 
  return(WTagOffset);
}


void getDayByNumber(int nr, int& d, int& m, int& y)
{
  d=1;
  m=1;
  y=1999;
  
  while (nr > daysOfYear(y))
    {
      nr -= daysOfYear(y);
      y++;
    }
  while (nr > MaxTag(m, y))
    {
      nr -= MaxTag(m, y);
      m++;
    }
  d = nr;
}


void ostern(int& nTag, int& nMonat, int nJahr) 
//  Viele weitere Kalenderdaten berechnen sich aus dem Osterdatum. 
//  So ist Aschermittwoch der 46. Tag vor, Christi Himmelfahrt der 39., 
//  Pfingstsonntag der 49. und Fronleichnam der 60. Tag nach dem Ostersonntag.
//  
//  
//  http://www.ptb.de/deutsch/org/4/43/432/oste.htm
//  
//  In den nachfolgenden Gleichungen bezeichnen  INT(a/b) den ganzzahligen Teil
//  des Quotienten zweier Zahlen a und b und MOD(a, b) den
//  nicht-negativen Rest den a beim Teilen durch b lässt.  Zu berechnen ist im
//   einzelnen für die Jahreszahl X : 
  
  
// 1.    K = INT( X / 100 );
// 2.    M = 15 + INT(( 3·K+3 ) / 4 ) - INT(( 8·K+13 ) / 25 );
// 3.    S = 2 - INT(( 3·K+3 ) / 4 );
// 4.    A = MOD( X, 19 );
// 5.    D = MOD( 19·A+M, 30 );
// 6.    R = INT( D / 29 ) + ( INT( D / 28 ) - INT( D / 29 )) · INT( A / 11 );
// 7.    OG = 21 + D - R; - Märzdatum des Ostervollmonds 
//                (= 14. Tag des ersten Monats im Mondkalender, genannt Nisanu)
// 8.    SZ = 7 - MOD( X + INT( X / 4 ) + S, 7 ); - Datum des 1. Sonntags im März
// 9.    OE = 7 - MOD( OG - SZ, 7 );


// Dann ist OS = OG + OE das Datum des Ostersonntags, als Datum im Monat März dargestellt. 
// Der 32. März entspricht dem 1. April usw. 

// Liegt der Ostertermin (Os) erst einmal fest, so berechnen sich daraus weitere besondere 
// Kalenderdaten, und zwar 

// Os - 46: Aschermittwoch,
// Os + 39: Christi Himmelfahrt,
// Os + 49: Pfingstsonntag,
// Os + 60: Fronleichnam  
{
  int K, M, S, A, D, R, OG, SZ, OE;
  
  K = nJahr / 100;
  M = 15 + (((3*K)+3) / 4) - (((8*K)+13) / 25);
  S = 2 - (((3*K)+3) / 4);
  A = nJahr % 19;
  D = ((19*A)+M) % 30;
  R = ( D / 29 ) + (((D / 28) - (D / 29)) * ( A / 11 ));
  OG = 21 + D - R;
  SZ = 7 - ( (nJahr + (nJahr / 4) + S) % 7 );
  OE = 7 - ( (OG - SZ) % 7 );
  
  nTag = OG + OE;
  nMonat = 3;
  
  if (nTag > 31)
    {
      nTag -= 31;
      nMonat = 4;
    }
}


