/**
 * This files' content is taken from the net and modified by JWW.
 * 
 * I found it on stackoverflow while searching for a way to use a
 * std::istream and std::ostream with a FILE (needed for WIN32);
 * gcc's extension __gnu_cxx::stdio_filebuf<char> lead to strange
 * errors on WIN32.
 * 
 * The code was said to come from the french FAQ of fr.comp.lang.c++
 */

#ifndef FILESTREAM_H
# define FILESTREAM_H

# include <cstdio>
# include <assert.h>
# include <iostream>
# include <streambuf>


// streambuf minimal encapsulant un FILE*
//    - utilise les tampons de FILE donc n'a pas de tampon interne en
//      sortie et a un tampon interne de taille 1 en entree car l'interface
//      de streambuf ne permet pas de faire moins;
//    - ne permet pas la mise en place d'un tampon
//    - une version plus complete devrait permettre d'acceder aux
//      informations d'erreur plus precises de FILE* et interfacer aussi
//      les autres possibilites de FILE* (entre autres synchroniser les
//      sungetc/sputbackc avec la possibilite correspondante de FILE*)

class FILEbuf: public std::streambuf
{
public:

  explicit FILEbuf(FILE* cstream) : cstream_(cstream)
  {
    // le constructeur de streambuf equivaut a
    // setp(NULL, NULL);
    // setg(NULL, NULL, NULL);
    assert(cstream != NULL);
  }

  ~FILEbuf()
  {
    if (cstream_)
    {
      std::fclose(cstream_);
//      std::cout << "closed FILE*\n";
    }
  }
  
  bool is_open() const 
  {
    return(cstream_ != 0); 
  }
  
protected:

  std::streambuf* setbuf(char_type* s, std::streamsize n)
  {
    // ne fait rien, ce qui est autorise.  Une version plus complete
    // devrait vraissemblablement utiliser setvbuf
    return NULL;
  }

  int_type overflow(int_type c);
  
  int sync()
  {
    return (fflush(cstream_) == 0 ? 0 : -1);
  }


  int_type underflow();

private:

  FILE*    cstream_;
  char     inputBuffer_[1];
};


class oFILEstream: public std::ostream 
{
public:
  explicit oFILEstream(FILE* fp) : std::ostream(&m_sb), m_sb(fp)
  {
  }
  
  bool is_open() const 
  { 
    return(m_sb.is_open());
  }
private:
  FILEbuf m_sb;  
};

class iFILEstream: public std::istream
{
public:
  explicit iFILEstream(FILE* fp) : std::istream(&m_sb), m_sb(fp)
  {
  }
  
  bool is_open() const 
  { 
    return(m_sb.is_open());
  }
private:
  FILEbuf m_sb;
};


#endif
