// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2007  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "hirestime.h"

#include <stdlib.h>
#include <iostream>
#include <errno.h>

void HiResTime::Capture()
{
#ifndef WIN32
  gettimeofday(&tv, 0);
#endif
}
  
void HiResTime::GetTimeSinceCapture(struct timeval* diff)
{
#ifndef WIN32
  struct timeval tvl;
  
  gettimeofday(&tvl, 0);
  
  GetTimeDiff(diff, &tvl, &tv);
#endif
}

int HiResTime::GetUSecSinceCapture()
{
#ifndef WIN32
  struct timeval tvl;

  gettimeofday(&tvl, 0);
  
  return(GetUSecDiff(&tvl, &tv));
#else
  return(0);
#endif
}

int HiResTime::GetMSecSinceCapture()
{
#ifndef WIN32
  struct timeval tvl;

  gettimeofday(&tvl, 0);
  
  return(GetMSecDiff(&tvl, &tv));
#else
  return(0);
#endif
}

void HiResTime::GetTimeDiff(struct timeval* result, struct timeval* x, struct timeval* y)
{
#ifndef WIN32
  // Siehe auch
  //   The GNU C Library Reference Manual, 21.2 Elapsed Time
  
  /* Perform the carry for the later subtraction by updating y. */
  if (x->tv_usec < y->tv_usec) 
  {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
    y->tv_usec -= 1000000 * nsec;
    y->tv_sec += nsec;
  }
  if (x->tv_usec - y->tv_usec > 1000000) 
  {
    int nsec = (x->tv_usec - y->tv_usec) / 1000000;
    y->tv_usec += 1000000 * nsec;
    y->tv_sec -= nsec;
  }
     
  /* Compute the time remaining to wait.
   tv_usec is certainly positive. */
  result->tv_sec = x->tv_sec - y->tv_sec;
  result->tv_usec = x->tv_usec - y->tv_usec;
#endif
}

int HiResTime::GetUSecDiff(struct timeval* x, struct timeval* y)
{
#ifndef WIN32
  struct timeval result;
  
  GetTimeDiff(&result, x, y);
      
  return(result.tv_sec * 1000000 + result.tv_usec);
#else
  return(0);
#endif
}

int HiResTime::GetMSecDiff(struct timeval* x, struct timeval* y)
{
#ifndef WIN32
  struct timeval result;
  
  GetTimeDiff(&result, x, y);
      
  return(result.tv_sec * 1000 + result.tv_usec / 1000);
#else
  return(0);
#endif
}

void HiResTime::sleep_us(int time_us)
{
#ifndef WIN32
  // man 3 usleep sagt:
  //   "This function is obsolete. Use nanosleep(2) or setitimer(2) instead."
  // usleep(time_us);
  
  struct timespec req, rem;
  int    retval;
  req.tv_sec  = 0;
  req.tv_nsec = (long)time_us * 1000;
  
  do
  {
    retval = nanosleep(&req, &rem);
    if (retval == -1)
    {
      if (errno == EINTR)
      {
        req.tv_nsec = rem.tv_nsec;
      }
      else
      {
        std::cerr << "caution: dirty hack...using exit(-1)\n";
        exit(-1);
      }
    }
  }
  while (retval != 0);
#endif
}

