/*
 * Copyright (C) 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */
#include "imgxform.h"

#include <iostream>
#include <cmath>

namespace ImgXForm
{

   typedef struct {
     int nx0;
     int nxd;
   } TPre;
   
   unsigned char* Bilinear(const unsigned char* in, int nWIn, int nHIn, int nCh,
                           int nWOut, int nHOut)
   {
     // todo: optimization 
     //    - don't jump around in input buffer; use a line buffer, calc all valy0 into it first
     //    - simd (gcc: vector extension) might speed this up, but:
     //        * data has to be aligned to 16bit (hard to do with nCh=3)
     //        * may not pay off as there is little calculation compared
     //          to load/store
     //        * which syntax to use? There is gcc builtin, gcc automatic
     //          and some other (more widespread) mm-syntax. See stackoverflow.
  
     unsigned char* out = new unsigned char[nWOut * nHOut * nCh];
     unsigned char* pOut = out;
          
     float mulx = nWIn/(float)(nWOut);
     float muly = nHIn/(float)(nHOut);
          
     // wenn man nur fx vorberechnet:                 286ms, 268ms
     // wenn man nx0 und fxd vorberechnet:            259ms, 242ms
     // wenn man nx0 und fxd geschickt verschachtelt: 259ms, 242ms
     // Festkomma-Berechnung:                         105ms,  98ms !!!
     // letzter x-Schleifendurchlauf rausgelöst:      101ms,  95ms
     // variables nCh:                                105ms,  98ms -- dafür lohnen sich getrennte Routinen nicht

     TPre* aPre = new TPre[nWOut-1];
     for (int x=0; x<nWOut-1; x++)
     {
       float fx = x*mulx;
       aPre[x].nx0 = fx;
       aPre[x].nxd = (fx - aPre[x].nx0)*256;
     }
     
     for (int y=0; y<nHOut; y++)
     {
       float fy = y*muly;
       int ny0 = fy;
       int nyd = (fy - ny0) * 256;
       
       int ny0off = ny0 * nWIn * nCh;
       int ny1off = ny0off;
       if (y < nHOut-1)
         ny1off += nWIn * nCh;

       for (int x=0; x<nWOut-1; x++)
       {
         int nx0 = aPre[x].nx0;
         int nxd = aPre[x].nxd;
         
         int ii0 = ny0off + nx0 * nCh; // mit Ausnahme des letzten Durchlaufs ist ein fester Offset zwischen ii0 und ii1
         int ii1 = ny1off + nx0 * nCh;         
         for (int c=0; c<nCh; c++)
         {
           int val0 = in[ii0 + c];
           int val1 = in[(ii0 + c) + nCh];
           int valy0 = val0 + ((nxd * (val1-val0)) >> 8);
           
           val0 = in[ii1 + c];
           val1 = in[(ii1 + c) + nCh];
           int valy1 = val0 + ((nxd * (val1-val0)) >> 8);
           
           int val = valy0 + ((nyd * (valy1-valy0)) >> 8);

           *(pOut++) = val;
         }
       }

       {
         int ii0 = ny0off + (nWIn-1) * nCh; // todo: mit Ausnahme des letzten Durchlaufs ist ein fester Offset zwischen ii0 und ii1
         int ii1 = ny1off + (nWIn-1) * nCh;         
         for (int c=0; c<nCh; c++)
         {
           int valy0 = in[ii0 + c];           
           int valy1 = in[ii1 + c];
           
           int val = valy0 + ((nyd * (valy1-valy0)) >> 8);

           *(pOut++) = val;
         }
       }
              
     }
     
     delete[] aPre;
     
     return(out);
   }

   // mostly copied from Fl_Image.cxx of fltk-1.1.10, GPL v2
   unsigned char* Sample(const unsigned char* array, int w, int h, int d,
                         int W, int H)
   {
     // OK, need to resize the image data; allocate memory and
     unsigned char*       new_ptr;       // Pointer into new array
     const unsigned char* old_ptr;       // Pointer into old array
     int                c,              // Channel number
       sy,              // Source coordinate
       dx, dy,          // Destination coordinates
       xerr, yerr,      // X & Y errors
       xmod, ymod,      // X & Y moduli
       xstep, ystep,    // X & Y step increments
       line_d;          // stride from line to line
          
     // Figure out Bresenheim step/modulus values...
     xmod   = w % W;
     xstep  = (w / W) * d;
     ymod   = h % H;
     ystep  = h / H;
     line_d = w * d;
     
     // Allocate memory for the new image...
     unsigned char* new_array = new unsigned char [W * H * d];
     
     // Scale the image using a nearest-neighbor algorithm...
     for (dy = H, sy = 0, yerr = H, new_ptr = new_array; dy > 0; dy --) 
     {
       for (dx = W, xerr = W, old_ptr = array + sy * line_d; dx > 0; dx --) 
       {
         for (c = 0; c < d; c ++)
           *new_ptr++ = old_ptr[c];
         
         old_ptr += xstep;
         xerr    -= xmod;
         
         if (xerr <= 0) 
         {
           xerr    += W;
           old_ptr += d;
         }
       }
       
       sy   += ystep;
       yerr -= ymod;
       if (yerr <= 0) 
       {
         yerr += H;
         sy ++;
       }
     }

     return(new_array);
   }
   
   unsigned char* Shrink(const unsigned char* in, int& nW, int& nH, int nCh,
                         int nDiv)
   {
     // Overflow considerations:
     //   It should be possible to shrink a picture of 6000x4500 pixel
     //   to a 100x100 thumbnail, which means nDiv=60, so 3600 pixel
     //   are reduced to one. 256*3600 easily fits into an int32.
     //   On scaling, the sum is multiplied by nMul (<256); the result
     //   fits into an int32, too.
     //
     // nMul should not become too small.
     //   255 = 2^nShift / nDiv^2
     //   255*nDiv^2 = 2^nShift
     int nShift = floor(log(255*nDiv*nDiv)/log(2.0));
     // nDiv=60 results in nShift=19.

     int nWOut = nW / nDiv;
     int nHOut = nH / nDiv;
            
     unsigned char* out = new unsigned char[nWOut * nHOut * nCh];
     unsigned char* pOut = out;
     int nMul = ((1<<nShift)/nDiv)/nDiv;
     
     int dx0 = nCh;
     int d1  = nCh*nW - dx0*nDiv;
     
     for (int y=0; y<nHOut; y++)
     {
       for (int x=0; x<nWOut; x++)
       {
         int io = (y*nW+x)*nDiv*nCh;
         for (int c=0; c<nCh; c++)
         {
           int sum = 0;
           const unsigned char* pIn = &(in[io + c]);
           for (int py=0; py<nDiv; py++)
           {
             for (int px=0; px<nDiv; px++)
             {
               sum += *pIn;
               pIn += dx0;
             }
             pIn += d1;
           }
           *(pOut++) = (sum * nMul) >> nShift;
         }
       }
     }

     nW = nWOut;
     nH = nHOut;
     return(out);
   }
      
};
