/*
 * Copyright (C) 2003-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "lib_conversions.h"
#include <ctype.h>
#include <math.h>

#define DEBUG 0

#include <stdio.h>

std::string doubleToString(double dVal)
{
  char tmp[30];
  int  nStart = 0;

  sprintf(tmp, "%2.8g", dVal);
  while (tmp[nStart] == ' ')
    nStart++;
  
  return(&(tmp[nStart]));
}

std::string itoStr(long long int nr, char fill, unsigned char digits, bool prettyneg)
{
  std::string  tmp = "";
  char    digit;
  bool    neg = false;
  int     Cnt;
  
  if (nr < 0) 
    {
      neg = true;
      nr *= -1;
    }

  
  if (nr == 0)
    {
      for (int Cnt = 0; Cnt < digits-1; Cnt++) tmp += fill;
      tmp += '0';
    }
  else 
    {
      Cnt = 0;
      while (Cnt < digits || nr > 0 || neg)
        {
          if (nr > 0) 
            digit = (nr % 10)+48;
          else 
	    {
	      if (neg) 
		{
		  if (prettyneg) digit = '-';
		  else 
		    {
		      tmp = '-' + tmp;
		      digit = fill;
		    }
		  neg = false;
		}
	      else digit = fill;
	    }
	    
          tmp = digit + tmp;
          nr /= 10;
	  Cnt++;
        }
    }
  
  return(tmp);
}

std::string itoStr(long long int nr, int nDecPtAt, bool fShorten)
{
  // printf("itoStr %li %i %i wird zu ", nr, nDecPtAt, fShorten);
  
  bool fNeg = false;
  if (nr < 0)
  {
    fNeg = true;
    nr   = -nr;
  }
  
  if (nDecPtAt < 0)
    nDecPtAt = 0;
  
  std::string tmp = itoStr(nr, '0', nDecPtAt+1);
  
  // Fast der gleiche Code ist in liveplotgroup.cpp,
  //   const char* T_LivePlotGroup::timeStrB(char* dst, int wert)
  if (nDecPtAt)
  {
    int nDecPtAtLoc = nDecPtAt;
    // In einem String steht rechtsbündig eine Zahl, in die ein Dezimalpunkt
    // eingefügt werden soll. Vorne ist (falls nötig) mit Nullen aufgefüllt.
    if (fShorten)
    {
      // Die Stellen, die rechts vom Dezimalpunkt Null sind, kann man löschen.
      int s=0;
      while (tmp[tmp.length()-1-s] == '0' && nDecPtAtLoc > 0)
      {
        nDecPtAtLoc--;
        s++;
      }
      tmp.resize(tmp.length()-s);
    }
    
    // Der Dezimalpunkt muss nur eingefügt werden, wenn es auch Nachkommastellen gibt:
    if (nDecPtAtLoc)
    {
      int len = tmp.length();
      int src = len-nDecPtAtLoc;
    
      tmp += " ";
      for (int n=len+1; n>src; n--)
      {
        if (tmp[n-1] == ' ')
          tmp[n] = '0';
        else
          tmp[n] = tmp[n-1];
      }
      tmp[src] = '.';
    }
  }
  
  if (fNeg)
    tmp = "-" + tmp;

  // printf("%s\n", tmp.c_str());
  
  return(tmp);
}

std::string ByteSizeToStr(long long int nr)
{
  std::string   suf  = " Byte";
  double        size = nr;
  unsigned char post = 1;
  
  if (size > 1024)
  {
    size /= 1024;
    suf = " kByte";
  }
  if (size > 1024)
  {
    size /= 1024;
    suf = " MByte";
  }
  
  if (size > 10)
    post = 0;
  
  return(ftoStr(size, 1, post) + suf);
}

long int mypow10(int exponent)
  // ich will int als output, nicht double
{
  int tmp = 1;

  for (int Cnt=0; Cnt<exponent; Cnt++) tmp *= 10;
  
  return(tmp);
}

std::string ftoStr(double        nr, 
                   unsigned char digitspre, 
                   unsigned char digitspost, 
                   bool          prettyneg,
                   bool          fComma)
{
  std::string   tmp;
  int      pre;
  int      post;
  bool     neg = false;

  if (isnan(nr))
    return("NaN");
  else if (isinf(nr))
    return("inf");
          
  if (nr < 0) 
  {
    neg = true;
    nr *= -1;
  }
  
  pre = int(floor(nr));
  // Nachkommastellen werden gerundet
  post = int(floor(   (nr - pre) * mypow10(digitspost)  +0.5 ));
  if (post == mypow10(digitspost))
  {
    post = 0;
    pre++;
  }
  
  tmp = itoStr(pre, ' ', digitspre);
  if (digitspost > 0)
  {
    if (fComma)
      tmp += ",";
    else
      tmp += ".";
    tmp += itoStr(post, '0', digitspost);
  }
  
  if (neg)
  {
    int pos;
    
    pos = tmp.length()-1;
    while ((tmp[pos] != ' ') && (pos > 0)) 
      pos--;
    
    if (pos == 0 && tmp[pos] != ' ') 
      tmp = ' ' + tmp; // nur für den Notfall
    
    if (prettyneg) 
      tmp[pos] = '-';
    else 
      tmp = tmp.substr(0, pos+1) + '-' + tmp.substr(pos+1, tmp.length()-pos);      
  }
  
  return(tmp);
}

int hex2int(char cHex)
{
  int wert = cHex;
  if (wert >=97) wert-= 87;
  if (wert >=65) wert-= 55;      
  if (wert >=48) wert-= 48;
  return(wert);
}

int hex2int(std::string hexIn)
{
  int tmp = 0;
  int wert1 = 1;
  int wert2;
  
  for (unsigned int n= 0; n< hexIn.length(); n++)
    {
      wert2 = hexIn[hexIn.length()-n-1];
      if (wert2 >=97 ) wert2-= 87;
      if (wert2 >=65 ) wert2-= 55;      
      if (wert2 >= 48) wert2-= 48;            
      tmp+= wert2 * wert1;
      wert1*= 16;
    }
  return(tmp);
}

std::string strU(std::string in) 
{
  std::string tmp = "";
  
  for (std::string::size_type uCnt = 0; uCnt < in.length(); uCnt++)
    {
      tmp += toupper(in[uCnt]); 
    } 
  return(tmp); 
}

std::string strL(std::string in) 
{
  std::string tmp = "";
  
  for (std::string::size_type uCnt = 0; uCnt < in.length(); uCnt++)
    {
      tmp += tolower(in[uCnt]); 
    } 
  return(tmp); 
}

std::string trim(std::string in)
{
  int nL  = in.length();
  int nCnt;
  
  // Die Anzahl der führenden Zeichen ermitteln
  for (nCnt=0; nCnt < nL && isspace(in[nCnt]); nCnt++)
    {
    }
    
  // Die führenden Zeichen löschen
  in.replace(0, nCnt, "");
  
  nL = in.length();
  if (nL > 0)
    {
      // Die Anzahl der abschliessenden Zeichen ermitteln
      for (nCnt = nL-1; nCnt > 0 && isspace(in[nCnt]); nCnt--)
        {
        }
      // Die abschliessenden Zeichen löschen
      in.replace(nCnt+1, nL-nCnt, "");     
    }
  
  return(in);
}

std::string collapse_space(std::string in)
{
  int state = 0;
  std::string::size_type nL  = in.length();
  std::string::size_type pos = 0;
    
  for (std::string::size_type nCnt=0; nCnt < nL; nCnt++)
  {
    switch (state)
    {
      case 0:
        if (isspace(in[nCnt]))
        {
          pos = nCnt;
          state = 1;
        }
        break;
        
      case 1:
        if (isspace(in[nCnt]) == 0)
        {
          if (nCnt-pos > 1)
          {
            in.replace(pos, nCnt-pos, " ");
            nCnt -= nCnt-pos;
          }
          state = 0;
        }
        break;        
    }
  }
  if (state == 1 && nL-pos > 1)
  {
    in.replace(pos, nL-pos, " ");
  }
      
  return(in);
}

std::string collapse_char(std::string in, char ch)
{
  int state = 0;
  std::string::size_type nL  = in.length();
  std::string::size_type pos = 0;
  std::string repl = "";
  repl.push_back(ch);
    
  for (std::string::size_type nCnt=0; nCnt < nL; nCnt++)
  {
    switch (state)
    {
      case 0:
        if (in[nCnt] == ch)
        {
          pos = nCnt;
          state = 1;
        }
        break;
        
      case 1:
        if (in[nCnt] != ch)
        {
          if (nCnt-pos > 1)
          {
            in.replace(pos, nCnt-pos, repl);
          }
          state = 0;
        }
        break;        
    }
  }
  if (state == 1 && nL-pos > 1)    
    in.replace(pos, nL-pos, repl);
      
  return(in);
}

std::string itoHexStr(long long unsigned int nr, int len)
{
  std::string tmp = "";
  char        ch;

  while (nr > 0 || len > 0)
  {
    ch = 48 + (nr & 0x0F);
    if (ch > 57) 
      ch += 7;
    tmp = ch + tmp;
    
    nr >>= 4;
    len -= 4;
  }

  return(tmp);  
}

void float2MulShift(float fl, int& mul, int& shift)
{
  int          nMulBits = 15;
  long long    lMaxNum  = (1l << nMulBits)-1;
  long long    lDiv;
  
  // Dieses sollte auf jeden Fall möglich sein:
  mul   = (long long)(fl);
  shift = 0;
       
#if DEBUG == 1
  printf("lMaxNum = %lli, fl=%f\n", lMaxNum, fl);
#endif
  
  // Nun was gescheites berechnen:
  lDiv = (long long)(lMaxNum / fl);
              
#if DEBUG == 1
  printf("lDiv = %lli\n", lDiv);
#endif
  
  while ((1LL<<shift) < lDiv)
  {
    shift++;
  }
  shift--;
  
#if DEBUG == 1
  printf("shift = %i\n", shift);
#endif
  
  mul = (int)floor(  (fl * (1LL<<shift)) +0.5 );
       
#if DEBUG == 1
  printf("mul = %i\n", mul);
#endif
  
  // Solange mul eine gerade Zahl ist kann man sie und den Schiebefaktor
  // halbieren ohne das Ergebnis ungenauer zu machen.
  while ((mul & 1) == 0 && shift > 0)
  {
    mul >>= 1;
    shift--;
  }
}

std::string strWithMinLen(std::string in, std::string::size_type nMin)
{
  if (in.length() >= nMin)
    return(in);
  else
  {
    return(in + std::string(nMin - in.length(), ' ')       );
  }
}

void test_itoStr(std::string ref,
                 int num, int nDecPt, bool fShort = true)
{
  std::string tmp = itoStr(num, nDecPt, fShort);
  if (ref.compare(tmp) != 0)
  {
    printf("Fehler: %i %i %i *%s* != *%s*\n", num, nDecPt, fShort, tmp.c_str(), ref.c_str());
  }
}
                 
void test_collapse_space(std::string in, std::string ref)
{
  if (collapse_space(in).compare(ref) != 0)
    printf("collapse_space fehler: *%s* -> *%s*\n", in.c_str(), collapse_space(in).c_str());
}

void lib_conversions_tests()
{
  printf("Keine Nachkommastellen\n");
  test_itoStr(  "0",   0, 0, false);
  test_itoStr(  "5",   5, 0, false);
  test_itoStr( "10",  10, 0, false);
  test_itoStr( "15",  15, 0, false);
  test_itoStr("100", 100, 0, false);
  test_itoStr("150", 150, 0, false);
  
  printf("Eine Nachkommastelle\n");
  test_itoStr( "0.0",   0, 1, false);
  test_itoStr( "0.5",   5, 1, false);
  test_itoStr( "1.0",  10, 1, false);
  test_itoStr( "1.5",  15, 1, false);
  test_itoStr("10.0", 100, 1, false);
  test_itoStr("15.0", 150, 1, false);

  printf("Zwei Nachkommastellen\n");
  test_itoStr("0.00",   0, 2, false);
  test_itoStr("0.05",   5, 2, false);
  test_itoStr("0.10",  10, 2, false);
  test_itoStr("0.15",  15, 2, false);
  test_itoStr("1.00", 100, 2, false);
  test_itoStr("1.50", 150, 2, false);

  printf("Drei Nachkommastellen\n");
  test_itoStr("0.000",   0, 3, false);
  test_itoStr("0.005",   5, 3, false);
  test_itoStr("0.010",  10, 3, false);
  test_itoStr("0.015",  15, 3, false);
  test_itoStr("0.100", 100, 3, false);
  test_itoStr("0.150", 150, 3, false);
  
  printf("Keine Nachkommastellen\n");
  test_itoStr(  "0",   0, 0, true);
  test_itoStr(  "5",   5, 0, true);
  test_itoStr( "10",  10, 0, true);
  test_itoStr( "15",  15, 0, true);
  test_itoStr("100", 100, 0, true);
  test_itoStr("150", 150, 0, true);
  
  printf("Eine Nachkommastelle\n");
  test_itoStr( "0",     0, 1, true);
  test_itoStr( "0.5",   5, 1, true);
  test_itoStr( "1",    10, 1, true);
  test_itoStr( "1.5",  15, 1, true);
  test_itoStr("10",   100, 1, true);
  test_itoStr("15",   150, 1, true);

  printf("Zwei Nachkommastellen\n");
  test_itoStr("0",      0, 2, true);
  test_itoStr("0.05",   5, 2, true);
  test_itoStr("0.1",   10, 2, true);
  test_itoStr("0.15",  15, 2, true);
  test_itoStr("1",    100, 2, true);
  test_itoStr("1.5",  150, 2, true);

  printf("Drei Nachkommastellen\n");
  test_itoStr("0",       0, 3, true);
  test_itoStr("0.005",   5, 3, true);
  test_itoStr("0.01",   10, 3, true);
  test_itoStr("0.015",  15, 3, true);
  test_itoStr("0.1",   100, 3, true);
  test_itoStr("0.15",  150, 3, true);
  
  
  printf("Weitere\n");
  test_itoStr("-0.004", -4, 3);
  test_itoStr("738.65", 73865, 2, true);
  test_itoStr("72.88", 7288,  2, true);
  
  test_collapse_space("basdf",   "basdf");
  test_collapse_space(" basdf",  " basdf");
  test_collapse_space("basdf ",  "basdf ");
  test_collapse_space(" basdf ", " basdf ");
  test_collapse_space("  basdf",  " basdf");
  test_collapse_space("basdf  ",  "basdf ");
  test_collapse_space("  basdf  ", " basdf ");
  test_collapse_space("ba sdf",   "ba sdf");
  test_collapse_space("ba  sdf",   "ba sdf");
}
