// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2003-2006, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "scandir.h"

#ifdef SCAN_LINUX
#endif

#include <string.h>
#include <stdlib.h> // für free
#include "filetools.h"

ScanDir::ScanDir(const char *d)
{
  nNrEntries = -1;
  
#ifdef SCAN_FLTK
  nNrEntries = fl_filename_list(d, &list);
#endif
#ifdef SCAN_LINUX
  nNrEntries = scandir(d, &list, NULL, alphasort);
#endif
}

ScanDir::~ScanDir()
{
  if (nNrEntries >= 0)
  {
    for (int i=nNrEntries; i>0; )
    {
      free((void*)(list[--i]));
    }
    free((void*)list);
  }
}


std::string ScanDir::findBasenameInDir(std::string basename, std::string dir)
{
  std::vector<std::string> res;
  
  findBasenameInDir(basename, dir, res);
  if (res.size()>0)
    return(res[0]);
  else
    return("");
}

void        ScanDir::findBasenameInDir(std::string basename, std::string dir, 
                                       std::vector<std::string>& res)
{
  ScanDir*    sd = new ScanDir(dir.c_str());
  std::string tmp;
  
  if (dir[dir.length()-1] != '/')
    dir += "/";
  
  for (int cnt=0; cnt<sd->nNrEntries; cnt++)
  {
    if (strcmp(sd->list[cnt]->d_name, ".") != 0 && strcmp(sd->list[cnt]->d_name, "..") != 0)
    {
      tmp = dir + sd->list[cnt]->d_name;
      
      if (FileTools::isDir(tmp))
      {
        findBasenameInDir(basename, tmp + "/", res);
      }
      else
      {
        tmp = sd->list[cnt]->d_name;
        if (tmp.find(basename) == 0 && tmp.length() > basename.length()+1 && tmp[basename.length()] == '.')
          res.push_back(dir + tmp);
      }
    }
  }
}

void ScanDir::findInDir(std::string               dir,
                        bool                      fRecursive,
                        std::vector<std::string>& res)
{
  ScanDir*    sd = new ScanDir(dir.c_str());
  std::string tmp;
  
  if (dir[dir.length()-1] != '/')
    dir += "/";
  
  for (int cnt=0; cnt<sd->nNrEntries; cnt++)
  {
    if (strcmp(sd->list[cnt]->d_name, ".") != 0 && strcmp(sd->list[cnt]->d_name, "..") != 0)
    {
      tmp = dir + sd->list[cnt]->d_name;
      
      if (FileTools::isDir(tmp))
      {
        if (fRecursive)
          findInDir(tmp + "/", fRecursive, res);
      }
      else
      {
        tmp = sd->list[cnt]->d_name;
        res.push_back(dir + tmp);
      }
    }
  }
}

