/*
 * Copyright (C) 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "utf8.h"

#include <ctype.h>
#include <FL/fl_utf8.h> // todo: base shouldn't depend on fltk

namespace UTF8
{
   bool isspace(int nChar)
   {
     return( (nChar & 0x80) == 0 && ::isspace(nChar) != 0 );
   }
   
   std::string trim(std::string in)
   {
     int nL  = in.length();
     int nCnt;
     
     // Die Anzahl der führenden Zeichen ermitteln
     for (nCnt=0; nCnt < nL && UTF8::isspace(in[nCnt]); nCnt++)
     {
     }
     
     // Die führenden Zeichen löschen
     in.replace(0, nCnt, "");
     
     nL = in.length();
     if (nL > 0)
     {
       // Die Anzahl der abschliessenden Zeichen ermitteln
       for (nCnt = nL-1; nCnt > 0 && UTF8::isspace(in[nCnt]); nCnt--)
       {
       }
       // Die abschliessenden Zeichen löschen
       in.replace(nCnt+1, nL-nCnt, "");     
     }
     
     return(in);
   }
   
   std::string collapse_space(std::string in)
   {
     int state = 0;
     std::string::size_type nL  = in.length();
     std::string::size_type pos = 0;
     
     for (std::string::size_type nCnt=0; nCnt < nL; nCnt++)
     {
       switch (state)
       {
         case 0:
           if (UTF8::isspace(in[nCnt]))
           {
             pos = nCnt;
             state = 1;
           }
           break;
           
         case 1:
           if (!UTF8::isspace(in[nCnt]))
           {
             if (nCnt-pos > 1)
             {
               in.replace(pos, nCnt-pos, " ");
             }
             state = 0;
           }
           break;        
       }
     }
     if (state == 1 && nL-pos > 1)    
       in.replace(pos, nL-pos, " ");
     
     return(in);
   }
   
   std::string to_lower(std::string in)
   {
     int len = in.length();
     std::string strOut = "";
     if (len)
     {
       char* pOut = new char[4*len];     
       int ret = fl_utf_tolower((const unsigned char*)(in.c_str()), len, pOut);
       strOut.assign(pOut, ret);
       delete[] pOut;
     }
     
     return(strOut);
   }   
};
